<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:camed="http://jcam.org.uk/editor" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:as="http://www.oasis-open.org/committees/cam">
	<xsl:param name="stylesheet">styles.css</xsl:param>
<xsl:output method="html" indent="yes"/>
<xsl:template match="/">
	<html>
			<head>
				<xsl:element name="link">
					<xsl:attribute name="rel">stylesheet</xsl:attribute>
					<xsl:attribute name="type">text/css</xsl:attribute>
					<xsl:attribute name="href"><xsl:value-of select="$stylesheet"/></xsl:attribute>
				</xsl:element>
			</head>
	<body>
	<xsl:apply-templates select="//@as:*[starts-with(name(),'as:Parameter')]" mode="param"/>
	<table border="1" width="800pt">
	<th>Name</th>
	<th>Rules</th>
	<xsl:apply-templates select="*" mode="xml" />
	</table>
	</body>
	</html>
</xsl:template>




<xsl:template match="*" mode="xml">
	<xsl:variable name="xpathName"><xsl:call-template name="xpathName"/></xsl:variable>
	<tr>
	<td class="path"><xsl:value-of select="$xpathName"/></td>
	<xsl:variable name="name" select="name()"/>
	
	<td class="rule">
	<table>
	<tr>
	<td class="rule">
	<xsl:variable name="rules">
	<xsl:if test="count(*) = 0 and not(starts-with(text(),'%'))">setValue=<xsl:value-of select="."/><br/></xsl:if>
	<xsl:apply-templates select="./@as:*[not(contains(name(),'-'))]">
			<xsl:with-param name="name"><xsl:value-of select="$name"/></xsl:with-param>
	</xsl:apply-templates>
	</xsl:variable>
	<xsl:choose>
	<xsl:when test="string-length(normalize-space($rules)) > 1">
		<xsl:copy-of select="$rules"/>
	</xsl:when>
	<xsl:otherwise>makeMandatory=true
	</xsl:otherwise>
	</xsl:choose>
	</td>
	</tr>
	<tr>
	<td>
		<xsl:call-template name="annotation"/>
	</td>
	</tr>
	</table>
	</td>

	</tr>



	<xsl:apply-templates select="@*[not(starts-with(name(),'as:'))]" mode="xml"/>
	<xsl:apply-templates select="*" mode="xml"/>
	
</xsl:template>

<xsl:template match="@*[not(starts-with(name(),'as:'))]" mode="xml">
	<xsl:variable name="xpathName"><xsl:call-template name="xpathAttrName"/></xsl:variable>
	<tr>
	<td class="path"><xsl:value-of select="$xpathName"/></td>
	<xsl:variable name="name" select="local-name()"/>

	<td  class="rule">
	<table>
	<tr>
	<td class="rule">
	<xsl:variable name="rules"><xsl:if test="not(starts-with(.,'%'))">setValue=<xsl:value-of select="."/><br/></xsl:if>
	<xsl:apply-templates select="../@as:*[contains(name(),'-')]">
			<xsl:with-param name="name"><xsl:value-of select="$name"/></xsl:with-param>
	</xsl:apply-templates></xsl:variable>
	<xsl:choose>
	<xsl:when test="string-length(normalize-space($rules)) > 1">
		<xsl:value-of select="$rules"/>
	</xsl:when>
	<xsl:otherwise>makeMandatory=true
	</xsl:otherwise>
	</xsl:choose>
	</td>
	</tr>
	<tr>
	<td>
		<xsl:call-template name="annotationAttr"/>
	</td>
	</tr>
	</table>
	</td>

	</tr>

</xsl:template>



<xsl:template match="@as:*[not(contains(name(),'-'))]">
	<xsl:value-of select="local-name()"/>=<xsl:value-of select="."/><br/>
</xsl:template>

<xsl:template match="@as:*[contains(name(),'-')]">
<xsl:param name="name">noname</xsl:param>
<xsl:choose>
<xsl:when test="contains($name,'noname')">
	<xsl:value-of select="substring-before(local-name(),'-')"/>=<xsl:value-of select="."/><br/>
</xsl:when>
<xsl:otherwise>
	<xsl:if test="contains(local-name(),$name)">
	<xsl:value-of select="substring-before(local-name(),'-')"/>=<xsl:value-of select="."/><br/>
	</xsl:if>
</xsl:otherwise>
</xsl:choose>
</xsl:template>


<xsl:template match="text()">
	<td><xsl:value-of select="."/></td>
</xsl:template>

<xsl:template match="@as:*[starts-with(name(),'as:Parameter')]" mode="param">
	<h1><xsl:value-of select="substring-after(local-name(),'Parameter-')"/>=<xsl:value-of select="."/></h1>
</xsl:template>

<xsl:template match="camed:*" mode="xml"/>

<xsl:template name="annotation">
<xsl:variable name="xpath"><xsl:call-template name="xpath"></xsl:call-template></xsl:variable>
<xsl:apply-templates select=".//camed:annotation[@item=$xpath]"/>

</xsl:template>

<xsl:template name="annotationAttr">
<xsl:variable name="xpath"><xsl:call-template name="xpathAttr"></xsl:call-template></xsl:variable>
<xsl:apply-templates select="..//camed:annotation[@item=$xpath]"/>

</xsl:template>

<xsl:template match="camed:annotation">
<!--	<tr>
	<th colspan="3">
	Annotation for <xsl:value-of select="@item"/>
	</th>
	</tr>-->
	<table border="1">
	<tr>
	<xsl:apply-templates select="camed:documentation"/>
	</tr>
	</table>
</xsl:template>
<xsl:template match="camed:documentation">
	<td valign="top">
	<table>
	<tr>
	<th style="font-size:8pt"><xsl:value-of select="@type"/></th>
	</tr>
	<tr>
	<td style="font-size:8pt"><xsl:value-of select="."/></td>
	</tr>
	</table>
	</td>
</xsl:template>

<xsl:template name="xpath">
<xsl:variable name="parent"><xsl:value-of select="name(..)"/></xsl:variable>
<xsl:choose>
<xsl:when test="contains($parent,'#document')">//<xsl:value-of select="name()"/></xsl:when>
<xsl:when test="string-length($parent) > 0">//<xsl:value-of select="$parent"/>/<xsl:value-of select="name()"/></xsl:when>
<xsl:otherwise>//<xsl:value-of select="name()"/></xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="xpathAttr">
<xsl:variable name="parent"><xsl:value-of select="name(..)"/></xsl:variable>
<xsl:choose>
<xsl:when test="contains($parent,'#document')">//<xsl:value-of select="name()"/></xsl:when>
<xsl:when test="string-length($parent) > 0">//<xsl:value-of select="$parent"/>/@<xsl:value-of select="name()"/></xsl:when>
<xsl:otherwise>//@<xsl:value-of select="name()"/></xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="xpathName">
<xsl:variable name="parent"><xsl:value-of select="name(..)"/></xsl:variable>
<xsl:choose>
<xsl:when test="contains($parent,'#document')">/<xsl:value-of select="name()"/></xsl:when>
<xsl:when test="string-length($parent) > 0">/<xsl:value-of select="$parent"/>/<xsl:value-of select="name()"/></xsl:when>
<xsl:otherwise>/<xsl:value-of select="name()"/></xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="xpathAttrName">
<xsl:variable name="parent"><xsl:value-of select="name(..)"/></xsl:variable>
<xsl:choose>
<xsl:when test="string-length($parent) > 0">/<xsl:value-of select="$parent"/>/@<xsl:value-of select="name()"/></xsl:when>
<xsl:otherwise>/@<xsl:value-of select="name()"/></xsl:otherwise>
</xsl:choose>
</xsl:template>

</xsl:stylesheet><!-- Stylus Studio meta-information - (c)1998-2002 eXcelon Corp.
<metaInformation>
<scenarios/><MapperInfo srcSchemaPath="" srcSchemaRoot="" srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/>
</metaInformation>
-->