<?xml version="1.0" encoding="UTF-8"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:w="http://jcam.org.uk/editor/picklist/1.0" 
    exclude-result-prefixes="xs xsd" version="2.0">

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/ 

Author: David RR Webber, April, 2008 -->

    <!-- 

Purpose : Take CAM template instance and applies CAM picklist to the rules section.
Original : 04/22/2008 
Initials : drrw 
CAM ver : 1.1 
Description : Part of a set of tools to ease alignment of XSD based validations and documentation 
and CAM templates. 
Updated - Inits - Change 
04/22/08 drrw Initial release 
08/31/09 drrw change w namespace URI
    -->
    <xsl:param name="wantlistURL">EDXL-wantlist-demo.xml</xsl:param>
    <xsl:param name="showXcl" as="xsd:boolean">true</xsl:param>
    
    <xsl:output indent="yes"/>
    
    <xsl:variable name="version">1.02</xsl:variable>
    <xsl:variable name="wanted" select="document($wantlistURL)"/>

    <xsl:template match="/">

        <xsl:message>Processing starting.   Version: <xsl:value-of select="$version"/></xsl:message>
        <xsl:message><xsl:value-of select="concat('PickList rules=',count($wanted/w:WantList/descendant::*))"></xsl:value-of></xsl:message>
        
        <xsl:variable name="ns-prefix">
            <xsl:value-of select="./@targetNamespace"/>
        </xsl:variable>              
            
            <xsl:for-each select="*">
                <!-- parses the entire XML input -->
                <xsl:copy>
                    <xsl:copy-of select=".//namespace::*"/>
                    <xsl:copy-of select="./@*"/>
                    <xsl:apply-templates mode="map" select="*"/>                  
                </xsl:copy>
            </xsl:for-each>

            <xsl:message>Finished</xsl:message>

    </xsl:template>
    
    <xsl:template match="node()/child::*" mode="map">
        <xsl:variable name="thisGrp" select="concat(name(../.),'\',name(.))"/>
        <xsl:variable name="XcludeGrp">
            <xsl:choose>
                <xsl:when test="$wanted/w:WantList//Group/@w:n=$thisGrp">
                    <xsl:variable name="chkRoot" select="$wanted/w:WantList//Group[@w:n=$thisGrp]"/>
                    <xsl:variable name="thisRoot" select="$chkRoot/../../@w:n[1]"/>
                    <xsl:variable name="action" select="$wanted/w:WantList//Group[@w:n=$thisGrp]/@w:isXclude"/>
                    <xsl:value-of select="$action[1]"/>
                </xsl:when>
                <xsl:otherwise>0</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="not($XcludeGrp='0') and $showXcl"><xsl:comment>Excluded Group: <xsl:value-of select="name(.)"/></xsl:comment></xsl:if>
        <xsl:if test="$XcludeGrp='0'">
            <xsl:copy>
               
            <xsl:variable name="thisElement" select="node()"></xsl:variable>
            <!-- xsl:element name="{local-name(.)}" -->
                <!-- xsl:comment><xsl:value-of select="$action[1]"/> - <xsl:value-of  select="name(../.)"/> - [<xsl:value-of select="$thisRoot[1]"></xsl:value-of>]</xsl:comment></xsl:if -->
                <!-- xsl:apply-templates select="./@*" mode="map"/-->
                
            <xsl:for-each select="./@*">
                <xsl:variable name="this" select="concat(name(../../.),'\',name(../.),'\@',name(.))"/>
                <xsl:variable name="Xclude">
                    <xsl:choose>
                        <xsl:when test="$wanted/w:WantList//Attribute/@w:n=$this">
                            <xsl:variable name="chkRoot" select="$wanted/w:WantList//Attribute[@w:n=$this]"/>
                            <xsl:variable name="thisRoot" select="$chkRoot/../../@w:n[1]"/>
                            <xsl:variable name="action" select="$wanted/w:WantList//Attribute[@w:n=$this]/@w:isXclude"/>
                            <xsl:value-of select="$action[1]"/>
                        </xsl:when>
                        <xsl:otherwise>0</xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:if test="$Xclude='0'">
                    <xsl:copy-of select="."/>
                </xsl:if>
            </xsl:for-each>
            <xsl:copy-of select="$thisElement[1]"/>
                
            <xsl:for-each select="*">
                <xsl:choose>
                    <xsl:when test="count(./descendant::node()) > 1">
                         <xsl:apply-templates mode="map" select="."/>
                         <xsl:apply-templates mode="map" select="./@*"/>
                    </xsl:when>
                    <xsl:when test="name()='as:constraint'">
                        <xsl:variable name="ruleType" select="substring-before(@action,'(')"/>
                        <xsl:variable name="XPath" select="if (contains(@action,',')) then substring-before(substring-after(@action,'('),',') else substring-before(substring-after(@action,'('),')')"/>
                        <xsl:variable name="myXPath">
                            <xsl:choose>
                                <xsl:when test=" starts-with($XPath,'//')"><xsl:value-of select="substring-after($XPath,'//')"/></xsl:when>
                                <xsl:when test=" starts-with($XPath,'/')"><xsl:value-of select="substring-after($XPath,'/')"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="$XPath"/></xsl:otherwise>
                            </xsl:choose>                     
                        </xsl:variable>
                        <xsl:variable name="ruleXPath" select="replace($myXPath,'/','\\')"/>
                        <xsl:variable name="Xclude" as="xsd:boolean">                 
                        <xsl:choose>                            
                            <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Attribute[$ruleXPath=@w:n]/@w:isXclude='1')">true</xsl:when>
                            <xsl:when test="contains($ruleXPath,'@') and(contains(substring-before($ruleXPath,'\@'),'\')) and($wanted/w:WantList//Element[ends-with(@w:n,substring-before($ruleXPath,'\@'))]/@w:isXclude='1')">true</xsl:when>
                            <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Group[substring-before($ruleXPath,'\@')=@w:n]/@w:isXclude='1')">true</xsl:when>
                            <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Attribute[contains($ruleXPath,substring-after(@w:n,'\'))]/@w:isXclude='1')">
                                <xsl:variable name="this" select="@action"/>
                                <xsl:variable name="chkTotal" select="//as:constraint/@action=$this"/>       
                                <xsl:choose>
                                    <xsl:when test="count($chkTotal)=1">true<!-- xsl:message>Match Attr0: <xsl:value-of select="$this"></xsl:value-of></xsl:message--></xsl:when>
                                    <xsl:otherwise>false</xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Attribute[contains(@w:n,$ruleXPath)]/@w:isXclude='0')">
                                <xsl:variable name="chkElm" select="$wanted/w:WantList//Attribute[contains(@w:n,$ruleXPath)]/parent::node()"/>
                                <xsl:variable name="chkGrp" select="$chkElm[1]/parent::node()"/>                              
                                <xsl:choose>
                                    <xsl:when test="$chkElm[1]/@w:isXclude='1' and(count($chkElm)=1)">true<!-- xsl:message>Match Elm1: <xsl:value-of select="$chkElm[1]/@w:n"></xsl:value-of></xsl:message--></xsl:when>
                                    <xsl:when test="$chkGrp[1]/@w:isXclude='1'">true<!-- xsl:message>Match Grp1: <xsl:value-of select="$chkGrp[1]/@w:n"></xsl:value-of></xsl:message--></xsl:when>
                                    <xsl:otherwise>false</xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Attribute[$ruleXPath=@w:n]/@w:isXclude='0')">
                                <xsl:variable name="chkElm" select="$wanted/w:WantList//Attribute[$ruleXPath=@w:n]/parent::node()[1]"/>
                                <xsl:variable name="chkGrp" select="$chkElm[1]/parent::node()"/>                              
                                <xsl:choose>
                                    <xsl:when test="$chkElm[1]/@w:isXclude='1'">true<!-- xsl:message>Match Elm2: <xsl:value-of select="$chkElm[1]/@w:n"></xsl:value-of></xsl:message--></xsl:when>
                                    <xsl:when test="$chkGrp[1]/@w:isXclude='1'">true<!-- xsl:message>Match Grp2: <xsl:value-of select="$chkGrp[1]/@w:n"></xsl:value-of></xsl:message--></xsl:when>
                                    <xsl:otherwise>false</xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:when test="$wanted/w:WantList//Group[@w:n=$ruleXPath]/@w:isXclude='1'">true</xsl:when>
                            <xsl:when test="$wanted/w:WantList//Element[ends-with(@w:n,$ruleXPath)]/@w:isXclude='1'">true</xsl:when>
                            <xsl:when test="$wanted/w:WantList//Element[ends-with(@w:n,$ruleXPath)]/@w:isXclude='0'">
                                <xsl:variable name="chkGrp" select="$wanted/w:WantList//Element[ends-with(@w:n,$ruleXPath)]/parent::node()"/>
                                <!-- xsl:message>chkGrp <xsl:value-of select="$chkGrp[1]/@w:isXclude"/> - <xsl:value-of select="$ruleXPath"/> -  <xsl:value-of select="$chkGrp[1]/@w:n"/></xsl:message -->
                                <xsl:choose>
                                    <xsl:when test="$chkGrp[1]/@w:isXclude='1'">true</xsl:when>
                                    <xsl:otherwise>false</xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:otherwise>false</xsl:otherwise>
                        </xsl:choose>
                        </xsl:variable>
                        <xsl:variable name="matchedRule">
                            <xsl:choose>
                                <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Attribute[contains($ruleXPath,substring-after(@w:n,'\'))]/@w:isXclude='1')">ATTRIB:-<xsl:value-of select="$wanted/w:WantList//Attribute[contains($ruleXPath,substring-after(@w:n,'\'))]/@w:n[1]"/></xsl:when>
                                <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Attribute[$ruleXPath=@w:n]/@w:isXclude='1')">ATTRIB:-<xsl:value-of select="$wanted/w:WantList//Attribute[$ruleXPath=@w:n]/@w:n[1]"/></xsl:when>
                                <xsl:when test="contains($ruleXPath,'@') and(contains(substring-before($ruleXPath,'\@'),'\')) and($wanted/w:WantList//Element[ends-with(@w:n,substring-before($ruleXPath,'\@'))]/@w:isXclude='1')">ATTR/ELEMENT:-<xsl:value-of select="$wanted/w:WantList//Element[ends-with(@w:n,substring-before($ruleXPath,'\@'))]/@w:n[1]"/></xsl:when>
                                <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Group[substring-before($ruleXPath,'\@')=@w:n]/@w:isXclude='1')">GROUP/ATTRIB:-<xsl:value-of select="$wanted/w:WantList//Group[substring-before($ruleXPath,'\@')=@w:n]/@w:n[1]"/></xsl:when>
                                <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Attribute[$ruleXPath=@w:n]/@w:isXclude='0')">GROUP/ELM/ATTRIB:-<xsl:value-of select="$wanted/w:WantList//Attribute[$ruleXPath=@w:n]/parent::node()/@w:n[1]"/></xsl:when>
                                <xsl:when test="contains($ruleXPath,'@') and($wanted/w:WantList//Attribute[contains($ruleXPath,substring-after(@w:n,'\'))]/@w:isXclude='0')">ELEMENT/ATTRIB:-<xsl:value-of select="$wanted/w:WantList//Attribute[contains($ruleXPath,substring-after(@w:n,'\'))]/parent::node()/@w:n[1]"/></xsl:when>
                                <xsl:when test="$wanted/w:WantList//Group[@w:n=$ruleXPath]/@w:isXclude='1'">GROUP:-<xsl:value-of select="$wanted/w:WantList//Group[@w:n=$ruleXPath]/@w:n[1]"/></xsl:when>
                                <xsl:when test="$wanted/w:WantList//Element[ends-with(@w:n,$ruleXPath)]/@w:isXclude='1'">ELEMENT:-<xsl:value-of select="$wanted/w:WantList//Element[ends-with(@w:n,$ruleXPath)]/@w:n[1]"/></xsl:when>
                                <xsl:when test="$wanted/w:WantList//Element[ends-with(@w:n,$ruleXPath)]/@w:isXclude='0'">GROUP/ELEMENT:-<xsl:value-of select="$wanted/w:WantList//Element[ends-with(@w:n,$ruleXPath)]/parent::node()/@w:n"/></xsl:when>
                                
                                <xsl:otherwise>false</xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:variable name="matchMsg" select="if (contains($matchedRule[1],' ')) then substring-before($matchedRule[1],' ') else $matchedRule[1]"/>
                        <xsl:if test="not($Xclude)"><xsl:copy-of select="."/></xsl:if>
                        <xsl:if test="$Xclude and($showXcl)">
                            <!-- xsl:message><xsl:value-of select="concat($ruleType,' ',$ruleXPath,' - for ',$matchMsg)"/></xsl:message -->
                            <xsl:comment xml:space="preserve"><xsl:value-of select="@action"/>
                            </xsl:comment></xsl:if>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:variable name="this" select="concat(name(../../.),'\',name(../.),'\',name(.))"/>
                        <xsl:variable name="Xclude">
                        <xsl:choose>
                            <xsl:when test="$wanted/w:WantList//Element/@w:n=$this">
                                <xsl:variable name="chkRoot" select="$wanted/w:WantList//Element[@w:n=$this]"/>
                                <xsl:variable name="thisRoot" select="$chkRoot/../../@w:n[1]"/>
                                <xsl:variable name="action" select="$wanted/w:WantList//Element[@w:n=$this]/@w:isXclude"/>
                                <xsl:value-of select="$action[1]"/>
                            </xsl:when>
                            <xsl:otherwise>0</xsl:otherwise>
                        </xsl:choose>
                        </xsl:variable>
                        <xsl:if test="not($Xclude='0') and $showXcl"><xsl:comment>Excluded: <xsl:value-of select="name(.)"/></xsl:comment></xsl:if>
                       <xsl:if test="$Xclude='0'">
                        <!-- output element and attributes -->
                           <xsl:copy>
                                <!-- check attribute not excluded -->
                               <xsl:for-each select="@*">
                                   <xsl:variable name="this" select="concat(name(../../.),'\',name(../.),'\@',name(.))"/>
                                   <xsl:variable name="Xclude">
                                       <xsl:choose>
                                           <xsl:when test="$wanted/w:WantList//Attribute/@w:n=$this">
                                               <xsl:variable name="chkRoot" select="$wanted/w:WantList//Attribute[@w:n=$this]"/>
                                               <xsl:variable name="thisRoot" select="$chkRoot/../../@w:n[1]"/>
                                               <xsl:variable name="action" select="$wanted/w:WantList//Attribute[@w:n=$this]/@w:isXclude"/>
                                               <xsl:value-of select="$action[1]"/>
                                           </xsl:when>
                                           <xsl:otherwise>0</xsl:otherwise>
                                       </xsl:choose>
                                   </xsl:variable>
                                   <xsl:if test="$Xclude='0'">
                                       <xsl:copy-of select="."/>
                                   </xsl:if>
                               </xsl:for-each>
                               <xsl:copy-of select="node()"/>
                           </xsl:copy>
                       </xsl:if>
                    </xsl:otherwise>
                </xsl:choose>   
            </xsl:for-each>
            <!-- /xsl:element -->
                
             </xsl:copy>
        </xsl:if>
    </xsl:template>   
    
    <xsl:template match="attribute::node()" mode="map">
        <!-- xsl:message><xsl:value-of select="concat('attrib: ',name(../.),'\@',name())"/></xsl:message -->
    </xsl:template>

</xsl:stylesheet>
