<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
    xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs xsd"
    xmlns:as="http://www.oasis-open.org/committees/cam"
    xmlns:camed="http://jcam.org.uk/editor"
    xmlns:look="http://jcam.org.uk/LookupLists"
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" version="2.0">
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> March 4, 2011</xd:p>
            <xd:p><xd:b>Author:</xd:b> DRRW</xd:p>
            <xd:p></xd:p>
        </xd:desc>
    </xd:doc>
        
        <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
            Author: David RR Webber, March, 2011 -->
        <!--
            Purpose  : Read in Dictionary file(s) - convert into XML for Eclipse UI Viewer tool; return details back to Java via stdout. 
            Original : 03/04/2011
            Initials : drrw
            CAM ver    : 1.1 
            Description : Part of a set of tools to provide open source tools for using OASIS CAM dictionaries.
            Updated  - Inits - Change
            03/04/11   drrw    Initial release
            03/30/11   drrw    Adjust namespace if missing from names
            03/30/11   drrw    Add selection of parent / itempath / item
            05/18/11   drrw    Enhance to use compiled dictionary format
            05/18/11   drrw    Enhance to work as compiler tool for dictionary format
            05/18/11   drrw    Add control file sort attribute
            05/18/11   drrw    More components - fix structure and rules
            05/19/11   drrw    Add compile mode for outputting compiled dictionary
            05/19/11   drrw    Fix empty namespace issue in Parent elements
            05/20/11   drrw    Add sort option for root level ABIE components
            05/20/11   drrw    Fix default namespace logic for compile option
            06/16/11   drrw    Resolve raft of issues with dictionary harvested from set of templates
            06/16/11   drrw    Resolve looping and context matching issues
            06/20/11   drrw    Fix to make sure a type rule is always written
            07/14/11   drrw    Revert parent node name suffix from Details to Type to match on context paths
            03/13/12   drrw    Retain namespaces from dictionary in compiled dictionary root element
            03/15/12   drrw    Fix context checking of parent items at root level (empty parent context)
            03/15/12   drrw    Fix complexType and simpleType suffix in item names
            12/02/12   drrw    Handle http:// URI in dictionary file locations
            12/10/12   drrw    Handle file:// prefixing for file locations references
            12/11/12   drrw    Improve singleton BBIE handling
            12/11/12   drrw    Revise file URI handling
            12/11/12   drrw    Use doc-available() function
            12/12/12   drrw    Relax dictionary not found condition from error to warning and continue
            12/12/12   drrw    Refine handling of root/unique components count if 0 condition
            12/20/12   drrw    Update logic for moreComponents generation
            02/21/12   drrw    Add annotations to dictionary loader for Java
        -->
    
    <xsl:param name="locationFilePath">file:/temp/test/</xsl:param>
    <xsl:param name="mode">collection</xsl:param><!-- collection or compile -->
    <xsl:param name="dictionaryName"></xsl:param><!-- if empty will use name from first root Property item -->
    <xsl:param name="sort">false</xsl:param><!-- true/false boolean -->
    
    <xsl:variable name="version">1.18</xsl:variable>
    <xsl:variable name="debug">0</xsl:variable>
    
    <xsl:variable name="thisLocationFilePath">
        <xsl:choose>
            <xsl:when test="string-length($locationFilePath) = 0"/>
            <xsl:when test="contains($locationFilePath,'://')">
                <xsl:value-of select="if(ends-with($locationFilePath,'/')) then $locationFilePath else concat($locationFilePath,'/')"/>
            </xsl:when>
            <xsl:otherwise><xsl:value-of select="if(starts-with($locationFilePath,'/') or (starts-with($locationFilePath,'.'))) then '' else '/'"/><xsl:value-of select="if(ends-with($locationFilePath,'/')) then $locationFilePath else concat($locationFilePath,'/')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="dictionaryControlFile" select="/"/>
    
    <xsl:variable name="dictionary2compile" select="/"/>
    
    <xsl:variable name="chkControlFile" select="count($dictionaryControlFile//camed:dictionary)"/>
    
    <xsl:variable name="stripCardinality" select="$dictionaryControlFile//camed:dictionary[@stripCardinality='true']/@shortname"></xsl:variable>
    
    <xsl:variable name="dictionary">
        <xsl:for-each select="$dictionaryControlFile//camed:dictionary">
            <xsl:variable name="thisFile" select="@location"/>
            <xsl:variable name="currentName" select="@name"/>
            <xsl:variable name="fileNamePath">
                <xsl:choose>
                    <xsl:when test="contains(@location,'://')"><xsl:value-of select="@location"/></xsl:when>
                    <xsl:when test="string-length($locationFilePath) = 0">
                        <xsl:choose>
                            <xsl:when test="starts-with(@location,'/')">file://<xsl:value-of select="@location"/></xsl:when>
                            <xsl:when test="starts-with(@location,'../')"><xsl:value-of select="@location"/></xsl:when><!-- compute offset path relative to locationFilePath -->
                            <xsl:when test="starts-with(@location,'./')"><xsl:value-of select="@location"/></xsl:when><!-- compute offset path relative to locationFilePath -->
                            <xsl:otherwise><xsl:value-of select="@location"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test="contains($thisLocationFilePath,'://')">
                        <xsl:choose>
                            <xsl:when test="starts-with(@location,'/')"><xsl:value-of select="concat($thisLocationFilePath,substring-after(@location,'/'))"/></xsl:when>
                            <xsl:when test="starts-with(@location,'../')"><xsl:value-of select="concat($thisLocationFilePath,@location)"/></xsl:when><!-- compute offset path relative to locationFilePath -->
                            <xsl:when test="starts-with(@location,'./')"><xsl:value-of select="concat($thisLocationFilePath,@location)"/></xsl:when><!-- compute offset path relative to locationFilePath -->
                            <xsl:otherwise><xsl:value-of select="concat($thisLocationFilePath,@location)"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test="string-length($locationFilePath) &gt; 0">
                        <xsl:value-of select="if(starts-with(@location,'/')) then concat($thisLocationFilePath,substring-after(@location,'/')) else concat($thisLocationFilePath,@location)"/>
                    </xsl:when>
                    <xsl:otherwise><xsl:value-of select="@location"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <components shortname="{@shortname}" namespaces="{@namespaces}">	
                <!-- xsl:copy-of select="document($thisFile)"/ -->
                <xsl:message>- processing dictionary - <xsl:value-of select="$currentName"/>: <xsl:value-of select="$fileNamePath"/> </xsl:message>
                <xsl:if test="not(doc-available($fileNamePath))">
                        <xsl:message>ERROR: Dictionary input file is not available! <xsl:value-of select="$fileNamePath"/></xsl:message>
                </xsl:if>
                <xsl:variable name="tempDictionary" select="document($fileNamePath)"/>
                <xsl:element name="ITEMSlookup">
                    <!-- Add namespace declarations -->
                    <xsl:variable name="namespaces" select="$tempDictionary/*/namespace::*"/>
                    <xsl:copy-of select="$namespaces"/>
                    <xsl:comment><xsl:value-of select="$tempDictionary/ITEMSlookup/comment()"></xsl:value-of></xsl:comment>
                    <!-- now unpack the context -->
                    <xsl:for-each select="$tempDictionary//Property">
                        <xsl:variable name="propertyDetails" select="."/>
                        <xsl:variable name="uniqueContext" select="tokenize(./@context,'&#xA;')"/>
                        <xsl:for-each select="$uniqueContext">
                            <xsl:if test="string-length(.) &gt; 0">
                                <xsl:element name="Property">
                                    <xsl:for-each select="$propertyDetails/@*">
                                        <xsl:choose>
                                            <xsl:when test="name()='context'"/><!-- skip existing context -->
                                            <xsl:when test="name()='group'"/><!-- skip  -->
                                            <xsl:when test="name()='where'"/><!-- skip  -->
                                            <xsl:when test="name()='notes'"/><!-- skip  -->
                                            <xsl:when test="name()='language'"/><!-- skip  -->
                                            <xsl:when test="name()='anno'"/><!-- skip  -->
                                            <xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
                                        </xsl:choose>
                                   </xsl:for-each>
                                    <xsl:attribute name="context" select="."/>
                                </xsl:element>
                            </xsl:if>
                        </xsl:for-each>	
                    </xsl:for-each>
                </xsl:element>
            </components>
        </xsl:for-each>
    </xsl:variable>
    
    <xsl:output method="xml" indent="yes" exclude-result-prefixes="xd xs xsd look"/>
    
    <xsl:template match="/">
        
        <xsl:choose>
            <xsl:when test="$mode='compile'"><!-- special mode to compile one dictionary -->
                <xsl:message>Starting Dictionary compiler - version <xsl:value-of select="$version"/></xsl:message>
                <xsl:variable name="dictionaryLongName" select="if($dictionaryName='') then $dictionary2compile/ITEMSlookup/Property[1]/@item else $dictionaryName"/>
                <xsl:message>Collecting namespaces for <xsl:value-of select="$dictionaryLongName"/></xsl:message>
                <xsl:variable name="ns-list">
                   <xsl:for-each select="/ITEMSlookup/Property[contains(@item,':')]/@item"><xsl:value-of select="substring-before(.,':')"/>|</xsl:for-each>
                </xsl:variable>
                <xsl:variable name="ns-uniques" select="distinct-values(tokenize($ns-list,'\|'))"/>
                <!-- now figure out the default namespace and put that first in the list -->
                <xsl:variable name="ns-default" select="if(contains(/ITEMSlookup/Property[1]/@item,':')) then '' else ','"/>
                <xsl:variable name="ns-parm"><xsl:value-of select="$ns-default"/><xsl:for-each select="$ns-uniques"><xsl:if test="not(. = '')"><xsl:if test="position() &gt; 1">,</xsl:if><xsl:value-of select="."/></xsl:if></xsl:for-each></xsl:variable>
                <!-- store namespace declarations -->
                <xsl:variable name="namespaces" select="/*/namespace::*"/>
                
                <xsl:variable name="dictionaryParams">
                    <camed:dictionary name="{if($dictionaryName='') then $dictionary2compile/ITEMSlookup/Property[1]/@item else $dictionaryName}" shortname="{$dictionary2compile/ITEMSlookup/Property[1]/@item}" location="compiled-dictionary.xml" namespaces="{$ns-parm}" stripCardinality="false">
                        <camed:annotation item="//camed:dictionary[@shortname='{$dictionary2compile/ITEMSlookup/Property[1]/@item}']">
                            <camed:documentation type="Description">Compiled dictionary for <xsl:value-of select="$dictionaryLongName"/></camed:documentation>
                        </camed:annotation> 
                    </camed:dictionary>
                </xsl:variable>
                <!-- Now compile the dictionary -->
                <xsl:call-template name="DictionaryCompiler">
                    <xsl:with-param name="dictionary" select="$dictionaryParams/*/."/>
                    <xsl:with-param name="dictionaryFile" select="$dictionary2compile"/>
                    <xsl:with-param name="stripCardinality" select="$stripCardinality"/>
                    <xsl:with-param name="namespaces" select="$namespaces"/>
                </xsl:call-template>
                <xsl:message>Finished Dictionary compiling</xsl:message>   
            </xsl:when>
            <xsl:otherwise>
                <!-- default - create collection -->
         <xsl:message>Starting Dictionary loader - version <xsl:value-of select="$version"/></xsl:message>        
        <xsl:if test="$chkControlFile &lt; 1"><xsl:message terminate="yes">ERROR: Dictionary control file is invalid!</xsl:message></xsl:if>

        <xsl:element name="Dictionaries">
            <xsl:namespace name="as">http://www.oasis-open.org/committees/cam</xsl:namespace>
            <xsl:namespace name="camed">http://jcam.org.uk/editor</xsl:namespace>
            
          <xsl:for-each select="$dictionaryControlFile//camed:collection">
            <xsl:element name="Collection">
                <xsl:attribute name="name" select="@name"/>
            <xsl:copy-of select="as:Namespaces"/>
                
            <xsl:for-each select="camed:dictionary">
            <xsl:variable name="currentName" select="@name"/>
            <xsl:variable name="fileNamePath">
                <xsl:choose>
                    <xsl:when test="contains(@location,'://')"><xsl:value-of select="@location"/></xsl:when>
                    <xsl:when test="string-length($locationFilePath) = 0">
                        <xsl:choose>
                            <xsl:when test="starts-with(@location,'/')">file://<xsl:value-of select="@location"/></xsl:when>
                            <xsl:when test="starts-with(@location,'../')"><xsl:value-of select="@location"/></xsl:when><!-- compute offset path relative to locationFilePath -->
                            <xsl:when test="starts-with(@location,'./')"><xsl:value-of select="@location"/></xsl:when><!-- compute offset path relative to locationFilePath -->
                            <xsl:otherwise><xsl:value-of select="@location"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test="contains($thisLocationFilePath,'://')">
                        <xsl:choose>
                            <xsl:when test="starts-with(@location,'/')"><xsl:value-of select="concat($thisLocationFilePath,substring-after(@location,'/'))"/></xsl:when>
                            <xsl:when test="starts-with(@location,'../')"><xsl:value-of select="concat($thisLocationFilePath,@location)"/></xsl:when><!-- compute offset path relative to locationFilePath -->
                            <xsl:when test="starts-with(@location,'./')"><xsl:value-of select="concat($thisLocationFilePath,@location)"/></xsl:when><!-- compute offset path relative to locationFilePath -->
                            <xsl:otherwise><xsl:value-of select="concat($thisLocationFilePath,@location)"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:when test="string-length($locationFilePath) &gt; 0">
                        <xsl:value-of select="if(starts-with(@location,'/')) then concat($thisLocationFilePath,substring-after(@location,'/')) else concat($thisLocationFilePath,@location)"/>
                    </xsl:when>
                    <xsl:otherwise><xsl:value-of select="@location"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
                <xsl:message>- processing dictionary file - <xsl:value-of select="$currentName"/>: <xsl:value-of select="$fileNamePath"/> </xsl:message>
            <xsl:if test="not(doc-available($fileNamePath))">
                <xsl:message>ERROR: Dictionary input file is not available! <xsl:value-of select="$fileNamePath"/></xsl:message>
            </xsl:if>
            <xsl:variable name="dictionaryFile" select="document($fileNamePath)"/>
            <xsl:variable name="shortname" select="@shortname"/>
          
            <xsl:choose>
                <xsl:when test="$dictionaryFile//Dictionary[@name=$currentName]">
                    <!-- Dictionary is already pre-compiled - so simply copy it -->
                    <xsl:message> - loading compiled dictionary <xsl:value-of select="$currentName"/></xsl:message>   
                    <xsl:copy-of select="$dictionaryFile//Dictionary[@name=$currentName]"/>
                </xsl:when>
                <xsl:when test="count($dictionaryFile//Dictionary) &gt; 0">
                    <!-- Dictionary is already pre-compiled - so simply copy it -->
                    <xsl:message>- ERROR loading compiled dictionary <xsl:value-of select="$currentName"/> not found!</xsl:message>   
                    <xsl:message>- ERROR check that compiled dictionary @name = [<xsl:value-of select="$currentName"/>] in your control file!</xsl:message>   
                </xsl:when>
                 <xsl:when test="$dictionaryFile//IEPD-Factory">
                     <xsl:element name="Dictionary">
                         <xsl:attribute name="name" select="@name"/>
                         <xsl:attribute name="description" select="camed:annotation/camed:documentation"/>
                         <xsl:attribute name="item" select="camed:annotation/@item"/>
                         <xsl:attribute name="shortname" select="@shortname"/>
                         <xsl:attribute name="namespaces" select="@namespaces"/>
                        <xsl:apply-templates mode="IEPD-Factory" select="$dictionaryFile"/>
                     </xsl:element>
                 </xsl:when>
                <!-- debugging aid to process only first entry in dictionary list -->
                <!-- xsl:when test="$debug = 0 and(position() = 2)"/>
                <xsl:when test="position() &gt; 2"/ -->
                 <xsl:otherwise>
                     <xsl:call-template name="DictionaryCompiler">
                         <xsl:with-param name="dictionary" select="."/>
                         <xsl:with-param name="dictionaryFile" select="$dictionaryFile"/>
                         <xsl:with-param name="stripCardinality" select="$stripCardinality"/>
                         <xsl:with-param name="namespaces"/>
                     </xsl:call-template>
                 </xsl:otherwise>
            </xsl:choose>   
            </xsl:for-each>
            </xsl:element>
        </xsl:for-each>
        </xsl:element>
         <xsl:message>Finished Dictionary loading</xsl:message>   
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="DictionaryCompiler">
        <xsl:param name="dictionary"/>
        <xsl:param name="dictionaryFile"/>
        <xsl:param name="stripCardinality"/>
        <xsl:param name="namespaces"/>
        <xsl:message> - compiling dictionary: <xsl:value-of select="$dictionary/./@shortname"/></xsl:message>
        <xsl:message> - compiler version: <xsl:value-of select="$version"/></xsl:message>
        <!-- xsl:message> - compiling dictionary: <xsl:copy-of select="$dictionary"/></xsl:message -->
        <xsl:for-each select="$dictionary/.">
           <xsl:message> - processing <xsl:value-of select="count($dictionaryFile//Property[@CCTS='ABIE'])"/> parent components</xsl:message>
           <xsl:variable name="rootParents" select="count($dictionaryFile//Property[@CCTS='ABIE'][not(contains(@context,'&#xA;/'))])"/>
           <xsl:message> - root/unique parents count: <xsl:value-of select="$rootParents"/></xsl:message>
           <xsl:variable name="shortname" select="@shortname"/>
           <xsl:variable name="sortParents" select="@sort"/>
           <xsl:element name="Dictionary">
            <xsl:attribute name="name" select="@name"/>
            <xsl:attribute name="description" select="camed:annotation/camed:documentation"/>
            <xsl:attribute name="item" select="camed:annotation/@item"/>
            <xsl:attribute name="shortname" select="@shortname"/>
            <xsl:attribute name="namespaces" select="@namespaces"/>
            <xsl:copy-of select="$namespaces"/>
            <xsl:comment><xsl:value-of select="$dictionaryFile/ITEMSlookup/comment()"></xsl:value-of></xsl:comment>
            
            <xsl:if test="$stripCardinality[. = $shortname]"><xsl:message> Strip Cardinality for <xsl:value-of select="$shortname"/> = true</xsl:message></xsl:if>
            <xsl:if test="$sort='true' or($sortParents='true')"><xsl:message> Sorting dictionary components</xsl:message></xsl:if>
            <xsl:variable name="defaultNamespace" select="if(contains(@namespaces,',')) then substring-before(@namespaces,',') else normalize-space(@namespaces)"/>
               
            <xsl:variable name="noDetailsChk" select="$dictionaryFile//Property[contains(@context,'Details/')]/@item"/><!-- Check to see if any components context end in the word Details/ -->
            <!-- xsl:if test="count($noDetailsChk/.) &gt; 0"><xsl:message> - details items : <xsl:value-of select="distinct-values($noDetailsChk)"/></xsl:message></xsl:if -->
            
            <xsl:for-each select="$dictionaryFile//Property[@CCTS='ABIE']">
               <xsl:sort select="if($sort='true' or($sortParents='true')) then @item else @CCTS"/>
                <xsl:if test="$debug = 1"><xsl:message> - <xsl:value-of select="@item"/></xsl:message></xsl:if>
                <xsl:if test="normalize-space(@context) = concat('/',@item) or($rootParents = 0)"><!-- Pick parents off the root - not within other parents -->
                    <xsl:variable name="ABIEdetails">
                        <xsl:choose>
                            <xsl:when test="ends-with(@item,'Type')"><!-- If its name ends in Type then rename it -->
                                <xsl:variable name="oldName" select="@item"/>
                                <xsl:variable name="newName" >
                                    <xsl:choose>
                                        <xsl:when test="ends-with(@item,'complexType')"><xsl:value-of select="replace(concat(@item,'ZZZ'),'complexTypeZZZ','Details')"/></xsl:when>
                                        <xsl:when test="ends-with(@item,'simpleType')"><xsl:value-of select="replace(concat(@item,'ZZZ'),'simpleTypeZZZ','')"/></xsl:when>
                                        <xsl:otherwise><xsl:value-of select="replace(concat(@item,'ZZZ'),'TypeZZZ','Details')"/></xsl:otherwise>
                                    </xsl:choose>
                                </xsl:variable>
                                <xsl:element name="Property">
                                    <xsl:for-each select="@*">
                                        <xsl:choose>
                                            <xsl:when test="name()='item'"><xsl:attribute name="item" select="$newName"/></xsl:when>
                                            <xsl:when test="name()='children'"><xsl:attribute name="children" select="replace(replace(.,concat('/',$oldName,'&#xA;'),concat('/',$newName,'&#xA;')),concat(';',$oldName,'&#xA;'),concat(';',$newName,'&#xA;'))"/></xsl:when>
                                            <xsl:when test="name()='context'"><xsl:attribute name="context" select="concat('/',$newName)"/></xsl:when>
                                            <xsl:when test="name()='label'"><xsl:variable name="labelText" select="substring-before(concat(normalize-space(.),'ZZZ'),'TypeZZZ')"/><xsl:attribute name="label" select="concat($labelText,'Details')"/></xsl:when>
                                            <xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:for-each>
                                </xsl:element>
                            </xsl:when>
                            <xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:if test="$debug=1"><xsl:message> - <xsl:value-of select="$ABIEdetails/Property[1]/@item"/></xsl:message></xsl:if>
                    <xsl:apply-templates mode="CAM" select="$ABIEdetails">
                        <xsl:with-param name="shortname" select="$shortname"/>
                        <xsl:with-param name="dictionary" select="$dictionaryFile"/>
                        <xsl:with-param name="itemPath" select="'|'"/>
                        <xsl:with-param name="parent"/>
                        <xsl:with-param name="defaultNS" select="$defaultNamespace"/>
                        <xsl:with-param name="noDetailsChk" select="$noDetailsChk"/>
                    </xsl:apply-templates>
                </xsl:if>
            </xsl:for-each>
               
            <xsl:if test="$rootParents &gt; 0">   
            <xsl:element name="Parent">
                   <xsl:attribute name="name">Components</xsl:attribute>
                   <xsl:attribute name="label">Unique Components</xsl:attribute>
                   <xsl:attribute name="definition">These are unique components and types available in the dictionary.</xsl:attribute>
               
               <xsl:for-each select="$dictionaryFile//Property[@CCTS='ABIE']">
                   <xsl:sort select="if($sort='true' or($sortParents='true')) then @item else @CCTS"/>
                   <xsl:if test="$debug = 1"><xsl:message> - <xsl:value-of select="@item"/></xsl:message></xsl:if>
                   <xsl:if test="not(contains(@context,'&#xA;/')) and(not(normalize-space(@context) = concat('/',@item)))"><!-- Pick unique parents that are not off the root - and not within other parents -->
                       <xsl:variable name="ABIEdetails">
                           <xsl:choose>
                               <xsl:when test="ends-with(@item,'Type')"><!-- If its name ends in Type then rename it -->
                                   <xsl:variable name="oldName" select="@item"/>
                                   <xsl:variable name="newName" >
                                       <xsl:choose>
                                           <xsl:when test="ends-with(@item,'complexType')"><xsl:value-of select="replace(concat(@item,'ZZZ'),'complexTypeZZZ','Details')"/></xsl:when>
                                           <xsl:when test="ends-with(@item,'simpleType')"><xsl:value-of select="replace(concat(@item,'ZZZ'),'simpleTypeZZZ','')"/></xsl:when>
                                           <xsl:otherwise><xsl:value-of select="replace(concat(@item,'ZZZ'),'TypeZZZ','Details')"/></xsl:otherwise>
                                       </xsl:choose>
                                   </xsl:variable>
                                   <xsl:element name="Property">
                                       <xsl:for-each select="@*">
                                           <xsl:choose>
                                               <xsl:when test="name()='item'"><xsl:attribute name="item" select="$newName"/></xsl:when>
                                               <xsl:when test="name()='children'"><xsl:attribute name="children" select="replace(replace(.,concat('/',$oldName,'&#xA;'),concat('/',$newName,'&#xA;')),concat(';',$oldName,'&#xA;'),concat(';',$newName,'&#xA;'))"/></xsl:when>
                                               <xsl:when test="name()='context'"><xsl:attribute name="context" select="concat('/',$newName)"/></xsl:when>
                                               <xsl:when test="name()='label'"><xsl:variable name="labelText" select="substring-before(concat(normalize-space(.),'ZZZ'),'TypeZZZ')"/><xsl:attribute name="label" select="concat($labelText,'Details')"/></xsl:when>
                                               <xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
                                           </xsl:choose>
                                       </xsl:for-each>
                                   </xsl:element>
                               </xsl:when>
                               <xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
                           </xsl:choose>
                       </xsl:variable>
                       <xsl:if test="$debug=1"><xsl:message> - <xsl:value-of select="$ABIEdetails/Property[1]/@item"/></xsl:message></xsl:if>
                       <xsl:apply-templates mode="CAM" select="$ABIEdetails">
                           <xsl:with-param name="shortname" select="$shortname"/>
                           <xsl:with-param name="dictionary" select="$dictionaryFile"/>
                           <xsl:with-param name="itemPath" select="'|'"/>
                           <xsl:with-param name="parent"/>
                           <xsl:with-param name="defaultNS" select="$defaultNamespace"/>
                           <xsl:with-param name="noDetailsChk" select="$noDetailsChk"/>
                       </xsl:apply-templates>
                   </xsl:if>
               </xsl:for-each>
            </xsl:element>
            </xsl:if>
            
            <xsl:variable name="leafComponents" select="count($dictionaryFile//Property[@CCTS='BBIE'][not(contains(substring-after(@context,'/'),'/'))])"/>
            
            <xsl:if test="$leafComponents &gt; 0">
            <xsl:message> - outputting leaf components: <xsl:value-of select="$leafComponents"/></xsl:message>
            <xsl:element name="Parent">
                <xsl:attribute name="name">SingletonComponents</xsl:attribute>
                <xsl:attribute name="label">Singleton Components; unreferenced</xsl:attribute>
                <xsl:attribute name="definition">These are standalone atomic components and types available in the dictionary.</xsl:attribute>
                <xsl:for-each select="$dictionaryFile//Property[@CCTS='BBIE']">
                    <xsl:sort select="if($sort='true' or($sortParents='true')) then @item else @CCTS"/>
                    <xsl:if test="@context = concat('/',@item)">
                        <xsl:element name="Child">
                            <xsl:attribute name="name" select="replace(@item,'simpleType','')"/>
                            <xsl:if test="contains(@cardinality,'repeatable')"><xsl:attribute name="repeatable">true</xsl:attribute></xsl:if>
                            <xsl:if test="contains(@cardinality,'optional')"><xsl:attribute name="optional">true</xsl:attribute></xsl:if>
                            <xsl:if test="string-length(@UID) &gt; 0"><xsl:attribute name="UID" select="@UID"/></xsl:if>
                            <xsl:if test="string-length(@label) &gt; 0"><xsl:attribute name="label" select="replace(@label,'&#xA;','')"/></xsl:if>
                            <xsl:if test="string-length(@definition) &gt; 0"><xsl:attribute name="definition" select="replace(@definition,'&#xA;','')"/></xsl:if>
                            <xsl:if test="string-length(@anno) &gt; 0"><xsl:attribute name="anno" select="@anno"/></xsl:if>
                            <xsl:variable name="childrenChk" select="substring-after(@children,@item)"/>
                            <xsl:for-each select="tokenize(@children,'&#xA;')">
                                <xsl:variable name="this" select="."/>
                                <xsl:if test="starts-with($this,'@')"><!-- Process attribute -->
                                    <xsl:element name="Attribute">
                                        <xsl:attribute name="name" select="substring-after($this,'@')"/>
                                        <!-- Lookup cardinality and UID -->
                                        <xsl:variable name="ASBIEdetails" select="$dictionaryFile//Property[@CCTS='ASBIE'][@item=substring-after($this,'@')]"/>
                                        <xsl:if test="contains($ASBIEdetails[1]/@cardinality[1],'optional')"><xsl:attribute name="optional">true</xsl:attribute></xsl:if>
                                        <xsl:if test="string-length($ASBIEdetails[1]/@UID[1]) &gt; 0"><xsl:attribute name="UID" select="$ASBIEdetails[1]/@UID[1]"/></xsl:if>
                                        <xsl:if test="string-length($ASBIEdetails[1]/@label[1]) &gt; 0"><xsl:attribute name="label" select="replace($ASBIEdetails[1]/@label[1],'&#xA;','')"/></xsl:if>
                                        <xsl:if test="string-length($ASBIEdetails[1]/@definition[1]) &gt; 0"><xsl:attribute name="definition" select="replace($ASBIEdetails[1]/@definition[1],'&#xA;','')"/></xsl:if>
                                        <xsl:if test="string-length($ASBIEdetails[1]/@anno[1]) &gt; 0"><xsl:attribute name="anno" select="$ASBIEdetails[1]/@anno[1]"/></xsl:if>
                                    </xsl:element>
                                </xsl:if>
                            </xsl:for-each>
                        </xsl:element>             
                    </xsl:if>
                </xsl:for-each>
            </xsl:element>
            </xsl:if>
               
            <!-- Now output the rules for section -->
            <xsl:element name="Rules">
                <xsl:variable name="uniqueRules" select="distinct-values($dictionaryFile//Property/@UID)"/>
                <xsl:for-each select="$uniqueRules">
                    <xsl:variable name="thisUID" select="."/>
                    <xsl:variable name="thisRule" select="$dictionaryFile//Property[@UID = $thisUID]"/>
                    <xsl:variable name="outputRule">
                        <xsl:choose>
                            <xsl:when test="$thisRule/@type = 'string'"><xsl:if test="string-length($thisRule[1]/@mask) &gt; 0 or(string-length($thisRule[1]/@rules) &gt; 0)">true</xsl:if></xsl:when>
                            <xsl:when test="string-length($thisRule[1]/@type) &gt; 0 or(string-length($thisRule[1]/@mask) &gt; 0) or(string-length($thisRule[1]/@rules) &gt; 0)">true</xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose>   
                    </xsl:variable>
                    <xsl:choose>
                        <xsl:when test="$outputRule='true'">
                            <xsl:element name="Rule">
                                <xsl:attribute name="UID"><xsl:value-of select="$thisUID"/></xsl:attribute>
                                <xsl:if test="string-length($thisRule[1]/@type) &gt; 0">
                                    <xsl:attribute name="type"><xsl:value-of select="$thisRule[1]/@type"/></xsl:attribute>
                                </xsl:if>
                                <xsl:if test="string-length($thisRule[1]/@mask) &gt; 0">
                                    <xsl:if test="not(string-length($thisRule[1]/@type) &gt; 0)"><!-- make sure we write out a default type if there is none -->
                                        <xsl:choose><!-- work out the type for the mask -->
                                            <xsl:when test="contains($thisRule[1]/@mask,'true|false')"><xsl:attribute name="type">boolean</xsl:attribute></xsl:when>
                                            <xsl:when test="contains($thisRule[1]/@mask,'|')"><xsl:attribute name="type">list</xsl:attribute></xsl:when>
                                            <xsl:when test="contains($thisRule[1]/@mask,'#.')"><xsl:attribute name="type">decimal</xsl:attribute></xsl:when>
                                            <xsl:when test="contains($thisRule[1]/@mask,'#')"><xsl:attribute name="type">integer</xsl:attribute></xsl:when>
                                            <xsl:when test="contains($thisRule[1]/@mask,'-DD''T''')"><xsl:attribute name="type">datetime</xsl:attribute></xsl:when>
                                            <xsl:when test="contains($thisRule[1]/@mask,'-DD')"><xsl:attribute name="type">date</xsl:attribute></xsl:when>
                                            <xsl:otherwise><xsl:attribute name="type">token</xsl:attribute></xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:if>
                                    <xsl:attribute name="mask"><xsl:value-of select="$thisRule[1]/@mask"/></xsl:attribute>
                                </xsl:if>
                                <xsl:if test="string-length($thisRule[1]/@rules) &gt; 0">
                                    <xsl:attribute name="rules"><xsl:value-of select="$thisRule[1]/@rules"/></xsl:attribute>
                                </xsl:if>
                            </xsl:element>
                        </xsl:when>
                        <xsl:otherwise>
                          <xsl:element name="Rule">
                            <xsl:attribute name="UID"><xsl:value-of select="$thisUID"/></xsl:attribute>
                            <xsl:attribute name="type">token</xsl:attribute>
                          </xsl:element>
                        </xsl:otherwise>
                    </xsl:choose>    
                </xsl:for-each>
            </xsl:element>
        </xsl:element>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template match="*" mode="IEPD-Factory">
       <xsl:comment>IEPD Factory dictionary mode</xsl:comment>
        <!-- Placeholder - no logic currently here for IEPD factory dictionaries -->
    </xsl:template>
    
    <xsl:template match="@*" mode="IEPD-Factory">
        <xsl:copy-of select="."/>
    </xsl:template>
    
    <xsl:template match="Property[@CCTS='ABIE']" mode="CAM">
        <xsl:param name="shortname"/>
        <xsl:param name="dictionary"/>
        <xsl:param name="itemPath"/>
        <xsl:param name="parent"/>
        <xsl:param name="defaultNS"/>
        <xsl:param name="noDetailsChk"/>
        <!-- xsl:message> - parent: <xsl:value-of select="$parent"/> - [<xsl:value-of select="@item"/>]</xsl:message -->
        <xsl:variable name="stripChildCardinality">
            <xsl:choose>
                <xsl:when test="$stripCardinality[. = $shortname]">true</xsl:when>
                <xsl:otherwise>false</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="newName" select="if(contains(@item,':')) then @item else if(string-length($defaultNS) &gt; 0) then concat($defaultNS,':',@item) else @item"/>
        
        <xsl:choose>
            <xsl:when test="contains(@item,'AllRootLevelTypeDefinitions')"/><!-- ignore root element -->
            <xsl:otherwise>
                <xsl:element name="Parent">
                    <xsl:attribute name="name" select="$newName"/>
                    <xsl:if test="contains(@cardinality,'repeatable')"><xsl:attribute name="repeatable">true</xsl:attribute></xsl:if>
                    <xsl:if test="contains(@cardinality,'optional')"><xsl:attribute name="optional">true</xsl:attribute></xsl:if>
                    <xsl:if test="string-length(@UID) &gt; 0"><xsl:attribute name="UID" select="@UID"/></xsl:if>
                    <xsl:if test="string-length(@label) &gt; 0"><xsl:attribute name="label" select="replace(@label,'&#xA;','')"/></xsl:if>
                    <xsl:if test="string-length(@definition) &gt; 0"><xsl:attribute name="definition" select="replace(@definition,'&#xA;','')"/></xsl:if>
                    <xsl:if test="string-length(@anno) &gt; 0"><xsl:attribute name="anno" select="@anno"/></xsl:if>
                    
                    <!-- xsl:variable name="thisParent" select="if(ends-with(@item,'Details')) then replace(concat(@item,'ZZZ'),'DetailsZZZ','Type') else @item"/ -->
                    <xsl:variable name="thisParent" select="if(count($noDetailsChk/.) = 0 and(ends-with(@item,'Details'))) then concat(substring(@item,1,string-length(@item) - 7),'Type') else @item"/>
                  
                    <xsl:variable name="childrenChk" select="substring-after(@children,@item)"/>
                   
                    <xsl:for-each select="distinct-values(tokenize(@children,'&#xA;'))">
                        <xsl:choose>
                            <xsl:when test="contains(.,'/')"/><!-- Not a child item - but the context path from the @children - so ignore -->
                            <xsl:when test="string-length(normalize-space(.)) &lt; 1"/><!-- ignore empty token -->
                           <xsl:when test="$thisParent = ."><xsl:if test="$debug &lt; 3"><xsl:message> - ignored recursive child <xsl:value-of select="."/></xsl:message></xsl:if></xsl:when><!-- ignore recursive references -->
                          <xsl:when test="position() &gt; 1"><!-- process from after first token -->
                            <xsl:variable name="this" select="normalize-space(.)"/>
                            <xsl:variable name="context" select="if(ends-with($this,concat('/',$thisParent))) then replace(concat('/',$this,'&#xA;'),'//','/') else replace(concat('/',normalize-space($thisParent),'/',$this,'&#xA;'),'//','/')"/>
                            <xsl:variable name="moreContext" select="if(ends-with($this,concat('/',$thisParent))) then replace(concat('/',$this),'//','/') else replace(concat('/',normalize-space($thisParent),'/',$this),'//','/')"/><!--  -->
                            <xsl:variable name="singleContext" select="if(ends-with($this,concat('/',$thisParent))) then replace(concat('/',$this),'//','/') else replace(concat('/',normalize-space($thisParent),'/',$this),'//','/')"/>
                            <xsl:variable name="comparePath">
                                <xsl:choose>
                                    <xsl:when test="$itemPath = '|'">/<xsl:value-of select="$parent"/>/</xsl:when>
                                    <xsl:otherwise><xsl:value-of  select="replace($itemPath,'\|','/')"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>
                            <xsl:variable name="parentContext">
                                <xsl:choose>
                                    <xsl:when test="ends-with($comparePath,$this)"><xsl:value-of select="$comparePath"/></xsl:when>
                                    <xsl:when test="not(ends-with($parent,$thisParent))"><xsl:value-of select="replace(concat(if(normalize-space($parent) = '') then '' else '/',normalize-space($parent),'/',$singleContext),'//','/')"/></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="replace(concat(if(normalize-space($parent) = '') then '' else '/',normalize-space($parent),'/',$this),'//','/')"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>
                            <xsl:variable name="fullContext" select="$parentContext"/>
                            <xsl:variable name="itemContext">
                                <xsl:choose>
                                    <xsl:when test="ends-with($comparePath,$this)"><xsl:value-of select="$comparePath"/></xsl:when>
                                    <xsl:when test="ends-with($this,$comparePath)"><xsl:value-of select="$this"/></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="replace(concat('/',$comparePath,$this),'//','/')"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>
                            
                            <!-- xsl:message> - paths: </xsl:message>
                            <xsl:message> - - this [<xsl:value-of select="$this"/>]</xsl:message>
                            <xsl:message> - - context <xsl:value-of select="$context"/></xsl:message>
                            <xsl:message> - - moreContext <xsl:value-of select="$moreContext"/></xsl:message>
                            <xsl:message> - - singleContext <xsl:value-of select="$singleContext"/></xsl:message>  
                            <xsl:message> - - parentContext <xsl:value-of select="$parentContext"/></xsl:message>                            
                            <xsl:message> - - fullContext <xsl:value-of select="$fullContext"/></xsl:message>
                            <xsl:message> - - parent [<xsl:value-of select="$parent"/>][<xsl:value-of select="$comparePath"/>]</xsl:message>
                            <xsl:message> - - itemContext <xsl:value-of select="$itemContext"/></xsl:message -->
                              
                            <xsl:choose>
                                <xsl:when test="$this = $thisParent"><!-- Recursive reference to parent -->
                                    <xsl:message> - recursive child of <xsl:value-of select="$thisParent"/></xsl:message>
                                    <xsl:element name="Parent">
                                        <xsl:attribute name="name" select="$this"/>
                                        <!-- xsl:attribute name="recursive" select="'true'"/>
                                        <xsl:attribute name="definition" select="'recursive child'"/ -->
                                    </xsl:element>
                                </xsl:when>
                                <xsl:when test="starts-with($this,'@') and($dictionary//Property[@CCTS='ASBIE'][@item=substring-after($this,'@')])"><!-- Attribute -->
                                    <xsl:element name="Attribute">
                                        <xsl:attribute name="name" select="substring-after($this,'@')"/>
                                        <!-- Lookup cardinality and UID -->
                                        <xsl:variable name="ASBIEdetails" select="$dictionary//Property[@CCTS='ASBIE'][@item=substring-after($this,'@')]"/>
                                        <xsl:if test="contains($ASBIEdetails[1]/@cardinality[1],'optional')"><xsl:attribute name="optional">true</xsl:attribute></xsl:if>
                                        <xsl:if test="string-length($ASBIEdetails[1]/@UID[1]) &gt; 0"><xsl:attribute name="UID" select="$ASBIEdetails[1]/@UID[1]"/></xsl:if>
                                        <xsl:if test="string-length($ASBIEdetails[1]/@label[1]) &gt; 0"><xsl:attribute name="label" select="replace($ASBIEdetails[1]/@label[1],'&#xA;','')"/></xsl:if>
                                        <xsl:if test="string-length($ASBIEdetails[1]/@definition[1]) &gt; 0"><xsl:attribute name="definition" select="replace($ASBIEdetails[1]/@definition[1],'&#xA;','')"/></xsl:if>
                                        <xsl:if test="string-length($ASBIEdetails[1]/@anno[1]) &gt; 0"><xsl:attribute name="anno" select="$ASBIEdetails[1]/@anno[1]"/></xsl:if>
                                    </xsl:element>
                                </xsl:when>
                                <xsl:when test="$dictionary//Property[@CCTS='BBIE'][@item=$this]"><!-- Matched child element -->
                                        <!-- Lookup cardinality and UID -->
                                        <xsl:variable name="BBIEdetails" select="$dictionary//Property[@CCTS='BBIE'][@item=$this]"/>
                                        <!-- Now ignore those without the correct context -->
                                        <xsl:variable name="contextBBIEdetails">
                                            <xsl:for-each select="$BBIEdetails">
                                                <xsl:if test="contains(@context,$context)"><match><xsl:for-each select="@*"><xsl:attribute name="{name()}" select="."/></xsl:for-each></match></xsl:if>
                                            </xsl:for-each>
                                        </xsl:variable>
                                    
                                      <xsl:element name="Child">
                                        <xsl:attribute name="name" select="replace(.,'simpleType','')"/>  
                                        <xsl:choose>
                                            <xsl:when test="count($contextBBIEdetails/match) &gt; 0"><!-- use the matched context details -->
                                                <xsl:if test="$stripChildCardinality='false'">
                                                    <xsl:if test="contains($contextBBIEdetails[1]/match[1]/@cardinality[1],'repeatable')"><xsl:attribute name="repeatable">true</xsl:attribute></xsl:if>
                                                    <xsl:if test="contains($contextBBIEdetails[1]/match[1]/@cardinality[1],'optional')"><xsl:attribute name="optional">true</xsl:attribute></xsl:if>
                                                </xsl:if>
                                                <xsl:if test="string-length($contextBBIEdetails[1]/match[1]/@UID[1]) &gt; 0"><xsl:attribute name="UID" select="$contextBBIEdetails[1]/match[1]/@UID[1]"/></xsl:if>
                                                <xsl:if test="string-length($contextBBIEdetails[1]/match[1]/@type[1]) &gt; 0"><xsl:attribute name="type" select="$contextBBIEdetails[1]/match[1]/@type[1]"/></xsl:if>
                                                <xsl:if test="string-length($contextBBIEdetails[1]/match[1]/@label[1]) &gt; 0"><xsl:attribute name="label" select="replace($contextBBIEdetails[1]/match[1]/@label[1],'&#xA;','')"/></xsl:if>                                      
                                                <xsl:if test="string-length($contextBBIEdetails[1]/match[1]/@definition[1]) &gt; 0"><xsl:attribute name="definition" select="replace($contextBBIEdetails[1]/match[1]/@definition[1],'&#xA;','')"/></xsl:if>
                                                <xsl:if test="string-length($contextBBIEdetails[1]/match[1]/@anno[1]) &gt; 0"><xsl:attribute name="anno" select="$contextBBIEdetails[1]/match[1]/@anno[1]"/></xsl:if>
                                            </xsl:when>
                                            <xsl:otherwise><!-- Pick first details in the list -->
                                                <xsl:if test="$stripChildCardinality='false'">
                                                    <xsl:if test="contains($BBIEdetails[1]/@cardinality[1],'repeatable')"><xsl:attribute name="repeatable">true</xsl:attribute></xsl:if>
                                                    <xsl:if test="contains($BBIEdetails[1]/@cardinality[1],'optional')"><xsl:attribute name="optional">true</xsl:attribute></xsl:if>
                                                </xsl:if>
                                                <xsl:if test="string-length($BBIEdetails[1]/@UID[1]) &gt; 0"><xsl:attribute name="UID" select="$BBIEdetails[1]/@UID[1]"/></xsl:if>
                                                <xsl:if test="string-length($BBIEdetails[1]/@type[1]) &gt; 0"><xsl:attribute name="type" select="$BBIEdetails[1]/@type[1]"/></xsl:if>
                                                <xsl:if test="string-length($BBIEdetails[1]/@label[1]) &gt; 0"><xsl:attribute name="label" select="replace($BBIEdetails[1]/@label[1],'&#xA;','')"/></xsl:if>                                      
                                                <xsl:if test="string-length($BBIEdetails[1]/@definition[1]) &gt; 0"><xsl:attribute name="definition" select="replace($BBIEdetails[1]/@definition[1],'&#xA;','')"/></xsl:if>
                                                <xsl:if test="string-length($BBIEdetails[1]/@anno[1]) &gt; 0"><xsl:attribute name="anno" select="$BBIEdetails[1]/@anno[1]"/></xsl:if>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:element>
                                </xsl:when>
                                <xsl:when test="$dictionary//Property[@CCTS='ABIE'][@item=$this][@context=$singleContext]">
                                    <xsl:variable name="newParent"  select="$dictionary//Property[@CCTS='ABIE'][@item=$this][@context=$singleContext]"/>
                                    <xsl:choose>
                                        <xsl:when test="contains($itemPath,concat('|',$this,'|'))"><!-- Already in parent, so prevent looping -->
                                            <xsl:message> - recursive parent - <xsl:value-of select="$this"/></xsl:message>
                                            <xsl:element name="Parent">
                                                <xsl:attribute name="name" select="$this"/>
                                                <!-- xsl:attribute name="recursive" select="'true'"/>
                                                <xsl:attribute name="definition" select="'recursive parent'"/ -->
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:if test="$debug = 1"><xsl:message>Parent: <xsl:value-of select="$this"/> - <xsl:value-of select="$thisParent"/></xsl:message></xsl:if>
                                            <xsl:message> - <xsl:value-of select="$thisParent"/> - <xsl:value-of select="$this"/></xsl:message>
                                            <xsl:apply-templates select="$newParent[1]" mode="CAM">
                                                <xsl:with-param name="shortname" select="$shortname"/>
                                                <xsl:with-param name="dictionary" select="$dictionary"/>
                                                <xsl:with-param name="itemPath" select="concat($itemPath,$this,'|')"/>
                                                <xsl:with-param name="parent" select="$thisParent"/>
                                                <xsl:with-param name="noDetailsChk" select="$noDetailsChk"/>
                                            </xsl:apply-templates>
                                        </xsl:otherwise>
                                    </xsl:choose>  
                                </xsl:when>
                                <xsl:when test="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$context)]">
                                    <xsl:variable name="newParent"  select="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$context)]"/>
                                    <xsl:choose>
                                        <xsl:when test="contains($itemPath,concat('|',$this,'|'))"><!-- Already in parent, so prevent looping -->
                                            <xsl:message> - recursive parent <xsl:value-of select="$this"/> - <xsl:value-of select="$itemPath"/></xsl:message>
                                            <xsl:element name="Parent">
                                                <xsl:attribute name="name" select="$this"/>
                                                <!-- xsl:attribute name="recursive" select="'true'"/>
                                                <xsl:attribute name="definition" select="'recursive parent'"/ -->
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:if test="$debug = 1"><xsl:message>- Parents: <xsl:value-of select="$this"/> - <xsl:value-of select="$thisParent"/></xsl:message></xsl:if>
                                            <xsl:apply-templates select="$newParent[1]" mode="CAM">
                                                <xsl:with-param name="shortname" select="$shortname"/>
                                                <xsl:with-param name="dictionary" select="$dictionary"/>
                                                <xsl:with-param name="itemPath" select="concat($itemPath,$this,'|')"/>
                                                <xsl:with-param name="parent" select="$thisParent"/>
                                                 <xsl:with-param name="noDetailsChk" select="$noDetailsChk"/>
                                            </xsl:apply-templates>
                                        </xsl:otherwise>
                                    </xsl:choose>  
                                </xsl:when>
                                <xsl:when test="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$parentContext)]">
                                    <xsl:variable name="newParent"  select="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$parentContext)]"/>
                                    <xsl:choose>
                                        <xsl:when test="contains($itemPath,concat('|',$this,'|'))"><!-- Already in parent, so prevent looping -->
                                            <xsl:element name="Parent">
                                                <xsl:attribute name="name" select="$this"/>
                                                <!-- xsl:attribute name="recursive" select="'true'"/>
                                                <xsl:attribute name="definition" select="'recursive parent loop reference'"/ -->
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:if test="$debug = 1"><xsl:message>- SingleParent: <xsl:value-of select="$this"/> - <xsl:value-of select="$singleContext"/></xsl:message></xsl:if>
                                            <xsl:apply-templates select="$newParent[1]" mode="CAM">
                                                <xsl:with-param name="shortname" select="$shortname"/>
                                                <xsl:with-param name="dictionary" select="$dictionary"/>
                                                <xsl:with-param name="itemPath" select="concat($itemPath,$this,'|')"/>
                                                <xsl:with-param name="parent" select="$singleContext"/>
                                                 <xsl:with-param name="noDetailsChk" select="$noDetailsChk"/>
                                            </xsl:apply-templates>
                                        </xsl:otherwise>
                                    </xsl:choose>  
                                </xsl:when>
                                <xsl:when test="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$fullContext)]">
                                    <xsl:variable name="newParent"  select="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$fullContext)]"/>
                                    <xsl:choose>
                                        <xsl:when test="contains($itemPath,concat('|',$this,'|'))"><!-- Already in parent, so prevent looping -->
                                            <xsl:element name="Parent">
                                                <xsl:attribute name="name" select="$this"/>
                                                <!-- xsl:attribute name="recursive" select="'true'"/>
                                                <xsl:attribute name="definition" select="'recursive parent loop'"/ -->
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:if test="$debug = 1"><xsl:message>- ParentParent: <xsl:value-of select="$this"/> - <xsl:value-of select="$parentContext"/></xsl:message></xsl:if>
                                            <xsl:apply-templates select="$newParent[1]" mode="CAM">
                                                <xsl:with-param name="shortname" select="$shortname"/>
                                                <xsl:with-param name="dictionary" select="$dictionary"/>
                                                <xsl:with-param name="itemPath" select="concat($itemPath,$this,'|')"/>
                                                <xsl:with-param name="parent" select="$parentContext"/>
                                                 <xsl:with-param name="noDetailsChk" select="$noDetailsChk"/>
                                            </xsl:apply-templates>
                                        </xsl:otherwise>
                                    </xsl:choose>  
                                </xsl:when>
                                <xsl:when test="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$itemContext)]">
                                    <xsl:variable name="newParent"  select="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$itemContext)]"/>
                                    <xsl:choose>
                                        <xsl:when test="contains($itemPath,concat('|',$this,'|'))"><!-- Already in parent, so prevent looping -->
                                            <xsl:element name="Parent">
                                                <xsl:attribute name="name" select="$this"/>
                                                <!-- xsl:attribute name="recursive" select="'true'"/>
                                                <xsl:attribute name="definition" select="'recursive parent loop'"/ -->
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:if test="$debug = 1"><xsl:message>- ItemParent: <xsl:value-of select="$this"/> - <xsl:value-of select="$parentContext"/></xsl:message></xsl:if>
                                            <xsl:apply-templates select="$newParent[1]" mode="CAM">
                                                <xsl:with-param name="shortname" select="$shortname"/>
                                                <xsl:with-param name="dictionary" select="$dictionary"/>
                                                <xsl:with-param name="itemPath" select="concat($itemPath,$this,'|')"/>
                                                <xsl:with-param name="parent" select="$parentContext"/>
                                                 <xsl:with-param name="noDetailsChk" select="$noDetailsChk"/>
                                            </xsl:apply-templates>
                                        </xsl:otherwise>
                                    </xsl:choose>  
                                </xsl:when>
                                <xsl:when test="$dictionary//Property[@CCTS='ABIE'][@item=$this][contains(@context,$moreContext)]">
                                    <xsl:variable name="newParentMore"  select="$dictionary//Property[@CCTS='ABIE'][@item=$this][contains(@context,$moreContext)]"/>
                                    <xsl:choose>
                                        <xsl:when test="contains($itemPath,concat('|',$this,'|'))"><!-- Already in parent, so prevent looping -->
                                            <xsl:element name="Parent">
                                                <xsl:attribute name="name" select="$this"/>
                                                <!-- xsl:attribute name="recursive" select="'true'"/>
                                                <xsl:attribute name="definition" select="'recursive parent loop'"/ -->
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:when test="contains($itemPath,concat('|',$parent,'|'))"><!-- Already in parent, so prevent looping -->
                                            <xsl:element name="Parent">
                                                <xsl:attribute name="name" select="$this"/>
                                                <!-- xsl:attribute name="recursive" select="'true'"/>
                                                <xsl:attribute name="definition" select="'recursive loop parent'"/ -->
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:if test="$debug = 1"><xsl:message>- MoreParent: <xsl:value-of select="$this"/> - <xsl:value-of select="$moreContext"/> <!-- - <xsl:value-of select="$itemPath"/> - parent: <xsl:value-of select="$parent"/> --></xsl:message></xsl:if>
                                            <xsl:apply-templates select="$newParentMore[1]" mode="CAM">
                                                <xsl:with-param name="shortname" select="$shortname"/>
                                                <xsl:with-param name="dictionary" select="$dictionary"/>
                                                <xsl:with-param name="itemPath" select="concat($itemPath,$this,'|')"/>
                                                <xsl:with-param name="parent" select="$thisParent"/>
                                                 <xsl:with-param name="noDetailsChk" select="$noDetailsChk"/>
                                            </xsl:apply-templates>
                                        </xsl:otherwise>
                                    </xsl:choose>  
                                </xsl:when>
                                <xsl:when test="$dictionary//Property[@CCTS='ABIE'][@item=$this] and($parent ='')"><!-- If at root level, then parent is null -->
                                    <xsl:if test="$debug = 1"><xsl:message>- RootLevelParent: <xsl:value-of select="$this"/> - <xsl:value-of select="$parentContext"/></xsl:message></xsl:if>
                                    <xsl:variable name="newParent"  select="$dictionary//Property[@CCTS='ABIE'][@item=$this]"/>
                                    <xsl:apply-templates select="$newParent[1]" mode="CAM">
                                        <xsl:with-param name="shortname" select="$shortname"/>
                                        <xsl:with-param name="dictionary" select="$dictionary"/>
                                        <xsl:with-param name="itemPath" select="concat($itemPath,$this,'|')"/>
                                        <xsl:with-param name="parent" select="$parentContext"/>
                                        <xsl:with-param name="noDetailsChk" select="$noDetailsChk"/>
                                    </xsl:apply-templates>
                                </xsl:when>
                                <xsl:when test="$dictionary//Property[@CCTS='ABIE'][@item=$this]">
                                    <xsl:message>Error: <xsl:value-of select="$this"/> - <xsl:value-of select="$itemContext"/> - <xsl:value-of select="$itemPath"/></xsl:message>
                                    <xsl:element name="Parent">
                                        <xsl:attribute name="name" select="$this"/>
                                        <xsl:attribute name="ERROR" select="concat($context,' - ',$itemContext)"/>
                                    </xsl:element>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:element name="Child">
                                        <xsl:attribute name="name" select="replace($this,'simpleType','')"/>
                                        <!-- Lookup cardinality and UID -->
                                        <xsl:variable name="this" select="replace(.,'simpleType','')"/>
                                        <xsl:variable name="BBIEdetails" select="$dictionary//Property[@CCTS='BBIE'][@item=$this]"/>
                                        <xsl:if test="contains($BBIEdetails/@cardinality[1],'repeatable')"><xsl:attribute name="repeatable">true</xsl:attribute></xsl:if>
                                        <xsl:if test="contains($BBIEdetails/@cardinality[1],'optional')"><xsl:attribute name="optional">true</xsl:attribute></xsl:if>
                                        <xsl:if test="string-length($BBIEdetails/@UID[1]) &gt; 0"><xsl:attribute name="UID" select="$BBIEdetails/@UID[1]"/></xsl:if>
                                        <xsl:if test="string-length($BBIEdetails/@label[1]) &gt; 0"><xsl:attribute name="label" select="replace($BBIEdetails/@label[1],'&#xA;','')"/></xsl:if>                                      
                                        <xsl:if test="string-length($BBIEdetails/@definition[1]) &gt; 0"><xsl:attribute name="definition" select="replace($BBIEdetails/@definition[1],'&#xA;','')"/></xsl:if>
                                        <xsl:if test="string-length($BBIEdetails/@anno[1]) &gt; 0"><xsl:attribute name="anno" select="$BBIEdetails/@anno[1]"/></xsl:if>
                                    </xsl:element>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                          <xsl:otherwise/><!-- ignore this item -->
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose>
        
            <!-- xsl:apply-templates select="@*" mode="CAM"/>
            <xsl:apply-templates select="*" mode="CAM"/>
            <xsl:apply-templates select="text()[1]" mode="CAM"/ -->
    </xsl:template>
    
    <xsl:template match="@*" mode="CAM">
       <xsl:message>Attribute <xsl:value-of select="name()"/></xsl:message>
    </xsl:template>
    
</xsl:stylesheet>
