<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:as="http://www.oasis-open.org/committees/cam" 
    xmlns:camed="http://jcam.org.uk/editor"
    xmlns:w2e="http://thisSheet"
    xmlns:look="http://jcam.org.uk/LookupLists"
    xmlns:dblook="http://jcam.org.uk/DbLookupLists"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:random="http://exslt.org/random" exclude-result-prefixes="w2e random">
    <!-- CAM kit utility program -->
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Authors: David RR Webber, May, 2008 -->
    <!--
        Purpose  : Parse CAM template internal format (CXF) and generate XSD schema.
        Creates XSD instance to be used as target for WSDL and code generator tools.
        Original : 05/14/2008
        Initials : drrw
        CAM ver    : 1.1 
        Description : Part of a set of tools to ease alignment of XSD based validations and documentation
        and CAM templates. Generates an XSD instance in W3C schema format
        Updated  - Inits - Change
        05/14/08   drrw   Initial release
        05/15/08   drrw   Add processing of namespace imports and definitions
        06/07/08   drrw   Enhance type mapping
        06/07/08   drrw  Add choice logic
        06/07/08   drrw  Add exclude handling
        06/07/08   drrw  Add type enumeration logic
        06/07/08   drrw  Add namespace declaring and import statements
        06/08/08   drrw  Resolve syntax for enumeration on element content
        06/10/08   drrw  Refactor findtype lookup 
        06/10/08   drrw  Boolean type for Valid|Invalid restrictions
        06/11/08   drrw  Element generation for namespace types
        06/11/08   drrw  Imports and namespaces in each import schema
        06/11/08   drrw  Element references to imported types
        06/12/08   drrw  Refactor findtype processing
        06/12/08   drrw  Complextype default extension base refactoring
        06/12/08   drrw  Update attribute enumerations to have all possible values
        06/13/08   drrw  Position of choice() within element list
        06/13/08   drrw  Multiple choices within one element
        06/13/08   drrw  Version attribute to match CAM version approach
        06/16/08   drrw  Require namespace in XSD parameter for elementFormDefault
        06/16/08   drrw  Create parent import package for child elements on change of namespace prefix
        06/16/08   drrw  Add second pass scan to write complexType definitions for children of parents into imports with base extensions
        06/16/08   drrw  Attributes on elements
        06/17/08   drrw  Refine outputting of import definitions to use parent groups too
        06/17/08   drrw  Attribute type referencing / inline in model mode
        06/18/08   drrw  Implement what goes where for element definitions
        06/18/08   drrw  Use references to elements instead of types where ns is in import
        06/19/08   drrw  Resolve exclude() logic for attributes
        06/19/08   drrw  Resolve exclude() logic for import elements
        06/20/08   drrw  Check for exclude() and stop if found
        06/21/08   drrw  Add support for setLength()
        06/25/08   drrw  version reporting and namespace parameter handling
        06/25/08   drrw  choice() logic and pack definition import
        07/21/08   drrw  namespace handling for new CXF with linefeed
        07/27/08   drrw  fix duplication of definitions of repeated referenced type items
        08/07/08   drrw  Support xsd:totalDigits for numeric masks
        08/08/08   drrw  Output xsd for digits
        09/17/08   drrw  Improve structure looping memory use by eliminated duplicates processing
        09/17/08   drrw  exclude xml namespace prefix
        10/01/08   drrw  Make imports use relative paths
        10/01/08   drrw  Include annotations in XSD
        10/03/08   drrw  Missing xsd:Element when not up or down complexType
        10/03/08   drrw  Duplicate declaration of complexType
        10/08/08   drrw   0 in number mask; assumes repeat operators are 1-9 i.e. not 10,20,30.
        10/16/08   drrw  Number range outputting to xsd restriction
        11/04/08   drrw  setStringMask to xsd:pattern 
        12/11/08   drrw  Add support for fixed values
        01/03/09   drrw  Multiple choice() groups skipped after first
        01/16/09   drrw  Ignore non-xsd prefixed attributes in xsd:element definition
        01/16/09   drrw  xsd:decimal not xsd:float for restriction totalDigits or fractionDigits
        03/26/09   drrw  Add parameter to allow output to different XSD styles and types
        03/29/09   mmer  Removed default value from xsdOutput and printed it out
        04/01/09   drrw  Fix generation of attributes on parent with no child elements
        05/06/09   drrw  Fix generation of attributes of root node elements
        05/06/09   drrw  For elements with attribute collections only - create complexType with simpleContent
        05/06/09   drrw  Output union of datatypes as simpleTypes
        05/06/09   drrw  Prevent duplicate name definitions
        05/08/09   drrw  Make simpleTypes globally defined
        05/11/09   drrw  Handle namespaces with simpleTypes
        05/11/09   drrw  Output option for NIEM exchange schema header notice text
        05/12/09   drrw  Better output file handling and memory management
        05/15/09   drrw  Resolve duplicate definitions and improve change of namespace handling
        05/15/09   drrw  Handling of datatype(any)
        05/15/09   drrw  Definition of union items
        05/16/09   drrw  Missing ##any definition of duplicate element type
        05/18/09   drrw  Switch reference to name / type for datatype(any)
        05/18/09   drrw  Fix outputting of simpleTypes to also do complexTypes for datatype(any) items
        05/19/09   drrw  Add appinfo tag of root element output to main schema
        05/23/09   drrw  Fix duplicate definition issue for "look down" items
        09/09/09   drrw  When root element has namespace - put reference definition in element declaration
        09/09/09   drrw  Handle complex structure constructs - as child of root node with namespace 
        09/09/09   drrw  Singleton simpleType elements failing duplicate check on packed definition
        09/10/09   drrw  Support XMLBeans style code list as simpleType 
        09/11/09   drrw  Fix issue with base complexType declarations and simpleType conflict
        09/11/09   drrw  When root element has namespace not everything can be a ref, some things are element with name and type
        09/11/09   drrw  Circular definition occurring for certain complexType on down change 
        09/16/09   drrw  Up same prefix element output
        09/18/09   drrw  Implement WSDL mode output
        01/06/10   drrw  Support xsd folder structure output mapping by namespace
        03/25/10   drrw  Tweak folder path outputting - relative path import cannot be used for actual file output path 
        03/29/10   drrw   More folder path generation tweaks
        03/31/10   drrw   Tidy up debugging comments
        06/05/10   drrw   Fix setNumberMask for 0000 and datatype(boolean) handling
        06/05/10   drrw   Attribute generation on root element child xpath
        06/05/10   drrw   Relative path handling -for includes within includes it uses the primary path for that namespace
        12/11/10   drrw   Attribute generation of nested elements in imported components
        01/13/11   drrw   Add support for lookup list enumerations
        01/14/11   drrw   Output of fixed attribute - add normalization of space
        01/25/11   drrw   Output tracking messages to console for schema components
        01/25/11   drrw   Performance enhancement for very large templates of components
        01/31/11   drrw   Consolidate element/attribute looping to reduce memory footprint
        
        02/20/11   drrw   Improve handling of annotation linefeeds - with trim() of whitespace
        02/20/11   drrw   Insert missing attributes on leaf elements
        02/20/11   drrw   Generation of minOccurs=0 not on top level ref= elements
        03/24/11   drrw   Generate UNIX style file path separators
        
        03/30/11   drrw   Element name blank - substring-after(x,':') issue
        03/30/11   drrw   Filepath offsets missing parent folder name
        03/30/11   drrw   Duplicate declarations
 
        06/10/11   drrw   Fix attribute outputting on root and empty elements
        06/11/11   drrw   Fix XSD generation minOccurs=0 not on top level ref= elements
        06/12/11   drrw   Fix when parent element is datatype(any)
        06/12/11   drrw   Fix duplicate declaration issues for like named items in different namespaces
        06/12/11   drrw   Fix duplicate simple and complexType declarations for same item      
        06/13/11  drrw    Fix target namespace pattern for ns on root same as children issue   
        06/13/11   drrw   Move NIEM IEPD structure folders under domains folder        
        06/14/11   drrw   Import path offset calculations for child imports in imports - no change, now appears to work
        06/10/11   drrw   Fix ref with simpleType bug
        06/09/11   drrw   Improve output of xsd:pattern regexp
        02/23/12   drrw   Output JAXB binding file
        03/30/12   drrw   Fix for xsd add xmlns import path relative path logic
        03/30/12   drrw   datatype(hexBinary) output support
        04/17/12   drrw   Retained filename by namespace use
        04/17/12   drrw   Improve JAXB binding package details - dot syntax and reverse URL
        04/18/12   drrw   Prevent generation of empty element names with no namespace prefix in complexType
        04/18/12   drrw   When no namespace make local declaration enumeration restrictValues list instead of global
        04/18/12   drrw   When number range includes decimal places and signs - then should generate decimal and min / max schema assertions
        04/22/12   drrw   SimpleType output bug for mixed type - added evaluation report rule to detect this
        04/23/12   drrw   JAXB duplicate name reporting not matching actual XPath
        04/23/12   drrw   JAXB namespace exclusion and package name generation changes
        04/23/12   drrw   Suppress look and dblook namespace imports and also empty imports
        04/23/12   drrw   xsd:date for setDateMask('YYYY-MM-DD') matching
        04/24/12   drrw   Change JAXB to make duplicate components names as comments
        04/24/12   drrw   Add ConTessa support annotations to schema
        02/04/13   drrw   Add appinfo annotation support
        03/11/13   drrw   Add restrictValues enumeration annotations to xsd enumerations lists
        03/12/13   drrw   Fix datatype() outputting to matching xsd: types - e.g. double, float, anyURI etc
        03/21/13   drrw   Update appinfo handling for URN
       
    -->
    <xsl:param name="fileNamePath">new-schema</xsl:param>
    <!-- name stub and filepath location for the import schemas to be written to -->
    <xsl:param name="requireNS">true</xsl:param>
    <xsl:param name="xsdOutput">default</xsl:param><!-- default, typed, NIEM -->
    
    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
    <xsl:output name="exchangeXSD" method="xml" indent="yes" omit-xml-declaration="yes"/>
    <xsl:output name="importXSD" method="xml" indent="yes" omit-xml-declaration="yes"/>
    
    <xsl:variable name="version">1.63</xsl:variable>    
    <xsl:variable name="debug">0</xsl:variable>
    <xsl:variable name="dateTimeMask">setDateMask(YYYY-MM-DD'T'HH:MI:SSZ)</xsl:variable>
    <xsl:variable name="date">setDateMask(YYYY-MM-DDZ)</xsl:variable>
    <xsl:variable name="date1">setDateMask(YYYY-MM-DD)</xsl:variable>
    <xsl:variable name="year">setDateMask(YYYY)</xsl:variable>
    <xsl:variable name="yearM">setDateMask(YYYY-MM)</xsl:variable>
    <xsl:variable name="month">setDateMask(--MM)</xsl:variable>
    <xsl:variable name="monthD">setDateMask(--MM-DD)</xsl:variable>
    <xsl:variable name="day">setDateMask(---DD)</xsl:variable>
    <xsl:variable name="time">setDateMask(HH:MI:SS.SZ)</xsl:variable>
    <xsl:variable name="idRef">datatype(IDREF)</xsl:variable>
    <xsl:variable name="idRefs">datatype(IDREFS)</xsl:variable>
    <xsl:variable name="id">datatype(ID)</xsl:variable>
    <xsl:variable name="anyType">datatype(any)</xsl:variable>
    <xsl:variable name="duration">datatype(duration)</xsl:variable>
    <xsl:variable name="token">datatype(token)</xsl:variable>
    <xsl:variable name="boolean">restrictValues('true'|'false')</xsl:variable>
    <xsl:variable name="decimal">setNumberMask(######.##)</xsl:variable>
    <xsl:variable name="integer">setNumberMask(######)</xsl:variable>
    <xsl:variable name="int">setNumberMask(###)</xsl:variable>
    <xsl:variable name="isNumeric">123456789</xsl:variable>
    <xsl:variable name="period">.</xsl:variable>
    <xsl:variable name="slash">/</xsl:variable>
    <xsl:variable name="ignoreNS">|as|w2e|xsd|xs|camed|xmi|xml|look|dblook|</xsl:variable>
    
    <xsl:variable name="rootSuffix">-RootType</xsl:variable><!-- used for schema with namespace on root node -->
    
    <xsl:variable name="filenameStub">
        <xsl:choose>
            <xsl:when test="contains($fileNamePath,'\')"><xsl:value-of select="reverse(tokenize($fileNamePath,'\\'))[1]"></xsl:value-of></xsl:when>
            <xsl:when test="contains($fileNamePath,'/')"><xsl:value-of select="reverse(tokenize($fileNamePath,'/'))[1]"></xsl:value-of></xsl:when>
            <xsl:when test="$fileNamePath=''"><xsl:value-of select="//as:AssemblyStructure/as:Structure/@ID"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$fileNamePath"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="baseNS"
        select="substring-before(//as:AssemblyStructure/as:Structure/as:Element/@name,':')"/>
    <xsl:variable name="seed"
        select="random:random-sequence(1,seconds-from-dateTime(current-dateTime()))"/>
    <xsl:variable name="source" select="/"/>
    <xsl:variable name="namespaces" select="//as:Namespaces"/>
    <xsl:variable name="loadNamespaces">
        <xsl:element name="this">
            <xsl:for-each select="$namespaces/as:namespace">
                <xsl:variable name="urn" select="if(starts-with(.,'&#xA;')) then substring-after(.,'&#xA;') else ."/>     
                <xsl:namespace name="{@prefix}">
                    <xsl:value-of select="$urn"/>
                </xsl:namespace>
            </xsl:for-each>
        </xsl:element>
    </xsl:variable>
    
    <xsl:include href="mapping-ns-utils.xsl"/> 
    
    <xsl:variable name="ns-maps" select="camed:InsertMappings()"/>

    <xsl:variable name="nsContentModels">
        <xsl:message> - Create components for each ns model</xsl:message>
        <xsl:for-each select="//as:AssemblyStructure/as:Structure/as:Element">
            <xsl:apply-templates select="." mode="nsModels"/>
        </xsl:for-each>
    </xsl:variable>
    
    <xsl:variable name="duplicateElements">
        <xsl:for-each select="//as:Element">
            <xsl:variable name="this" select="@name"/>
            <xsl:if test="not(contains($this,':')) and(count(descendant::as:Element|as:Attribute) &gt; 1) and(count(//as:Element[@name=$this]) &gt; 1)">
                <xsl:variable name="chkSame">
                <xsl:for-each select="//as:Element[@name=$this]" exclude-result-prefixes="#all">
                    <xsl:if test="count(as:Element) &gt; 0">
                        <chk name="{@name}" child="{count(as:Element)}"></chk>
                    </xsl:if>                    
                </xsl:for-each>
                </xsl:variable>
                <xsl:if test="count($chkSame/chk) &gt; 1">
                    <dup name="{@name}" path="{@path}"/>
                </xsl:if>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>
    
    <xsl:variable name="defaultExchangePath" select="camed:mapNS-2-path('','Exchange',$xsdOutput,$ns-maps)"/>
    
    <xsl:variable name="lookupTables" select="/as:CAM/as:Extension/*"/>
    
    <xsl:variable name="nsCount" select="count(//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))])"/>
    
    <xsl:variable name="defsElements">
        <xsl:message>Cataloguing elements by namespace</xsl:message>
    <xsl:for-each
        select="//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))]">
        <xsl:message> -- <xsl:value-of select="@prefix"/></xsl:message>
        <xsl:variable name="ns" select="@prefix"/>
        <xsl:variable name="nsURI" select="if(starts-with(text(),'&#xA;')) then substring-after(text(),'&#xA;') else text()"/>
       
            <xsl:for-each select="$source//as:Element[starts-with(@name,concat($ns,':'))]">
                <xsl:copy-of select="."/>
                <xsl:if test="./parent::as:Element/as:Attribute"><!-- if parent has attributes, then we need to check their namespaces -->
                    <xsl:element name="as:Element">
                        <xsl:attribute name="name"><xsl:value-of select="./parent::as:Element/@name"/></xsl:attribute>
                        <xsl:attribute name="path"><xsl:value-of select="./parent::as:Element/@path"/></xsl:attribute>
                    </xsl:element>
                </xsl:if>
            </xsl:for-each>
        </xsl:for-each>
    </xsl:variable>
    
    <xsl:variable name="declElemAll" select="distinct-values($defsElements/as:Element/@name)"/>
       
    <xsl:variable name="defsAttribs">
        <xsl:message>Cataloguing attributes by namespace</xsl:message>
        <xsl:for-each
            select="//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))]">
            <xsl:message> -- <xsl:value-of select="@prefix"/></xsl:message>
            <xsl:variable name="ns" select="@prefix"/>
        <xsl:for-each select="$source//as:Attribute[starts-with(@name,concat($ns,':'))]">
            <xsl:copy-of select="."/>
        </xsl:for-each>
        </xsl:for-each>
    </xsl:variable>
       
        <xsl:variable name="declAttrib" select="distinct-values($defsAttribs/as:Attribute/@name)"/>      

    <xsl:template match="/">
        <xsl:message>CAM 2 XSD schema: version <xsl:value-of select="$version"/></xsl:message>
        <xsl:message>FilePath: <xsl:value-of select="$fileNamePath"/></xsl:message>
        <xsl:message>XSD mode: <xsl:value-of select="$xsdOutput"/></xsl:message>
        <xsl:message>Namespaces: <xsl:value-of select="$requireNS"/></xsl:message>

        <!-- Compute actual output path to write to --> 
        <xsl:variable name="ns-resultNamePath" select="camed:resolve-ns-outputpath($fileNamePath,$filenameStub,$defaultExchangePath,'')"/>
        <xsl:variable name="writeResultPath" select="concat($ns-resultNamePath,'.xsd')"/>
        <xsl:message>Writing schema to: <xsl:value-of select="$writeResultPath"/></xsl:message>
        
        <!-- xsl:message> Duplicates [<xsl:copy-of select="$duplicateElements"></xsl:copy-of>]</xsl:message -->
        <!-- xsl:message>[<xsl:value-of select="$filenameStub"/>][<xsl:value-of select="$fileNamePath"/>]</xsl:message -->
        <!-- xsl:message>Content models: <xsl:copy-of select="$nsContentModels"/></xsl:message -->
        <!-- xsl:message>rules: <xsl:copy-of select="$ns-maps"/></xsl:message -->
        <xsl:if test="count(//as:AssemblyStructure/as:Structure/as:Element)=0">
            <xsl:comment>Input file is not valid CAM internal format (.cxf)</xsl:comment> 
            <xsl:message terminate="yes">Input file is not valid CAM internal format (.cxf)</xsl:message>
        </xsl:if>
        <xsl:if test="count(//as:AssemblyStructure/as:Structure/as:Element/as:Rule/as:constraint[contains(substring-before(@action,'('),'exclude')]) &gt; 0">
            <xsl:comment>Please compress template before generating XSD (.cxf)</xsl:comment>
            <xsl:message terminate="yes">Please compress template before generating XSD (.cxf)</xsl:message>
        </xsl:if>
        
        <xsl:message> <xsl:value-of select="count($defsElements//as:Element)"/> elements found</xsl:message> 
        <xsl:message> <xsl:value-of select="count($declElemAll)"/> unique elements</xsl:message>
        <xsl:message> <xsl:value-of select="count($defsAttribs//as:Attribute)"/> attributes found</xsl:message>
        <xsl:message> <xsl:value-of select="count($declAttrib)"/> unique attributes</xsl:message>
        
        <xsl:variable name="that" select="random:random-sequence(1,$seed)"/>
        <xsl:variable name="rootElement" select="//as:AssemblyStructure/as:Structure/as:Element/@name"/>
        <xsl:variable name="rootPrefix" select="substring-before($rootElement,':')"/>
        <xsl:choose>
            <xsl:when test="$xsdOutput='WSDL'">
                <wsdl:definitions 
                    xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
                    xmlns:soap="http://schemas.xmlsoap.org/wsdl/soap/" 
                    xmlns:xs="http://www.w3.org/2001/XMLSchema"
                    xsl:exclude-result-prefixes="as camed dblook look"
                   >
                    <xsl:variable name="baseExchangeNS">
                        <xsl:choose>
                            <xsl:when test="//as:namespace[contains(.,'niem')]">http://jcam.org.uk/exchange/wsdl/NIEM</xsl:when>
                            <xsl:otherwise>http://jcam.org.uk/exchange/wsdl</xsl:otherwise>
                        </xsl:choose>  
                    </xsl:variable>                    
                    <xsl:namespace name="schema"><xsl:value-of select="$baseExchangeNS"/></xsl:namespace>
                    <xsl:attribute name="targetNamespace"><xsl:value-of select="$baseExchangeNS"/></xsl:attribute>
                    
                    <xsl:variable name="importNS">
                        <xsl:choose>
                            <xsl:when test="contains($rootElement,':')"><xsl:value-of select="as:CAM/as:Namespaces/as:namespace[@prefix=$rootPrefix]"/></xsl:when>
                            <xsl:otherwise></xsl:otherwise>
                        </xsl:choose>                        
                    </xsl:variable>
                    <xsl:variable name="importRootElement" select="if(contains($rootElement,':')) then substring-after($rootElement,':') else $rootElement"></xsl:variable>
                    
                    <xsl:comment>
                        WSDL autogenerated by CAM-2-xsd 
                        Description: <xsl:value-of select="//as:Header/as:Description"/> 
                        Owner: <xsl:value-of select="//as:Header/as:Owner"/> 
                        Date: <xsl:value-of select="//as:Header/as:DateTime"/> 
                        Version: <xsl:value-of select="//as:Header/as:Version"/> 
                        Script: v<xsl:value-of select="$version"/> run
                    </xsl:comment>
                    
                    <xsl:comment>
                        
                        This WSDL is for:  <xsl:value-of select="$rootElement"/> 
                        
                        Request root element set to - <xsl:value-of select="$importRootElement"/>
                        
                        EDITING:
                            
                            Globally replace NameOfResponseRootElement with response root element name
                            
                            Add second import for response message schema
                            
                            Change soap:address at bottom of wsdl to reflect actual server port address

                    </xsl:comment>
                                        
                   <wsdl:types>
                       <xsl:element name="xs:import">
                           <xsl:if test="string-length($importNS) &gt; 0"><xsl:attribute name="namespace"><xsl:value-of select="$importNS"/></xsl:attribute></xsl:if>
                           <xsl:attribute name="schemaLocation" select="replace(concat($filenameStub,'.xsd'),'.wsdl','')"/>
                       </xsl:element>
                      
                       <xsl:comment>
                           
                           EDIT: add your second import here for response schema 
                           
                       </xsl:comment>
     
                   </wsdl:types>

                    <wsdl:message name="SOAP_Request">
                        <wsdl:part element="schema:{$importRootElement}" name="RequestDetail"/><xsl:comment>CHECK: this should match actual request one</xsl:comment>
                    </wsdl:message>
                    <wsdl:message name="SOAP_Response">
                        <wsdl:part element="schema:NameOfResponseRootElement" name="ResponseDetail"/><xsl:comment>EDIT: placeholder message part here, so change to actual response</xsl:comment>
                    </wsdl:message>
                    <wsdl:portType name="Base_Services">
                        <wsdl:operation name="ServiceDetail">
                            <wsdl:input message="schema:{$importRootElement}" name="{$importRootElement}"/>
                            <wsdl:output message="schema:NameOfResponseRootElement" name="NameOfResponseRootElement"/>
                        </wsdl:operation>
                    </wsdl:portType>
                    <wsdl:binding name="SOAPbinding" type="schema:BaseType">
                        <soap:binding style="document" transport="http://schemas.xmlsoap.org/soap/http"/>
                        <wsdl:operation name="ActionDetails">
                            <soap:operation soapAction=""/>
                            <wsdl:input name="{$importRootElement}"><xsl:comment>CHECK: should match to actual query name</xsl:comment>
                                <soap:body use="literal"/>
                            </wsdl:input>
                            <wsdl:output name="NameOfResponseRootElement"><xsl:comment>EDIT: change to match actual response name</xsl:comment>
                                <soap:body use="literal"/>
                            </wsdl:output>
                        </wsdl:operation>
                    </wsdl:binding>
                    <wsdl:service name="BaseService">
                        <wsdl:port binding="schema:SOAPbinding" name="defaultPort">
                            <soap:address location="http://mySOAPserver:8080/services"/><xsl:comment>EDIT: change this to your actual service address</xsl:comment>
                        </wsdl:port>
                    </wsdl:service>
                    
                </wsdl:definitions>
                
            </xsl:when>
            <xsl:when test="$xsdOutput='JAXB binding'"><!-- Create JAXB binding file -->
                
                <jxb:bindings 
                    version="2.0"
                    xmlns:jxb="http://java.sun.com/xml/ns/jaxb"
                    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                    xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                    xmlns:xjc="http://java.sun.com/xml/ns/jaxb/xjc"
                    xsl:exclude-result-prefixes="as camed look dblook" ><!-- removed  xsi:schemaLocation="http://java.sun.com/xml/ns/jaxb bindingschema_2_0.xsd"  -->
                    
                    <xsl:comment>
                        JAXB binding autogenerated by CAM-2-xsd 
                        This binding is for:  <xsl:value-of select="$rootElement"/> 
                        Description: <xsl:value-of select="//as:Header/as:Description"/> 
                        Owner: <xsl:value-of select="//as:Header/as:Owner"/> 
                        Date: <xsl:value-of select="//as:Header/as:DateTime"/> 
                        Version: <xsl:value-of select="//as:Header/as:Version"/> 
                        Script: v<xsl:value-of select="$version"/> used
                    </xsl:comment>
                    
                    <!-- jxb:globalBindings 
                        fixedAttributeAsConstantProperty="false" 
                        typesafeEnumBase="xsd:NCName" 
                        typesafeEnumMaxMembers="20000"
                        choiceContentProperty="false" 
                        typesafeEnumMemberName="generateError" 
                        enableFailFastCheck="false" 
                        generateIsSetMethod="true"
                        underscoreBinding="asCharInWord"
                        generateElementProperty="true">
                    </jxb:globalBindings -->
                    
                    <!-- Decide if we need targetNamespace and settings or not -->
                    <xsl:variable name="childElements" select="//as:AssemblyStructure/as:Structure/as:Element/child::as:Element/@name"/>
                    <xsl:variable name="childNamePrefixes"><xsl:for-each select="$childElements"><xsl:value-of select="substring-before(.,':')"/>|</xsl:for-each></xsl:variable>
                    <xsl:variable name="childNamePrefixChk" select="distinct-values(tokenize($childNamePrefixes,'\|'))"/>
                    <xsl:variable name="matchedRootPrefix"><xsl:for-each select="$childNamePrefixChk"><xsl:value-of select="if(. = $rootPrefix or(. = '')) then 'Yes' else 'No'"/></xsl:for-each></xsl:variable>
                    
                    <!-- Now create package definitions per namespace import -->
                    <xsl:for-each select="//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))]">
                        <xsl:variable name="ns" select="@prefix"/>
                        <xsl:variable name="nsURI" select="if(starts-with(text(),'&#xA;')) then substring-after(text(),'&#xA;') else text()"/>
                        <xsl:variable name="nsURIname">
                            <xsl:choose>
                                <xsl:when test="starts-with($nsURI,'http://')"><xsl:value-of select="substring-after($nsURI,'http://')"/></xsl:when>
                                <xsl:when test="starts-with($nsURI,'urn:')"><xsl:value-of select="substring-after($nsURI,'urn:')"/></xsl:when>
                                <xsl:when test="contains($nsURI,':')"><xsl:value-of select="substring-after($nsURI,':')"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="$nsURI"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:variable name="packageName" select="w2e:packageName($nsURIname)"/>   
                        
                        <xsl:if test="not($rootPrefix=$ns) or($nsCount &lt; 2) or(not(contains($matchedRootPrefix,'No')))"><!-- create import for each namespace except if ns of the root element; that will use targetnamespace instead -->
                            <!-- Lookup folder path -->
                            <xsl:variable name="ns-pathLocation" select="camed:mapNS-2-path($ns,$nsURI,$xsdOutput,$ns-maps)"/>
                            <!-- Compute relative path -->
                            <xsl:variable name="nodeOffsetValue" select="camed:computeOffset($defaultExchangePath,$ns-pathLocation)"/>
                            <xsl:variable name="resultImportPath" select="replace(concat($nodeOffsetValue,$filenameStub,'-',$ns,'.xsd'),'//','/')"/>
                            <xsl:message> - binding: <xsl:value-of select="$resultImportPath"/>  (<xsl:value-of select="$defaultExchangePath"/>)</xsl:message>
                            <jxb:bindings schemaLocation="{$resultImportPath}" node="/xsd:schema">
                                <jxb:schemaBindings>
                                    <jxb:package name="{$packageName}"/>
                                    <jxb:nameXmlTransform>
                                        <jxb:elementName suffix="Element"/>
                                    </jxb:nameXmlTransform>
                                </jxb:schemaBindings>
                                <!-- Label any duplicate named elements -->
                                <xsl:variable name="nsComponents">
                                    <xsl:for-each select="$declElemAll"><xsl:if test="starts-with(.,concat($ns,':'))"><xsl:value-of select="concat(.,'|')"/></xsl:if></xsl:for-each>
                                </xsl:variable>
                                <xsl:variable name="duplicateComponents">
                                    <xsl:for-each select="tokenize($nsComponents,'\|')">
                                        <xsl:variable name="this" select="substring-after(.,':')"/>
                                        <xsl:for-each select="$declElemAll">
                                            <xsl:variable name="thisName" select="if(contains(.,':')) then substring-after(.,':') else ."/>
                                            <xsl:if test="string-length($this) &gt; 0 and(not(starts-with(.,$ns))) and($thisName = $this)"><xsl:value-of select="."/>|</xsl:if>
                                        </xsl:for-each>
                                    </xsl:for-each>
                                </xsl:variable>
                                <xsl:if test="count(tokenize($duplicateComponents,'\|')) &gt; 1">
                                <xsl:comment>NOTE: Restore these as needed to resolve duplicate component names conflicts (not all may apply)</xsl:comment>
                                <xsl:for-each select="tokenize($duplicateComponents,'\|')">
                                    <xsl:if test="string-length(.) &gt; 0">
                                        <!-- jxb:bindings node=".//xsd:element[@name='{substring-after(.,':')}']">
                                                <jxb:property name="{concat(substring-after(.,':'),upper-case(substring($ns,1,1)),substring-after($ns,substring($ns,1,1)))}"/>
                                            </jxb:bindings -->
   <xsl:comment>
     &lt;jxb:bindings node=".//xsd:element[@name='<xsl:value-of select="substring-after(.,':')"/>']"&gt;
       &lt;jxb:property name="<xsl:value-of select="concat(substring-after(.,':'),upper-case(substring($ns,1,1)),substring-after($ns,substring($ns,1,1)))"/>"/&gt;
     &lt;/jxb:bindings&gt;
   </xsl:comment>
                                    </xsl:if>
                                </xsl:for-each>
                                </xsl:if>
                            </jxb:bindings>
                        </xsl:if>
                    </xsl:for-each>                     
                </jxb:bindings>                
            </xsl:when>
        <xsl:otherwise><!-- Normal output mode - default, NIEM or Library -->
                       
        <xsl:variable name="exchangeSchemaXSD">            
          <xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema">
            <xsl:copy-of select="$loadNamespaces//namespace::*"/>
            <xsl:variable name="nsCount" select="count(//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))])"/>
              
          <!-- Decide if we need targetNamespace and settings or not -->
          <xsl:variable name="childElements" select="//as:AssemblyStructure/as:Structure/as:Element/child::as:Element/@name"/>
          <xsl:variable name="childNamePrefixes"><xsl:for-each select="$childElements"><xsl:value-of select="substring-before(.,':')"/>|</xsl:for-each></xsl:variable>
          <xsl:variable name="childNamePrefixChk" select="distinct-values(tokenize($childNamePrefixes,'\|'))"/>
          <xsl:variable name="matchedRootPrefix"><xsl:for-each select="$childNamePrefixChk"><xsl:value-of select="if(. = $rootPrefix or(. = '')) then 'Yes' else 'No'"/></xsl:for-each></xsl:variable>
          <xsl:message> - checking ns - <xsl:value-of select="$matchedRootPrefix"/> - <xsl:value-of select="$childNamePrefixChk"/> - <xsl:value-of select="$childNamePrefixes"/> - <xsl:value-of select="$childElements"/></xsl:message>
  
              <xsl:if test="string-length($rootPrefix) &gt; 0 and($nsCount &gt; 1)">            
                  <xsl:choose><!-- If root namespace is same as child element, no, otherwise yes -->
                      <xsl:when test="contains($matchedRootPrefix,'No')">
                          <xsl:variable name="rootURItext" select="//as:Namespaces/as:namespace[@prefix=$rootPrefix]"/>
                          <xsl:variable name="rootURI" select="if(starts-with($rootURItext,'&#xA;')) then substring-after($rootURItext,'&#xA;') else $rootURItext"/>
                          <xsl:attribute name="targetNamespace"><xsl:value-of select="$rootURI"/></xsl:attribute>
                      </xsl:when>
                      <xsl:otherwise/><!-- Namespaces all match, so target namespace not needed -->   
                  </xsl:choose> 
            </xsl:if>
            <xsl:attribute name="version">
                <xsl:value-of select="//as:Header/as:Version"/>
            </xsl:attribute>
            <xsl:if test="$requireNS='true'">
                <xsl:attribute name="elementFormDefault">qualified</xsl:attribute>
            </xsl:if>
            <xsd:annotation>
                <xsd:documentation> 
                        <xsl:value-of select="camed:mode-comment-text($xsdOutput,'Exchange',$ns-maps)"/> 
                    CAM Template Header information: 
                    
                    Description: <xsl:value-of select="//as:Header/as:Description"/> 
                    Owner: <xsl:value-of select="//as:Header/as:Owner"/> 
                    Date: <xsl:value-of select="//as:Header/as:DateTime"/> 
                    Version: <xsl:value-of select="//as:Header/as:Version"/> 
                    XSD generator: v<xsl:value-of select="$version"/> 
                    NS Mode: <xsl:value-of select="$requireNS"/> 
                    XSDOutput Mode: <xsl:value-of select="$xsdOutput"/>
                   
                    <xsl:value-of select="camed:mode-comment-end($xsdOutput,'exchange',$ns-maps)"/> 
                </xsd:documentation>
                <xsd:appinfo source="CAM"><xsl:value-of select="if(contains($rootElement,':')) then substring-after($rootElement,':') else $rootElement"/></xsd:appinfo>
            </xsd:annotation>
            <xsl:if test="contains($xsdOutput,'NIEM')">
                <xsd:annotation>
                    <xsd:appinfo>
                        <i:ConformantIndicator xmlns:i="http://niem.gov/niem/appinfo/2.0">true</i:ConformantIndicator>
                    </xsd:appinfo>
                </xsd:annotation>
            </xsl:if>
              
            <xsl:for-each select="//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))]">
                <xsl:variable name="ns" select="@prefix"/>
                <xsl:variable name="nsURI" select="if(starts-with(text(),'&#xA;')) then substring-after(text(),'&#xA;') else text()"/>
                <xsl:if test="not($rootPrefix=$ns) or($nsCount &lt; 2) or(not(contains($matchedRootPrefix,'No')))"><!-- create import for each namespace except if ns of the root element; that will use targetnamespace instead -->
                    <!-- Lookup folder path -->
                    <xsl:variable name="ns-pathLocation" select="camed:mapNS-2-path($ns,$nsURI,$xsdOutput,$ns-maps)"/>
                    <!-- Compute relative path -->
                    <xsl:variable name="nodeOffsetValue" select="camed:computeOffset($defaultExchangePath,$ns-pathLocation)"/>
                    <xsl:variable name="resultImportPath" select="replace(concat($nodeOffsetValue,$filenameStub,'-',$ns,'.xsd'),'//','/')"/>
                    <xsl:message> - importing: <xsl:value-of select="$resultImportPath"/>  (<xsl:value-of select="$defaultExchangePath"/>)</xsl:message>
                    <xsd:import namespace="{$nsURI}" schemaLocation="{$resultImportPath}"/>
                </xsl:if>
            </xsl:for-each>
            <xsl:for-each select="//as:AssemblyStructure/as:Structure/as:Element"><!-- select output structure elements -->
                <xsl:variable name="QName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                <xsl:message><xsl:value-of select="@name"/></xsl:message>
                
                <xsl:variable name="valuesRule" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
                <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
                <xsl:variable name="lookupRule" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/>
                
                <xsl:variable name="values" select="w2e:annoValues(.,$valuesRule,$lookupRule)" />
                <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,$lookupRule)"/>

                <xsl:variable name="baseType">
                    <xsl:call-template name="findType">
                        <xsl:with-param name="rules" select="as:Rule"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:message> -- Root element(s)</xsl:message>
                <xsd:element name="{$QName}" id="{generate-id()}">
                    <xsl:if test="contains(@name,':')"><!-- If root element has namespace prefix then reference to the type defined in the import for that namespace -->
                        <xsl:choose>
                            <xsl:when test="$nsCount &lt; 2"><xsl:attribute name="type" select="@name"/></xsl:when><!-- point to the ns import defined complexType definition -->
                           <xsl:when test="starts-with(@name,concat($rootPrefix,':'))">
                               <xsl:choose>
                                   <xsl:when test="contains($matchedRootPrefix,'No')"><xsl:attribute name="type" select="concat(@name,$rootSuffix)"/></xsl:when>
                                   <xsl:otherwise><xsl:attribute name="type" select="concat(if(contains(@name,':')) then substring-after(@name,':') else @name,$rootSuffix)"/></xsl:otherwise>
                               </xsl:choose>    
                           </xsl:when>
                               <xsl:otherwise><xsl:attribute name="type" select="@name"/></xsl:otherwise>
                         </xsl:choose>
                    </xsl:if>
                    <xsl:if test="count(camed:annotation) &gt; 0">
                        <xsl:call-template name="outputAnnotation">
                            <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                        </xsl:call-template>
                    </xsl:if>
                    <xsl:if test="not(contains(@name,':'))"><!-- For root element without namespace then define type here, otherwise the type attribute above provides type linkage -->
                    <xsd:complexType>
                        <xsd:sequence>
                            <xsl:choose>
                                <xsl:when test="as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]">
                                    <xsl:variable name="choiceRules" select="distinct-values(as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]/@action)"/>
                                    <xsl:variable name="xsdChoice1">
                                        <xsd:choice>
                                            <xsl:for-each select="as:Element">
                                                <xsl:if test="contains($choiceRules[1],concat('''',@name,''''))">
                                                  <xsl:apply-templates select="." />
                                                </xsl:if>
                                            </xsl:for-each>
                                        </xsd:choice>
                                    </xsl:variable>
                                    <xsl:variable name="xsdChoice2">
                                        <xsl:if test="count($choiceRules[2]) &gt; 0">
                                        <xsd:choice>
                                            <xsl:for-each select="as:Element">
                                                <xsl:if test="contains($choiceRules[2],concat('''',@name,''''))">
                                                    <xsl:apply-templates select="." />
                                                </xsl:if>
                                            </xsl:for-each>
                                        </xsd:choice>
                                        </xsl:if>
                                    </xsl:variable>
                                    <xsl:variable name="xsdChoice3">
                                        <xsl:if test="count($choiceRules[3]) &gt; 0">
                                        <xsd:choice>
                                            <xsl:for-each select="as:Element">
                                                <xsl:if test="contains($choiceRules[3],concat('''',@name,''''))">
                                                    <xsl:apply-templates select="." />
                                                </xsl:if>
                                            </xsl:for-each>
                                        </xsd:choice>
                                        </xsl:if>
                                    </xsl:variable>

                                    <xsl:for-each select="as:Element"><!-- loop thru each child element -->
                                        <xsl:message> --- <xsl:value-of select="@name"/></xsl:message>
                                        <xsl:variable name="thisName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                                        <xsl:variable name="choiceItem">
                                            <xsl:call-template name="chkChoiceItem">
                                                <xsl:with-param name="name" select="$thisName"/>
                                                <xsl:with-param name="xsdChoice" select="if ($xsdChoice1//xsd:element[@name=$thisName]) then $xsdChoice1 else if  ($xsdChoice2//xsd:element[@name=$thisName]) then $xsdChoice2 else $xsdChoice3"/>
                                            </xsl:call-template>
                                        </xsl:variable>
                                        <xsl:choose>
                                            <xsl:when test="$choiceItem='none'">
                                                <xsl:apply-templates select="."/>
                                            </xsl:when>
                                            <xsl:when test="$choiceItem='skip'"/>
                                            <xsl:otherwise>
                                                <xsl:copy-of select="$choiceItem"/>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:for-each>
                                    
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:apply-templates select="as:Element"/><!-- Process all the child elements -->
                                </xsl:otherwise>
                            </xsl:choose>
                            <xsl:if test="string-length($values) &gt; 0">
                                <xsl:variable name="listBaseType">
                                    <xsl:choose>
                                        <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                                        <xsl:otherwise>xsd:string</xsl:otherwise>
                                    </xsl:choose>
                                </xsl:variable>
                                <xsd:simpleType>
                                    <xsd:restriction base="{$listBaseType}">
                                        <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                            <xsl:call-template name="setLength">
                                                <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                            </xsl:call-template>
                                        </xsl:if>
                                        <xsl:if test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                                            <xsl:call-template name="setPattern">
                                                <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                                            </xsl:call-template>
                                        </xsl:if>
                                        <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                                    </xsd:restriction>
                                </xsd:simpleType>
                            </xsl:if>
                        </xsd:sequence>
                        <xsl:choose><!-- Process the attributes for this root element -->
                            <xsl:when test="count(child::*/as:Element)=0"><xsl:apply-templates select="child::*/as:Attribute"/></xsl:when>
                            <xsl:otherwise><xsl:apply-templates select="as:Attribute"/></xsl:otherwise>
                        </xsl:choose>
                    </xsd:complexType>
                    </xsl:if>
                </xsd:element>
            </xsl:for-each>

             <xsl:comment> Parent complexTypes <xsl:value-of select="'&#xD; &#xD;' "/></xsl:comment>
              <xsl:message> -- Parent complexTypes</xsl:message>

            <xsl:choose>
                <xsl:when test="starts-with($rootElement,concat($rootPrefix,':'))">
                    <xsl:for-each select="//as:AssemblyStructure/as:Structure/as:Element"><!-- select local root element definitions -->
                        <xsl:apply-templates select="." mode="root"/>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:for-each select="//as:AssemblyStructure/as:Structure/as:Element"><!-- select local root element definitions -->
                        <xsl:apply-templates select="." mode="group"/>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
            
            <xsl:comment> Local simpleTypes (default namespace) <xsl:value-of select="'&#xD; &#xD;' "/></xsl:comment>
              <xsl:message> -- Local simpleTypes</xsl:message>
            <!-- collect all the element simple type definitions needed -->
            <xsl:variable name="simpleTypeDefs">
               <types>
                 <xsl:for-each select="//as:AssemblyStructure/as:Structure/as:Element">
                    <xsl:apply-templates select="." mode="simpleType"/>
                 </xsl:for-each>
               </types>
            </xsl:variable>

            <xsl:variable name="simpleTypeNames" select="distinct-values($simpleTypeDefs/types/xsd:simpleType/@name)"/>
            <xsl:variable name="complexTypeNames" select="distinct-values($simpleTypeDefs/types/xsd:complexType/@name)"/>
            <!-- output only the first matching name simple type definition -->
            <xsl:for-each select="$simpleTypeNames">
                <xsl:variable name="thisType" select="."/>
                <xsl:copy-of select="$simpleTypeDefs/types//xsd:simpleType[@name=$thisType][1]"/>
            </xsl:for-each>
            <xsl:for-each select="$complexTypeNames"><!-- note: these should only be datatype(any) items -->
                <xsl:variable name="thisType" select="."/>
                <xsl:copy-of select="$simpleTypeDefs/types//xsd:complexType[@name=$thisType][1]"/>
            </xsl:for-each>
            
        </xsd:schema>
        </xsl:variable>
            
       <!-- Now output the master parent schema -->
        <xsl:message> - Generating master schema to <xsl:value-of select="$writeResultPath"/></xsl:message>
        <xsl:result-document exclude-result-prefixes="w2e" format="exchangeXSD" indent="yes"
                href="{$writeResultPath}">
                <xsl:copy-of select="$exchangeSchemaXSD" exclude-result-prefixes="w2e"/>
        </xsl:result-document>
            
        <xsl:copy-of select="$exchangeSchemaXSD" exclude-result-prefixes="w2e"/>
            
        <!-- xsl:message>duplicates [<xsl:copy-of select="$duplicateElements"/>]</xsl:message -->
        <!-- xsl:message>nsContent [
            <xsl:for-each select="$nsContentModels/pack/*"><xsl:copy-of select="."/></xsl:for-each>]</xsl:message -->
            <xsl:message> - Processing import schemas:</xsl:message>
        <xsl:apply-templates select='/' mode="imports"/>
        
        </xsl:otherwise>
        </xsl:choose>
        <xsl:message> - Schema components completed.</xsl:message>
 </xsl:template>
        
<xsl:template match="/" mode="imports">        
        <!-- Now output each of the import files by namespace -->
        <xsl:variable name="nsCount" select="count(//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))])"/>
        <xsl:for-each
            select="//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))]">
            <xsl:message> -- ns - <xsl:value-of select="@prefix"/></xsl:message>
            <xsl:variable name="ns" select="@prefix"/>
            <xsl:variable name="nsURI" select="if(starts-with(text(),'&#xA;')) then substring-after(text(),'&#xA;') else text()"/>
            
            <xsl:variable name="defsElements">
                <xsl:for-each select="$source//as:Element[starts-with(@name,concat($ns,':'))]">
                    <xsl:copy-of select="."/>
                    <xsl:if test="./parent::as:Element/as:Attribute"><!-- if parent has attributes, then we need to check their namespaces -->
                        <xsl:element name="as:Element">
                            <xsl:attribute name="name"><xsl:value-of select="./parent::as:Element/@name"/></xsl:attribute>
                            <xsl:attribute name="path"><xsl:value-of select="./parent::as:Element/@path"/></xsl:attribute>
                        </xsl:element>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:message> --- <xsl:value-of select="count($defsElements//as:Element)"/> elements found</xsl:message> 
            <xsl:message> --- computing unique elements</xsl:message>
            <xsl:variable name="declElem" select="distinct-values($defsElements/as:Element/@name)"/>
            <xsl:message> --- <xsl:value-of select="count($declElem)"/> unique elements</xsl:message>
            
            <xsl:variable name="defsAttribs">
                <xsl:for-each select="$source//as:Attribute[starts-with(@name,concat($ns,':'))]">
                    <xsl:copy-of select="."/>
                </xsl:for-each>
            </xsl:variable>
            <xsl:message> --- <xsl:value-of select="count($defsAttribs//as:Attribute)"/> attributes found</xsl:message>
            <xsl:variable name="declAttrib" select="distinct-values($defsAttribs/as:Attribute/@name)"/>
            
            <xsl:message> --- <xsl:value-of select="count($declAttrib)"/> unique attributes</xsl:message>
            
            <xsl:message> --- computing what goes where for elements</xsl:message>            
            <xsl:variable name="what-goes-where">
                <xsl:call-template name="whatgoeswhere">
                    <xsl:with-param name="thisNS" select="$ns"></xsl:with-param>
                </xsl:call-template>
            </xsl:variable>
            <xsl:message> --- what goes where done</xsl:message>
            <!-- xsl:if test="$ns='ansi'"><xsl:message>elements [<xsl:value-of  select="$declElem"/>]</xsl:message></xsl:if>
            <xsl:if test="$ns='ansi'"><xsl:message>attributes [<xsl:value-of  select="$declAttrib"/>]</xsl:message></xsl:if -->
            <xsl:message> -- building import components</xsl:message>
            <xsl:variable name="imports">
                <xsl:choose>
                    <xsl:when test="$nsCount &lt; 2">
                        <xsl:for-each select="//as:Element/@name">
                            <xsl:if test="contains(.,':')">
                                <import>
                                    <xsl:value-of select="substring-before(.,':')"/>
                                </import>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>    
                <xsl:for-each select="$declElem">
                  <xsl:variable name="findIt" select="."/>  
                  <xsl:for-each select="$defsElements/as:Element[@name=$findIt][1]">
                      <!-- xsl:message> - - - <xsl:value-of select="$findIt"/></xsl:message -->
                    <xsl:for-each select="$source//as:Element[@name=$findIt][1]/as:Attribute/@name">
                        <xsl:if test="contains(.,':')">
                            <import>
                                <xsl:value-of select="substring-before(.,':')"/>
                            </import>
                        </xsl:if>
                    </xsl:for-each>
                    <xsl:for-each select="//as:Element/@name">
                        <xsl:if test="contains(.,':')">
                            <import>
                                <xsl:value-of select="substring-before(.,':')"/>
                            </import>
                        </xsl:if>
                    </xsl:for-each>
                </xsl:for-each>
                </xsl:for-each>
                    </xsl:otherwise>
                    </xsl:choose>
            </xsl:variable>
            <xsl:message> -- <xsl:value-of select="count($imports//import)"/> import components built</xsl:message>
            <xsl:variable name="imports-NS" select="distinct-values($imports/import)"/>
            <xsl:variable name="imports-Chk">
                <xsl:for-each select="$imports-NS">|<xsl:value-of select="."/>|</xsl:for-each>
            </xsl:variable>
            
            <xsl:variable name="ns-outputFolder" select="camed:mapNS-2-path($ns,$nsURI,$xsdOutput,$ns-maps)"/>
            <!-- Compute actual output path to write to --> 
            <xsl:variable name="ns-resultNamePath" select="camed:resolve-ns-outputpath($fileNamePath,$filenameStub,$ns-outputFolder,'')"/>
            <xsl:variable name="writeResultPath" select="concat($ns-resultNamePath,'-',$ns,'.xsd')"/>
            <xsl:message> - import for <xsl:value-of select="$ns"/> written to: <xsl:value-of select="$writeResultPath"/></xsl:message>
            <xsl:result-document exclude-result-prefixes="w2e" format="importXSD" indent="yes"
                href="{$writeResultPath}">
                <xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                    xmlns:as="http://www.oasis-open.org/committees/cam" targetNamespace="{$nsURI}">
                    <xsl:copy-of select="$loadNamespaces//namespace::*"/>
                    <xsl:if test="$requireNS='true'">
                        <xsl:attribute name="elementFormDefault">qualified</xsl:attribute>
                    </xsl:if>
                    <xsd:annotation>
                        <xsd:documentation> 
                           <xsl:value-of select="camed:mode-comment-import($xsdOutput,$filenameStub,$ns,$ns-maps)"/>                            
                           CAM Template Header information: 
                           
                           Description: <xsl:value-of select="//as:Header/as:Description"/> 
                            Owner: <xsl:value-of select="//as:Header/as:Owner"/> 
                            Date: <xsl:value-of select="//as:Header/as:DateTime"/> 
                            Version: <xsl:value-of select="//as:Header/as:Version"/> generator v<xsl:value-of select="$version"/> 
                            
                            <xsl:value-of select="camed:mode-comment-end($xsdOutput,'import',$ns-maps)"/>
                            <xsl:value-of select="$nsURI"/>
                        </xsd:documentation>
                    </xsd:annotation>
                    <xsl:if test="contains($xsdOutput,'NIEM')">
                        <xsl:variable name="ConTessa">
                            <xsl:choose>
                                <xsl:when test="contains($nsURI,'niem.gov')">false</xsl:when>
                                <xsl:when test="contains($nsURI,'lexs')">false</xsl:when>
                                <xsl:when test="contains($nsURI,'w3c')">false</xsl:when>
                                <xsl:when test="contains($nsURI,'w3.org')">false</xsl:when>
                                <xsl:when test="contains($nsURI,'oasis')">false</xsl:when>
                                <xsl:when test="contains($nsURI,'gml')">false</xsl:when>
                                <xsl:otherwise>true</xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsd:annotation>
                            <xsd:appinfo>
                                <i:ConformantIndicator xmlns:i="http://niem.gov/niem/appinfo/2.0"><xsl:value-of select="$ConTessa"/></i:ConformantIndicator>
                            </xsd:appinfo>
                        </xsd:annotation>
                    </xsl:if>
                    
                    <xsl:variable name="add-imports">
                        <xsl:for-each select="$what-goes-where//moves/move[@ns=$ns]/as:Element">
                            <xsl:for-each select="//as:Attribute/@name">
                                <xsl:if test="contains(.,':')">
                                    <import>
                                        <xsl:value-of select="substring-before(.,':')"/>
                                    </import>
                                </xsl:if>
                            </xsl:for-each>
                            <xsl:for-each select="//as:Element/@name">
                                <xsl:if test="contains(.,':')">
                                    <import>
                                        <xsl:value-of select="substring-before(.,':')"/>
                                    </import>
                                </xsl:if>
                            </xsl:for-each>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="add-imports-NS" select="distinct-values($add-imports/import)"/>
                    <xsl:variable name="add-imports-Chk">
                        <xsl:for-each select="$add-imports-NS">|<xsl:value-of select="."/>|</xsl:for-each>
                    </xsl:variable>
                   
                    <xsl:for-each
                        select="//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))]">
                        <xsl:variable name="ns-import" select="@prefix"/>
                        <xsl:variable name="nsURI-import" select="if(starts-with(text(),'&#xA;')) then substring-after(text(),'&#xA;') else text()"/>
                        <xsl:if test="not($ns=$ns-import)">
                            <xsl:choose>
                                <xsl:when test="contains($imports-Chk,concat('|',$ns-import,'|'))">
                                    <!-- Lookup path location -->
                                    <xsl:variable name="ns-pathLocation" select="camed:mapNS-2-path($ns-import,$nsURI-import,$xsdOutput,$ns-maps)"/>
                                    <!-- Compute relative path -->
                                    <xsl:variable name="nodeOffset" select="camed:computeOffset($ns-outputFolder,$ns-pathLocation)"/>
                                    <xsl:variable name="resultChildPath" select="concat($nodeOffset,$filenameStub,'-',$ns-import,'.xsd')"/>
                                    <xsl:message> - with xmlns import: <xsl:value-of select="$resultChildPath"/></xsl:message>
                                    <xsd:import namespace="{$nsURI-import}" schemaLocation="{$resultChildPath}"/>
                                </xsl:when>
                                <xsl:when test="contains($add-imports-Chk,concat('|',$ns-import,'|'))">
                                    <!-- Lookup path location -->
                                    <xsl:variable name="ns-pathLocation" select="camed:mapNS-2-path($ns-import,$nsURI-import,$xsdOutput,$ns-maps)"/>
                                    <!-- Compute relative path -->
                                    <xsl:variable name="nodeOffset" select="camed:computeOffset($ns-outputFolder,$ns-pathLocation)"/>
                                    <!-- xsl:message> - <xsl:value-of select="$ns-import"/> [<xsl:value-of select="$nsURI-import"/>] [<xsl:value-of select="$xsdOutput"/>]</xsl:message>
                                    <xsl:message> - <xsl:value-of select="$ns-pathLocation"/> [<xsl:value-of select="$ns-outputFolder"/>] [<xsl:value-of select="$ns-pathLocation"/>] [<xsl:value-of select="$nodeOffset"/>]</xsl:message -->
                                    <xsl:variable name="resultChildPath" select="concat($nodeOffset,$filenameStub,'-',$ns-import,'.xsd')"/>
                                    <xsl:message> - add xmlns import: <xsl:value-of select="$resultChildPath"/></xsl:message>
                                    <xsd:import namespace="{$nsURI-import}" schemaLocation="{$resultChildPath}"/>
                                </xsl:when>
                                <xsl:otherwise/>
                            </xsl:choose>
                        </xsl:if>
                    </xsl:for-each>
                    
                    <xsl:variable name="importSchemaContent">
                      <content>
                    <!-- xsl:message>items list A - <xsl:value-of select="$ns"/> - <xsl:value-of select="$what-goes-where//moves/move[@ns=$ns]/as:Element/@name"/></xsl:message -->
                    <xsl:for-each select="$what-goes-where//moves/move[@ns=$ns]/as:Element">
                        <xsl:if test="$debug=1"><xsl:comment>Moved type - <xsl:value-of select="@name"/><xsl:value-of select="'&#xD; &#xD;' "/>
                        </xsl:comment></xsl:if>
                        
                        <xsl:apply-templates select="." mode="complexType">
                            <xsl:with-param name="ref">false</xsl:with-param>
                            <xsl:with-param name="prefix"></xsl:with-param>
                        </xsl:apply-templates>
                    </xsl:for-each>
                    
                    <xsl:for-each select="$declAttrib">
                        <xsl:variable name="this" select="."/>
                        <xsl:variable name="getRule" select="$defsAttribs/as:Attribute[@name=$this]"/>
                        <xsl:choose>
                            <xsl:when
                                test="count($getRule//as:constraint[@action='excludeAttribute()']) = 0">
                                <xsl:apply-templates select="$getRule[1]" mode="type"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:variable name="getRuleToApply">
                                    <xsl:for-each select="$getRule">
                                        <xsl:if
                                            test="not(as:Rule/as:constraint[@action='excludeAttribute()'])">
                                            <xsl:copy-of select="."/>
                                        </xsl:if>
                                    </xsl:for-each>
                                </xsl:variable>
                                <!-- xsl:variable name="firstRule" select="$getRuleToApply/as:Attribute[1]"></xsl:variable -->
                                <xsl:apply-templates select="$getRuleToApply/as:Attribute[1]" mode="type"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                    <xsl:for-each select="$declElem">
                        <xsl:variable name="this" select="."/>
                        <xsl:variable name="getElem" select="$defsElements/as:Element[@name=$this][1]"/>
                        <!-- xsl:message>Element [<xsl:value-of select="$this"/>]</xsl:message -->
                        
                        <xsl:variable name="lookDown" select="$getElem/as:Element/substring-before(@name,':')"/>
                        <xsl:variable name="countDown" select="count($lookDown)" as="xsd:integer"/>
                        <xsl:variable name="chkDown" select="distinct-values($lookDown)" />
                        
                        <xsl:variable name="downChanged">
                            <xsl:if test="$countDown &gt; 0">
                                <xsl:for-each select="$chkDown">
                                    <xsl:choose>
                                        <xsl:when test=".=''"/>
                                        <xsl:when test="not(. = $ns)">
                                            <changed><xsl:value-of select="."/></changed>
                                        </xsl:when>
                                        <xsl:when test=". = $ns">
                                            <notchanged><xsl:value-of select="$ns"/></notchanged>
                                        </xsl:when>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:if>
                        </xsl:variable>

                        <xsl:variable name="getElemName" select="$getElem[1]/@name"/>
                        <xsl:variable name="getElemPrefix" select="substring-before($getElemName,':')"/>

                        <xsl:variable name="chkElemUp" select="$source//as:Element[@name=$this]/../../as:Element"/>
                        <xsl:variable name="chkElemName" select="$chkElemUp[1]/@name"/>
                        <xsl:variable name="chkElemUpPrefix" select="substring-before($chkElemName,':')"/>

                        <!-- xsl:message>chkElem[<xsl:value-of select="$chkElemName"></xsl:value-of>]</xsl:message -->

                        <xsl:choose>
                            <xsl:when test="count($downChanged//notchanged) &gt; 0 and($chkElemUpPrefix='' or($chkElemUpPrefix=$ns))">
                                <xsl:if test="$debug=1"><xsl:comment> prefix matched <xsl:value-of select="'&#xD; &#xD;' "/>
                                </xsl:comment></xsl:if>
                                <xsl:apply-templates select="$getElem[1]" mode="type">
                                    <xsl:with-param name="ref" select="'true'"/>
                                    <xsl:with-param name="prefix" select="$ns"/>
                                </xsl:apply-templates>
                            </xsl:when>
                            <xsl:when test="count($downChanged//changed) &gt; 0 and($countDown &gt; 0)">
                                <xsl:if test="$debug=1"><xsl:comment> down changed <xsl:value-of select="'&#xD; &#xD;' "/>
                                </xsl:comment></xsl:if>
                                <xsl:choose>
                                    <xsl:when test="$chkElemUpPrefix=''"><xsl:apply-templates select="$getElem[1]" mode="type"/></xsl:when>
                                    <xsl:when test="$ns = $lookDown[1]"/><!-- Prevent circular definition -->
                                    <xsl:otherwise>
                                        <xsd:element name="{substring-after($getElemName,':')}" type="{$getElemName}"/>
                                        
                                        <xsd:complexType name="{substring-after($getElemName,':')}">
                                            <xsd:annotation>
                                            <xsd:documentation>The container element for <xsl:value-of select="$getElemName"></xsl:value-of>
                                            content model items. </xsd:documentation>
                                            </xsd:annotation>
                                            <xsd:complexContent>
                                            <xsd:extension base="{concat($lookDown[1],':',substring-after($getElemName,':'))}"/>
                                            </xsd:complexContent>
                                            </xsd:complexType>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:when test="$chkDown = $ns and($chkElemUpPrefix = '')">
                                <xsl:if test="$debug=1"><xsl:comment> down same <xsl:value-of select="'&#xD; &#xD;' "/>
                                </xsl:comment></xsl:if>
                                <xsl:apply-templates select="$getElem[1]" mode="type"/>
                             </xsl:when>
                            <xsl:when test="$chkElemUpPrefix = ''">
                                <xsl:if test="$debug=1"><xsl:comment> up blank prefix <xsl:value-of select="'&#xD; &#xD;' "/>
                                </xsl:comment></xsl:if>
                                <xsl:apply-templates select="$getElem[1]" mode="type"/>
                            </xsl:when>
                            <xsl:when test="$chkElemUpPrefix = $getElemPrefix">
                                <xsl:if test="$debug=1"><xsl:comment> up same prefix <xsl:value-of select="'&#xD; &#xD;' "/>
                                </xsl:comment></xsl:if>
                                <xsl:if test="count($what-goes-where//moves/move[@ns=$ns]/as:Element[@name=$getElemName]) = 0">
                                    <!-- If element not declared do it here -->
                                    <xsl:if test="contains($getElemName,':')"><xsd:element name="{substring-after($getElemName,':')}" type="{$getElemName}"/></xsl:if>
                                    <!-- and then output matching type (simple or complex) -->
                                    <xsl:apply-templates select="$getElem[1]" mode="type">
                                        <xsl:with-param name="ref"/>
                                        <xsl:with-param name="prefix" select="$ns"/>
                                    </xsl:apply-templates>
                                </xsl:if>
                            </xsl:when>
                            <xsl:when test="$lookDown = '' or(count($lookDown) = 0)">
                                <xsl:if test="$debug=1"><xsl:comment> look down blank <xsl:value-of select="'&#xD; &#xD;' "/>
                                </xsl:comment></xsl:if>
                                <!-- If not duplicate - then output element -->
                                <xsl:variable name="thisName" select="if(contains($getElemName,':')) then substring-after($getElemName,':') else $getElemName"/>
                                <xsl:variable name="thatName" select="$getElemName"/>
                                <xsl:variable name="thatNS" select="if(contains($getElemName,':')) then substring-before($getElemName,':') else ''"/>
                                <!-- xsl:message><xsl:value-of select="$thisName"/> - <xsl:value-of select="$thatName"/> </xsl:message -->
                                <xsl:choose>
                                    <xsl:when test="count($duplicateElements/dup[@name=$thisName]) &gt; 0"><xsl:if test="$debug=1"><xsl:comment>skipped - duplicate element definition <xsl:value-of select="$thisName"/></xsl:comment></xsl:if></xsl:when>
                                    <xsl:when test="count($nsContentModels/pack//xsd:element[@name=$thisName][@type='']) &gt; 0"><xsl:if test="$debug=1"><xsl:comment>skipped - duplicate packed definition <xsl:value-of select="$thisName"/></xsl:comment></xsl:if></xsl:when>
                                    <xsl:when test="count($nsContentModels/pack[@type=$thatNS]/xsd:complexType[@name=$thatName]) &gt; 0"><xsl:if test="$debug=1"><xsl:comment>skipped - duplicate complexType definition <xsl:value-of select="$thatName"/></xsl:comment></xsl:if></xsl:when>
                                    <!-- When not needed as can only reach here if true - xsl:when test="count(as:Element//descendant::*) = 0 "></xsl:when -->
                                    <xsl:otherwise>
                                        <xsl:element name="xsd:element">
                                                    <xsl:attribute name="name" select="if(contains($getElemName,':')) then substring-after($getElemName,':') else $getElemName"></xsl:attribute>
                                                    <xsl:attribute name="type" select="$getElemName"></xsl:attribute>
                                        </xsl:element>
                                        <xsl:if test="$debug=1"><xsl:comment>output element type: <xsl:value-of select="$getElemName"/> </xsl:comment></xsl:if>
                                        <xsl:apply-templates select="$getElem[1]" mode="type"/>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:if test="$debug=1"><xsl:comment> declaration <xsl:value-of select="'&#xD; &#xD;' "/>
                                </xsl:comment></xsl:if>
                                <xsl:apply-templates select="$getElem[1]" mode="type"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>

                    <xsl:variable name="unpackModel"
                        select="$nsContentModels//pack[@type=$ns]/xsd:complexType"/>
                    
                    <xsl:copy-of select="$unpackModel"/>
                        </content>
                    </xsl:variable>
                    
                    <xsl:choose>
                        <xsl:when test="$debug = 1">
                            <xsl:copy-of select="$importSchemaContent/content/*"/><!-- Now write the XSD -->
                        </xsl:when>
                        <xsl:otherwise>
                            <!-- Now write only unique declarations -->
                            <xsl:variable name="importSchemaElements" select="distinct-values($importSchemaContent/content/xsd:element/@name)"/>
                            <!-- xsl:message> Elements - <xsl:value-of select="$importSchemaElements"/></xsl:message -->
                            <xsl:variable name="importSchemaComplexType" select="distinct-values($importSchemaContent/content/xsd:complexType/@name)"/>
                            <!-- xsl:message> ComplexTypes - <xsl:value-of select="$importSchemaComplexType"/></xsl:message -->
                            <xsl:variable name="importSchemaSimpleType" select="distinct-values($importSchemaContent/content/xsd:simpleType/@name)"/>
                            <!-- xsl:message> SimpleTypes - <xsl:value-of select="$importSchemaSimpleType"/></xsl:message -->
                            <xsl:for-each select="$importSchemaElements">
                                <xsl:variable name="this" select="."/>
                                <!-- xsl:message> E <xsl:value-of select="$this"/></xsl:message -->
                                <xsl:variable name="thisItem" select="$importSchemaContent/content/xsd:element[@name = $this][1]"/>
                                <xsl:copy-of select="$thisItem[1]"/>
                            </xsl:for-each>
                            <xsl:for-each select="$importSchemaComplexType">
                                <xsl:variable name="this" select="."/>
                                <!-- xsl:message> C <xsl:value-of select="$this"/></xsl:message -->
                                <xsl:variable name="thisItem" select="$importSchemaContent/content/xsd:complexType[@name = $this][1]"/>
                                <xsl:copy-of select="$thisItem[1]"/>
                            </xsl:for-each>
                            <xsl:for-each select="$importSchemaSimpleType">
                                <xsl:variable name="this" select="."/>
                                <!-- xsl:message> S <xsl:value-of select="$this"/></xsl:message -->
                                <xsl:if test="not($importSchemaComplexType[. = $this])"><!-- avoid duplicate simple and complex types -->
                                    <xsl:variable name="thisItem" select="$importSchemaContent/content/xsd:simpleType[@name = $this][1]"/>
                                    <xsl:copy-of select="$thisItem[1]"/>
                                </xsl:if>
                            </xsl:for-each>
                            <xsl:copy-of select="$importSchemaContent/content/xsd:attributeGroup"/>
                            <xsl:copy-of select="$importSchemaContent/content/xsd:attribute"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsd:schema>
            </xsl:result-document>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="as:Element" mode="type">
        <xsl:param name="ref"/>
        <xsl:param name="prefix"/>
        <xsl:variable name="this" select="@name"/>
        <xsl:variable name="that" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="thatNS" select="if(contains(@name,':')) then substring-before(@name,':') else ''"/>
        <!-- xsl:message>Check <xsl:value-of select="$this"/>[<xsl:value-of select="count($duplicateElements/dup[@name=$this])"></xsl:value-of>]</xsl:message -->
        <xsl:choose>
            <xsl:when test="count($duplicateElements/dup[@name=$this]) &gt; 0"><xsl:if test="$debug=1"><xsl:comment>skipped - duplicate element definition</xsl:comment></xsl:if></xsl:when>
            <xsl:when test="count($nsContentModels/pack/xsd:complexType[@name=$this]) &gt; 0"><xsl:if test="$debug=1"><xsl:comment>skipped - duplicate complexType definition</xsl:comment></xsl:if></xsl:when>
            <xsl:when test="count($nsContentModels/pack[@type=$thatNS]/xsd:complexType[@name=$that]) &gt; 0"><xsl:if test="$debug=1"><xsl:comment>skipped - duplicate complexType in ns <xsl:value-of select="$thatNS"/> definition</xsl:comment></xsl:if></xsl:when>
            <xsl:when test="count(as:Element//descendant::*) = 0 and count(.//as:Attribute) = 0">
                <!-- output the simple type definition -->
                <!-- xsl:comment>no descendants</xsl:comment -->
                <!-- xsl:message><xsl:copy-of select="."/></xsl:message -->
                <xsl:apply-templates select="." mode="xsd">
                    <xsl:with-param name="ref"  select="$ref"/>
                    <xsl:with-param name="prefix" select="$prefix"/>
                </xsl:apply-templates>
            </xsl:when>
            <xsl:when test="count(as:Element//descendant::*) = 0 and count(.//as:Attribute) &gt; 0">
                <xsl:variable name="elementType">
                    <xsl:call-template name="findType">
                        <xsl:with-param name="rules" select="as:Rule"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsd:complexType name="{$that}">
                    <xsl:comment> Element with <xsl:value-of select="count(.//as:Attribute)"/> attributes.</xsl:comment>
                    <xsl:choose>
                        <xsl:when test="$elementType = 'xsd:any'">
                            <xsd:annotation>
                                <xsd:documentation>The any type for <xsl:value-of select="$that"/>. </xsd:documentation>
                            </xsd:annotation>
                            <xsd:sequence>
                                <xsd:any namespace="##any" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
                            </xsd:sequence> 
                           <xsl:apply-templates select=".//as:Attribute"/>                                
                        </xsl:when>
                        <xsl:otherwise>
                            <xsd:simpleContent>
                                <xsd:extension base="{if(contains($elementType,'|')) then substring-before($elementType,'|') else $elementType}">
                                    <xsl:apply-templates select=".//as:Attribute"/>
                                </xsd:extension>
                            </xsd:simpleContent>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsd:complexType>
            </xsl:when>
            <xsl:otherwise><!-- output the complex type definition -->
                <xsl:apply-templates select="." mode="complexType">
                    <xsl:with-param name="ref"  select="$ref"/>
                    <xsl:with-param name="prefix" select="$prefix"/>
                </xsl:apply-templates>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="as:Element" mode="parent">
        <xsl:variable name="repeat" select="if (as:Rule/as:constraint[starts-with(@action,'setLimit')]) then as:Rule/as:constraint[starts-with(@action,'setLimit')]/substring-before(substring-after(@action,'('),')') else 'unbounded'"></xsl:variable>
        <xsl:variable name="minOccurs" select="if (as:Rule/as:constraint[@action='makeOptional()']) then '0' else '1'"/>
        <xsl:variable name="maxOccurs" select="if (as:Rule/as:constraint[@action='makeRepeatable()']) then $repeat else '1'"/>
        <xsl:variable name="this" select="@name"/>
        <xsl:variable name="fixedValue" select="normalize-space(@exampleValue)"/>
        <xsl:variable name="that" select="if (contains($this,':')) then substring-after($this,':') else $this"/>
        <!-- xsl:comment>parent</xsl:comment -->
        <xsl:element name="xsd:element">
            <xsl:attribute name="name" select="$this"/>
            <xsl:if test="not(as:Rule/as:constraint[contains(@action,'setLength')])">
                <xsl:attribute name="type" select="$that"/>
            </xsl:if>
            <xsl:attribute name="minOccurs" select="$minOccurs"/>
            <xsl:attribute name="maxOccurs" select="w2e:maxOccurs($maxOccurs)"/>
            <xsl:if test="string-length($fixedValue) &gt; 0 and not(starts-with($fixedValue,'%') and ends-with($fixedValue,'%'))">
                <xsl:attribute name="fixed" select="$fixedValue"/>
            </xsl:if>
            <xsl:attribute name="id" select="generate-id()"/>
            <xsl:if test="count(camed:annotation) &gt; 0">
                <xsl:call-template name="outputAnnotation">
                    <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                </xsl:call-template>
            </xsl:if>
            <xsl:choose>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberRange')]">
                    <xsl:variable name="numberRangeRule" select="as:Rule/as:constraint[contains(@action,'setNumberRange')]/@action"/>
                    <xsl:variable name="numberRangeDetail" select="substring-before(substring-after($numberRangeRule[1],'('),')')"/>
                    <!-- xsl:message> - numberRange <xsl:value-of select="$numberRangeRule"/> | <xsl:value-of select="$numberRangeDetail"/></xsl:message -->
                    <xsl:variable name="numberType">
                        <xsl:choose>
                            <xsl:when test="contains($numberRangeDetail,'.99')">xsd:float</xsl:when>
                            <xsl:when test="contains($numberRangeDetail,'.99')">xsd:decimal</xsl:when>
                            <xsl:when test="contains($numberRangeDetail,'.9')">xsd:float</xsl:when>
                            <xsl:otherwise>xsd:integer</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsd:simpleType>
                        <xsd:restriction base="{$numberType}">
                            <xsl:call-template name="setRange">
                                <xsl:with-param name="rangeRule" select="as:Rule/as:constraint[contains(@action,'setNumberRange')]/@action"/>
                            </xsl:call-template>
                        </xsd:restriction>
                    </xsd:simpleType>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'setLength')]">
                    <xsd:simpleType>
                        <xsd:restriction base="xsd:string">
                            <xsl:call-template name="setLength">
                                <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                            </xsl:call-template>
                        </xsd:restriction>
                    </xsd:simpleType>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                    <xsd:simpleType>
                        <xsd:restriction base="xsd:string">
                            <xsl:call-template name="setPattern">
                                <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                            </xsl:call-template>
                        </xsd:restriction>
                    </xsd:simpleType>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')] or(as:Rule/as:constraint[contains(@action,'lookup(')])"><!-- Added restrictValues logic here 2012 -->
                    <xsl:variable name="valuesRule" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
                    <xsl:variable name="lookupRule" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/>
                    
                    <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
                    
                    <xsl:variable name="values" select="w2e:annoValues(.,$valuesRule,$lookupRule)" />
                    <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,$lookupRule)"/>
                    
                    <xsl:variable name="baseType">
                        <xsl:call-template name="findType">
                            <xsl:with-param name="rules" select="as:Rule"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="listBaseType">
                        <xsl:choose>
                            <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                            <xsl:otherwise>xsd:string</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsd:simpleType>
                        <xsd:restriction base="{$listBaseType}">
                            <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                <xsl:call-template name="setLength">
                                    <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                </xsl:call-template>
                            </xsl:if>
                            <xsl:if test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                                <xsl:call-template name="setPattern">
                                    <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                                </xsl:call-template>
                            </xsl:if>
                            <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                        </xsd:restriction>
                    </xsd:simpleType>
                </xsl:when>
            </xsl:choose>
        </xsl:element>
    </xsl:template>

    <xsl:template match="as:Element" mode="ns">
        <xsl:param name="ref"/>
        <xsl:param name="prefix"/>
        <!-- xsl:comment>
            Element: <xsl:value-of select="as:Element/as:Rule/as:constraint[@action='makeOptional()']/@action"/> 
            </xsl:comment -->
        <xsl:variable name="repeat" select="if (as:Rule/as:constraint[starts-with(@action,'setLimit')]) then as:Rule/as:constraint[starts-with(@action,'setLimit')]/substring-before(substring-after(@action,'('),')') else 'unbounded'"></xsl:variable>
        <xsl:variable name="minOccurs" select="if (as:Rule/as:constraint[@action='makeOptional()']) then '0' else '1'"/>
        <xsl:variable name="maxOccurs" select="if (as:Rule/as:constraint[@action='makeRepeatable()']) then $repeat else '1'"/>
        <xsl:variable name="this" select="@name"/>
        <xsl:variable name="fixedValue" select="normalize-space(@exampleValue)"/>
        <xsl:message> --- <xsl:value-of select="@name"/></xsl:message>
        <xsl:element name="xsd:element">
           <xsl:choose>
               <xsl:when test="$ref='true' and(contains(@name,':') and(not($prefix=substring-before(@name,':'))))">
                   <xsl:attribute name="ref" select="@name"/>
               </xsl:when>
               <xsl:when test="not(contains(@name,':'))"><!-- If there is no namespace then make a local declaration -->
                   <xsl:attribute name="name" select="@name"/>
               </xsl:when>
               <xsl:when test="as:Rule/as:constraint[contains(@action,'setLength')]">
                   <xsl:attribute name="name" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
               </xsl:when>
               <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberRange')]">
                   <xsl:attribute name="name" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
               </xsl:when>
               <xsl:otherwise>
                   <xsl:attribute name="name" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                   <xsl:if test="not(as:Rule/as:constraint[contains(@action,'setNumberRange')] 
                                       or(as:Rule/as:constraint[contains(@action,'setLength')]) 
                                       or(as:Rule/as:constraint[contains(@action,'setStringMask')]))">
                       <xsl:attribute name="type" select="@name"/>
                   </xsl:if>
               </xsl:otherwise>
           </xsl:choose>
            <xsl:attribute name="minOccurs" select="$minOccurs"/>
            <xsl:attribute name="maxOccurs" select="w2e:maxOccurs($maxOccurs)"/>
            <xsl:if test="string-length($fixedValue) &gt; 0 and not(starts-with($fixedValue,'%') and ends-with($fixedValue,'%'))">
                <xsl:attribute name="fixed" select="$fixedValue"/>
            </xsl:if>
            <xsl:attribute name="id" select="generate-id()"/>
            <xsl:if test="count(camed:annotation) &gt; 0">
                <xsl:call-template name="outputAnnotation">
                    <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                </xsl:call-template>
            </xsl:if>
            <xsl:choose>
                <xsl:when test="$ref='true' and(contains(@name,':') and(not($prefix=substring-before(@name,':'))))"/><!-- referenced type, so not a simply type -->
                <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberRange')]">
                    <xsl:variable name="numberRangeRule" select="as:Rule/as:constraint[contains(@action,'setNumberRange')]/@action"/>
                    <xsl:variable name="numberRangeDetail" select="substring-before(substring-after($numberRangeRule[1],'('),')')"/>
                    <!-- xsl:message> - numberRange <xsl:value-of select="$numberRangeRule"/> | <xsl:value-of select="$numberRangeDetail"/></xsl:message -->
                    <xsl:variable name="numberType">
                        <xsl:choose>
                            <xsl:when test="contains($numberRangeDetail,'.999')">xsd:float</xsl:when>
                            <xsl:when test="contains($numberRangeDetail,'.99')">xsd:decimal</xsl:when>
                            <xsl:when test="contains($numberRangeDetail,'.9')">xsd:float</xsl:when>
                            <xsl:otherwise>xsd:integer</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsd:simpleType>
                        <xsd:restriction base="{$numberType}">
                            <xsl:call-template name="setRange">
                                <xsl:with-param name="rangeRule" select="as:Rule/as:constraint[contains(@action,'setNumberRange')]/@action"/>
                            </xsl:call-template>
                        </xsd:restriction>
                    </xsd:simpleType>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'setLength')]">
                    <xsd:simpleType>
                        <xsd:restriction base="xsd:string">
                            <xsl:call-template name="setLength">
                                <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                            </xsl:call-template>
                        </xsd:restriction>
                    </xsd:simpleType>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                    <xsd:simpleType>
                        <xsd:restriction base="xsd:string">
                            <xsl:call-template name="setPattern">
                                <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                            </xsl:call-template>
                        </xsd:restriction>
                    </xsd:simpleType>
                </xsl:when>
                <xsl:when test="not(contains(@name,':')) and (as:Rule/as:constraint[contains(@action,'restrictValues')] or(as:Rule/as:constraint[contains(@action,'lookup(')] ))">
                    <!-- Added restrictValues / lookup logic here 2012 -->
                    <xsl:variable name="valuesRule" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
                    <xsl:variable name="lookupRule" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/>
                    
                    <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
                    
                    <xsl:variable name="values" select="w2e:annoValues(.,$valuesRule,$lookupRule)" />
                    <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,$lookupRule)"/>
                    
                    <xsl:variable name="baseType">
                        <xsl:call-template name="findType">
                            <xsl:with-param name="rules" select="as:Rule"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="listBaseType">
                        <xsl:choose>
                            <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                            <xsl:otherwise>xsd:string</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsd:simpleType>
                        <xsd:restriction base="{$listBaseType}">
                            <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                <xsl:call-template name="setLength">
                                    <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                </xsl:call-template>
                            </xsl:if>
                            <xsl:if test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                                <xsl:call-template name="setPattern">
                                    <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                                </xsl:call-template>
                            </xsl:if>
                            <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                        </xsd:restriction>
                    </xsd:simpleType>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:element>
    </xsl:template>

    <xsl:template match="as:Element" mode="xsd">
        <xsl:param name="ref"/>
        <xsl:param name="prefix"/>
        <xsl:variable name="QName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <!-- xsl:message exclude-result-prefixes="#all">base [<xsl:value-of select="$QName"></xsl:value-of>][<xsl:copy-of select="as:Rule" exclude-result-prefixes="#all"></xsl:copy-of>]</xsl:message -->
        <xsl:variable name="type">
            <xsl:call-template name="findType">
                <xsl:with-param name="rules" select="as:Rule"/>
            </xsl:call-template>
        </xsl:variable>

        <xsl:choose>
            <xsl:when test="count(tokenize($type,'xsd:')) &gt; 2"><!-- union of types -->
                <xsl:if test="$debug=1"><xsl:comment>base simpleType <xsl:value-of select="$QName"/><xsl:value-of select="'&#xD; &#xD;' "/>
                </xsl:comment></xsl:if>
                <xsl:element name="xsd:simpleType">
                    <xsl:attribute name="name" select="$QName"/>
                    <xsl:attribute name="id" select="generate-id()"/>
                    <xsl:element name="xsd:union">
                        <xsl:attribute name="memberTypes" select="substring-after(replace($type,'xsd:',' xsd:'),' ')"/>
                    </xsl:element>    
               </xsl:element>
            </xsl:when>
            <xsl:when test="count(as:Element) = 0"><!-- simple singleton element -->
                <xsl:variable name="thisElement">
                    <as:Element name="{concat($QName,'._.cam._.')}" namespacePrefix="{@namespacePrefix}" exampleValue="{@exampleValue}">
                        <xsl:copy-of select="as:Rule|as:Attribute|camed:annotation"/>
                    </as:Element>
                </xsl:variable>
                <xsl:if test="$debug=1"><xsl:comment>output simpleType for <xsl:value-of select="$QName"/><xsl:value-of select="'&#xD; &#xD;' "/></xsl:comment></xsl:if>
                <xsl:apply-templates select="$thisElement" mode="simpleType"/>
            </xsl:when>
            <xsl:otherwise>
        
            <xsl:if test="$debug=1"><xsl:comment>base complexType <xsl:value-of select="$QName"/><xsl:value-of select="'&#xD; &#xD;' "/>
            </xsl:comment></xsl:if>
            
                    
           <!-- Check if definition already exists - and if not make one -->  
           <!-- xsl:comment> checking for <xsl:value-of select="$QName"/> - [<xsl:value-of select="$prefix"/>]</xsl:comment -->          
           <xsl:choose>
            <xsl:when test="$prefix=''"/>
               <xsl:when test="count($duplicateElements/dup[@name=$QName]) &gt; 0"><xsl:if test="$debug=1"><xsl:comment>skip element already has definition <xsl:value-of select="$QName"/></xsl:comment></xsl:if></xsl:when>
               <xsl:when test="count($nsContentModels/pack//xsd:element[@name=$QName][@type='']) &gt; 0"><xsl:if test="$debug=1"><xsl:comment>skip element - duplicate packed definition <xsl:value-of select="$QName"/></xsl:comment></xsl:if></xsl:when>
               <xsl:when test="count($nsContentModels/pack[@type=$prefix]/xsd:complexType[@name=$QName]) &gt; 0"><xsl:if test="$debug=1"><xsl:comment>skip element - duplicate complexType definition <xsl:value-of select="$QName"/></xsl:comment></xsl:if></xsl:when>
            <xsl:otherwise>
                <xsl:element name="xsd:element">
                    <xsl:attribute name="name" select="$QName"/>
                    <xsl:attribute name="id" select="generate-id()"/>
                    <xsl:attribute name="type" select="concat($prefix,':',$QName)"/>
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:element name="xsd:complexType">
            <xsl:attribute name="name" select="$QName"/>
            <!-- xsl:attribute name="id" select="generate-id()"/ removed - don't need this on complexType - causes conflicts -->
            <xsl:choose>
                <xsl:when test="$type='xsd:any'">
                    <xsd:sequence>
                        <xsd:any namespace="##any" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
                    </xsd:sequence>           
                </xsl:when>
                <xsl:when
                    test="not(as:Element | as:Attribute) and(not(as:Rule/as:constraint[contains(@action,'restrictValues')]) or(as:Rule/as:constraint[contains(@action,'restrictValues(''true''|''false'')')]) or(as:Rule/as:constraint[contains(@action,'restrictValues(''true''| ''false'')')]) or(as:Rule/as:constraint[contains(@action,'lookup(')]))">
                    <xsd:simpleContent>
                        <xsl:choose>
                            <xsl:when test="contains($type,'|')">
                                <xsd:extension base="{substring-before($type,'|')}"/>
                                    <!-- cannot add restrictions to complexTypes 
                                        xsd:restriction
                                        xsd:totalDigits value="{substring-before(substring-after($type,'|'),';')}" />
                                    <xsd:fractionDigits value="{substring-after($type,';')}" />
                                    </xsd:restriction -->
                            </xsl:when>
                            <xsl:otherwise><xsd:extension base="{$type}"/></xsl:otherwise>
                        </xsl:choose>
                    </xsd:simpleContent>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'')] or(as:Rule/as:constraint[contains(@action,'lookup(')])">
                    <xsl:if test="not(as:Rule/as:constraint[contains(@action,'(''true''|''false'')')])">
                        <xsl:variable name="valuesRule" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
                        <xsl:variable name="lookupRule" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/>
                        
                        <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
                        
                        <xsl:variable name="values" select="w2e:annoValues(.,$valuesRule,$lookupRule)" />
                        <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,$lookupRule)"/>
                        
                        <xsl:variable name="baseType">
                            <xsl:call-template name="findType">
                                <xsl:with-param name="rules" select="as:Rule"/>
                            </xsl:call-template>
                        </xsl:variable>
                        <xsl:choose>
                          <xsl:when test="count(as:Attribute) = 0">
                              <xsl:variable name="listBaseType">
                                  <xsl:choose>
                                      <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                                      <xsl:otherwise>xsd:string</xsl:otherwise>
                                  </xsl:choose>
                              </xsl:variable>
                              <xsd:simpleType>
                                  <xsd:restriction base="{$listBaseType}">
                                      <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                          <xsl:call-template name="setLength">
                                              <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                          </xsl:call-template>
                                      </xsl:if>
                                      <xsl:if test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                                          <xsl:call-template name="setPattern">
                                              <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                                          </xsl:call-template>
                                      </xsl:if>
                                      <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                                  </xsd:restriction>
                              </xsd:simpleType>
                          </xsl:when>
                          <xsl:otherwise>
                              <xsl:variable name="listBaseType">
                                  <xsl:choose>
                                      <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                                      <xsl:otherwise>xsd:string</xsl:otherwise>
                                  </xsl:choose>
                              </xsl:variable>
                          <xsd:simpleContent>
                            <xsd:restriction base="xsd:anyType">
                                <xsd:simpleType>
                                    <xsd:restriction base="{$listBaseType}">
                                        <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                            <xsl:call-template name="setLength">
                                                <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                            </xsl:call-template>
                                        </xsl:if>
                                        <xsl:if test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                                            <xsl:call-template name="setPattern">
                                                <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                                            </xsl:call-template>
                                        </xsl:if>
                                        <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                                    </xsd:restriction>
                                </xsd:simpleType>
                                <xsl:for-each select="as:Attribute">
                                    <!-- xsl:variable name="attrMode" select="if(contains(@name,':')) then 'ns' else 'local'"/ -->
                                    <xsl:choose>
                                        <xsl:when test="contains(@name,':')">
                                            <xsl:apply-templates select="." mode="ns"/>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:apply-templates select="."/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsd:restriction>
                          </xsd:simpleContent>
                          </xsl:otherwise>
                        </xsl:choose>
                    </xsl:if>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'setLength')]">
                    <xsl:variable name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                    <xsl:variable name="minLength" select="substring-after(substring-before($lengthRule[1],'-'),'(')"/>
                    <xsl:variable name="maxLength">
                        <xsl:choose>
                            <xsl:when test="contains($lengthRule[1],'-')"><xsl:value-of select="substring-before(substring-after($lengthRule[1],'-'),')')"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="substring-before(substring-after($lengthRule[1],'('),')')"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsd:simpleContent>
                        <xsd:restriction base="xsd:anyType">
                            <xsd:simpleType>
                                <xsd:restriction base="xsd:string">
                                    <xsl:if test="not($minLength='')"><xsd:minLength value="{$minLength}"/></xsl:if>
                                    <xsl:if test="not($maxLength='')"><xsd:maxLength value="{$maxLength}"/></xsl:if>
                                </xsd:restriction>
                            </xsd:simpleType>
                            <xsl:for-each select="as:Attribute">
                                <xsl:variable name="attrMode" select="if(contains(@name,':')) then 'ns' else 'local'"/>
                                <xsl:choose>
                                    <xsl:when test="contains(@name,':')">
                                        <xsl:apply-templates select="." mode="ns"/>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:apply-templates select="."/>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:for-each>
                        </xsd:restriction>
                    </xsd:simpleContent>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'setNumberRange')]">
                    <xsd:simpleContent>
                        <xsd:restriction base="xsd:anyType">
                        <xsd:simpleType>
                            <xsd:restriction base="xsd:integer">
                                <xsl:call-template name="setRange">
                                    <xsl:with-param name="rangeRule" select="as:Rule/as:constraint[contains(@action,'setNumberRange')]/@action"/>
                                </xsl:call-template>
                            </xsd:restriction>
                        </xsd:simpleType>
                      </xsd:restriction>
                   </xsd:simpleContent>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                    <xsd:simpleContent>
                        <xsd:restriction base="xsd:anyType">
                            <xsd:simpleType>
                                <xsd:restriction base="xsd:string">
                                    <xsl:call-template name="setPattern">
                                        <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                                    </xsl:call-template>
                                </xsd:restriction>
                            </xsd:simpleType>
                        </xsd:restriction>
                    </xsd:simpleContent>
                </xsl:when>
                <xsl:otherwise>
                    <xsd:simpleContent>
                        <xsd:extension base="{if(contains($type,'|')) then substring-before($type,'|') else $type}">
                            <xsl:if test="as:Attribute">
                                <xsl:apply-templates select="as:Attribute" mode="type"/>
                            </xsl:if>
                        </xsd:extension>
                    </xsd:simpleContent>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:element>
        </xsl:otherwise>
       </xsl:choose>
    </xsl:template>

    <xsl:template match="as:Element">
        <xsl:param name="ref"/>
        <xsl:param name="prefix"/>
        <xsl:variable name="QName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="element-mode" select="if(contains(@name,':')) then 'ns' else 'local'"/>
        <xsl:choose>
            <xsl:when test="$element-mode='ns'">
                <xsl:apply-templates select="." mode="ns">
                    <xsl:with-param name="ref" select="$ref"/>
                    <xsl:with-param name="prefix" select="$prefix"/>
                </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="repeat" select="if (as:Rule/as:constraint[starts-with(@action,'setLimit')]) then as:Rule/as:constraint[starts-with(@action,'setLimit')]/substring-before(substring-after(@action,'('),')') else 'unbounded'"></xsl:variable>
                <xsl:variable name="minOccurs" select="if (as:Rule/as:constraint[@action='makeOptional()']) then '0' else '1'"/>
                <xsl:variable name="maxOccurs" select="if (as:Rule/as:constraint[@action='makeRepeatable()']) then $repeat else '1'"/>
                <xsl:variable name="type">
                    <xsl:call-template name="findType">
                        <xsl:with-param name="rules" select="as:Rule"/>
                    </xsl:call-template>
                </xsl:variable>
                <!-- xsl:comment>excludeChk [<xsl:value-of select="as:Rule/as:constraint[@action='excludeTree()']/@xpath"></xsl:value-of>]</xsl:comment -->
                <xsl:if test="not(as:Rule/as:constraint[@action='excludeTree()'])">
                    <xsl:if test="not(as:Rule/as:constraint[@action='excludeElement()'])">

                        <!-- check if change of namespace about to occur -->
                        <xsl:variable name="nsPrefix" select="substring-before(@name,':')"/>
                        <xsl:variable name="tempNames">
                            <xsl:for-each select="as:Element/@name">
                                <xsl:value-of select="concat(.,'|')"/>
                            </xsl:for-each>
                        </xsl:variable>
                        <xsl:variable name="elementNames" select="tokenize($tempNames,'\|')"/>
                        <xsl:variable name="changeNS">
                            <xsl:for-each select="$elementNames">
                                <xsl:if test="not($nsPrefix=substring-before(.,':'))">true</xsl:if>
                            </xsl:for-each>
                        </xsl:variable>

                        <xsl:choose>
                            <xsl:when test="contains($changeNS,'true')">
                                <!-- xsl:message>Change <xsl:value-of select="@name"></xsl:value-of></xsl:message -->
                                <xsl:apply-templates select="." mode="parent"/>
                                <!-- xsd:complexType name="{if(contains(@name,':')) then substring-after(@name,':') else @name}" id="{generate-id()}">
                <xsd:sequence>
                    <xsl:apply-templates select="as:Element" mode="type"/>
                    <xsl:apply-templates select="as:Attribute" mode="type"/>
                </xsd:sequence>
            </xsd:complexType -->
                                <!-- xsl:apply-templates select="as:Element"/ -->
                            </xsl:when>
                            <xsl:otherwise>
                                <!-- xsl:comment>constraint</xsl:comment -->
                                <xsl:element name="xsd:element">
                                    <xsl:attribute name="name" select="$QName"/>
                                    <xsl:attribute name="minOccurs" select="$minOccurs"/>
                                    <xsl:attribute name="maxOccurs" select="w2e:maxOccurs($maxOccurs)"/>
                                    <xsl:if test="not(as:Element | as:Attribute)">
                                        <xsl:if
                                            test="not(as:Rule/as:constraint[contains(@action,'restrictValues')] or(as:Rule/as:constraint[contains(@action,'setLength')]) or(as:Rule/as:constraint[contains(@action,'lookup(')])) 
                                            or(as:Rule/as:constraint[contains(@action,'restrictValues(''true''|''false'')')]) 
                                            or(as:Rule/as:constraint[contains(@action,'restrictValues(''true''| ''false'')')])">
                                            <xsl:if test="not(contains($type,'|'))">
                                                <xsl:variable name="typesChk" select="tokenize($type,'xsd:')"/>
                                                <xsl:choose>
                                                    <xsl:when test="count($typesChk) &gt; 1">
                                                        <!-- we found a valid xsd type so -->
                                                        <!-- assign a named simpleType here: the definition for which will be output later, either in an import type or as simpleType definition -->
                                                        <xsl:attribute name="type" select="if(contains($QName,':')) then $QName else concat($QName,'Definition')"/>
                                                    </xsl:when>
                                                    <xsl:otherwise><xsl:attribute name="type" select="$type"/></xsl:otherwise>
                                                </xsl:choose>  
                                            </xsl:if>                                    
                                        </xsl:if>
                                    </xsl:if>
                                    <xsl:attribute name="id" select="generate-id()"/>
                                    <xsl:if test="count(camed:annotation) &gt; 0">
                                        <xsl:call-template name="outputAnnotation">
                                            <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                                        </xsl:call-template>
                                    </xsl:if>
                                    
                                    <xsl:if test="contains($type,'|')">
                                        <xsd:simpleType>
                                            <xsd:restriction base="{if(starts-with($type,'xsd:float')) then 'xsd:decimal' else substring-before($type,'|')}">
                                                <xsd:totalDigits value="{substring-before(substring-after($type,'|'),';')}" />
                                                <xsd:fractionDigits value="{substring-after($type,';')}" />
                                            </xsd:restriction> 
                                        </xsd:simpleType>
                                    </xsl:if>
                                    <xsl:if test="as:Rule/as:constraint[contains(@action,'restrictValues')] or(as:Rule/as:constraint[contains(@action,'lookup(')])">
                                        <xsl:if test="not(as:Rule/as:constraint[contains(@action,'restrictValues(''true''|''false'')')])">
                                            <xsl:variable name="valuesRule" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
                                            <xsl:variable name="lookupRule" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/>
                                            
                                            <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
                                            
                                            <xsl:variable name="values" select="w2e:annoValues(.,$valuesRule,$lookupRule)" />
                                            <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,$lookupRule)"/>
                                            
                                            <xsl:variable name="baseType">
                                                <xsl:call-template name="findType">
                                                    <xsl:with-param name="rules" select="as:Rule"/>
                                                </xsl:call-template>
                                            </xsl:variable>
                                            <xsl:choose>
                                               <xsl:when test="count(as:Attribute) = 0">
                                                   <xsl:if test="$debug=1"><xsl:comment>Simple code list values</xsl:comment></xsl:if>
                                                   <xsl:variable name="listBaseType">
                                                       <xsl:choose>
                                                           <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                                                           <xsl:otherwise>xsd:string</xsl:otherwise>
                                                       </xsl:choose>
                                                   </xsl:variable>
                                                   <xsd:simpleType>
                                                       <xsd:restriction base="{$listBaseType}">
                                                           <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                                               <xsl:call-template name="setLength">
                                                                   <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                                               </xsl:call-template>
                                                           </xsl:if>
                                                           <xsl:if test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                                                               <xsl:call-template name="setPattern">
                                                                   <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                                                               </xsl:call-template>
                                                           </xsl:if>
                                                           <xsl:if test="not($listBaseType='xsd:boolean')">
                                                               <xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/>
                                                           </xsl:if>
                                                       </xsd:restriction>
                                                   </xsd:simpleType>
                                               </xsl:when>
                                               <xsl:otherwise>                                            
                                               <xsd:complexType>
                                                <xsd:simpleContent>
                                                  <xsd:restriction base="xsd:anyType">
                                                  <xsd:simpleType>
                                                  <xsd:restriction base="xsd:string">
                                                      <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                                          <xsl:call-template name="setLength">
                                                              <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                                          </xsl:call-template>
                                                      </xsl:if>
                                                      <xsl:if test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                                                          <xsl:call-template name="setPattern">
                                                              <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                                                          </xsl:call-template>
                                                      </xsl:if>
                                                      <xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/>
                                                  </xsd:restriction>
                                                  </xsd:simpleType>
                                                  <xsl:for-each select="as:Attribute">
                                                  <xsl:variable name="attrMode" select="if(contains(@name,':')) then 'ns' else 'local'"/>
                                                  <xsl:choose>
                                                  <xsl:when test="contains(@name,':')"><xsl:apply-templates select="." mode="ns"/></xsl:when>
                                                  <xsl:otherwise><xsl:apply-templates select="."/></xsl:otherwise>
                                                  </xsl:choose>
                                                  </xsl:for-each>
                                                  </xsd:restriction>
                                                </xsd:simpleContent>
                                            </xsd:complexType>
                                                </xsl:otherwise>
                                               </xsl:choose>
                                        </xsl:if>
                                    </xsl:if>
                                    <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')] and(not(as:Element)) and(not(as:Rule/as:constraint[contains(@action,'restrictValues')]) or(as:Rule/as:constraint[contains(@action,'lookup(')]))">
                                        <xsd:complexType>
                                            <xsd:simpleContent>
                                                <xsd:restriction base="xsd:anyType">
                                                    <xsd:simpleType>
                                                        <xsd:restriction base="xsd:string">
                                                            <xsl:call-template name="setLength">
                                                                <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                                            </xsl:call-template>
                                                        </xsd:restriction>
                                                    </xsd:simpleType>
                                                    <xsl:for-each select="as:Attribute">
                                                        <xsl:variable name="attrMode" select="if(contains(@name,':')) then 'ns' else 'local'"/>
                                                        <xsl:choose>
                                                            <xsl:when test="contains(@name,':')"><xsl:apply-templates select="." mode="ns"/></xsl:when>
                                                            <xsl:otherwise><xsl:apply-templates select="."/></xsl:otherwise>
                                                        </xsl:choose>
                                                    </xsl:for-each>
                                                </xsd:restriction>
                                            </xsd:simpleContent>
                                        </xsd:complexType>
                                    </xsl:if>
                                    <xsl:if test="as:Element">
                                        <xsl:if test="$debug=1"><xsl:comment>Element definition <xsl:value-of select="@name"/><xsl:value-of select="'&#xD; &#xD;' "/>
                                        </xsl:comment></xsl:if>
                                            <!-- Main Output mode for all other structure members -->
                                            <xsd:complexType>
                                                <xsd:sequence>
                                                    <xsl:choose>
                                                        <xsl:when test="as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]">
                                                            <xsl:variable name="choiceRules" select="distinct-values(as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]/@action)"/>
                                                            <xsl:variable name="xsdChoice">
                                                                <xsd:choice>
                                                                    <xsl:for-each select="as:Element">
                                                                        <xsl:if test="contains($choiceRules[1],concat('''',@name,''''))">
                                                                            <xsl:apply-templates select="."/>
                                                                        </xsl:if>
                                                                    </xsl:for-each>
                                                                </xsd:choice>
                                                            </xsl:variable>
                                                            <xsl:for-each select="as:Element">
                                                                <xsl:variable name="thisName"
                                                                    select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                                                                <xsl:variable name="choiceItem">
                                                                    <xsl:call-template name="chkChoiceItem">
                                                                        <xsl:with-param name="name" select="$thisName"/>
                                                                        <xsl:with-param name="xsdChoice" select="$xsdChoice"/>
                                                                    </xsl:call-template>
                                                                </xsl:variable>
                                                                <xsl:choose>
                                                                    <xsl:when test="$choiceItem='none'">
                                                                        <xsl:apply-templates select="."/>
                                                                    </xsl:when>
                                                                    <xsl:when test="$choiceItem='skip'"/>
                                                                    <xsl:otherwise>
                                                                        <xsl:copy-of select="$choiceItem"/>
                                                                    </xsl:otherwise>
                                                                </xsl:choose>
                                                            </xsl:for-each>
                                                        </xsl:when>
                                                        <xsl:otherwise><xsl:apply-templates select="as:Element"/></xsl:otherwise>
                                                    </xsl:choose>
                                             </xsd:sequence>
                                      
                                            <xsl:for-each select="as:Attribute">
                                                <xsl:variable name="attrMode"
                                                  select="if(contains(@name,':')) then 'ns' else 'local'"/>
                                                <xsl:choose>
                                                  <xsl:when test="contains(@name,':')">
                                                  <xsl:apply-templates select="." mode="ns"/>
                                                  </xsl:when>
                                                  <xsl:otherwise>
                                                  <xsl:apply-templates select="."/>
                                                  </xsl:otherwise>
                                                </xsl:choose>
                                            </xsl:for-each>
                                            </xsd:complexType>
                                    </xsl:if>
                                    <xsl:if test="as:Attribute and(not(as:Element)) and(not(as:Rule/as:constraint[contains(@action,'restrictValues')]) or(as:Rule/as:constraint[contains(@action,'lookup(')])) and(not(as:Rule/as:constraint[contains(@action,'set')]))"><!-- Element with attributes and simple content model - no restrictions -->                                 
                                        <xsl:variable name="elementType">
                                            <xsl:call-template name="findType">
                                                <xsl:with-param name="rules" select="as:Rule"/>
                                            </xsl:call-template>
                                        </xsl:variable>
                                        <xsd:complexType>
                                            <xsl:message> - <xsl:value-of select="@name"/> of type <xsl:value-of select="$elementType"/> with <xsl:value-of select="count(.//as:Attribute)"/> attributes.</xsl:message>
                                            <!-- xsl:comment> element of type <xsl:value-of select="$elementType"/> with <xsl:value-of select="count(.//as:Attribute)"/> attributes.</xsl:comment -->
                                            <xsl:choose>
                                                <xsl:when test="$elementType = 'xsd:any'">
                                                    <xsd:annotation>
                                                        <xsd:documentation>The any type for <xsl:value-of select="@name"/>. </xsd:documentation>
                                                    </xsd:annotation>
                                                    <xsd:sequence>
                                                        <xsd:any namespace="##any" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
                                                    </xsd:sequence> 
                                                    <xsl:apply-templates select=".//as:Attribute"/>                                
                                                </xsl:when>
                                                <xsl:otherwise>
                                                    <xsd:simpleContent>
                                                        <xsd:extension base="{if(contains($elementType,'|')) then substring-before($elementType,'|') else $elementType}">
                                                            <xsl:apply-templates select=".//as:Attribute"/>
                                                        </xsd:extension>
                                                    </xsd:simpleContent>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                        </xsd:complexType>
                                      </xsl:if>
                                </xsl:element>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:if>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="as:Element" mode="complexType">
        <xsl:param name="ref"/>
        <xsl:param name="prefix"/>
       
        <xsl:variable name="QName"
            select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="type">
            <xsl:call-template name="findType">
                <xsl:with-param name="rules" select="as:Rule"/>
            </xsl:call-template>
        </xsl:variable>
        <!-- xsl:message>QName [<xsl:value-of select="$QName"/>]-[<xsl:value-of select="@name"/>]</xsl:message -->
        <xsl:if test="not(as:Rule/as:constraint[@action='excludeTree()'])">
            <xsl:if test="not(as:Rule/as:constraint[@action='excludeElement()'])">
                <!-- xsl:comment>complex</xsl:comment -->
                <xsl:element name="xsd:element"><!-- new definition -->
                    <xsl:attribute name="name" select="$QName"/>
                    <xsl:attribute name="type" select="@name"/>
                    <xsl:if test="count(camed:annotation) &gt; 0">
                        <xsl:call-template name="outputAnnotation">
                            <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                        </xsl:call-template>
                    </xsl:if>
                </xsl:element>
                
                <xsl:if test="$debug=1"><xsl:comment>Main line <xsl:value-of select="@name"/><xsl:value-of select="'&#xD; &#xD;' "/>
                </xsl:comment></xsl:if>
                
                <xsl:element name="xsd:complexType">
                    <xsl:attribute name="name" select="$QName"/>
                    <xsl:attribute name="id" select="generate-id()"/>
                    <xsl:if test="not(as:Element | as:Attribute)">
                        <xsl:if
                            test="not(as:Rule/as:constraint[contains(@action,'restrictValues')] or(as:Rule/as:constraint[contains(@action,'lookup(')])) or(as:Rule/as:constraint[contains(@action,'restrictValues(''true''|''false'')')]) or(as:Rule/as:constraint[contains(@action,'restrictValues(''true''| ''false'')')])">
                            <xsd:simpleContent>
                                <xsl:choose>
                                    <xsl:when test="contains($type,'|')">
                                        <xsd:restriction base="{if(starts-with($type,'xsd:float')) then 'xsd:decimal' else substring-before($type,'|')}">
                                            <xsd:totalDigits value="{substring-before(substring-after($type,'|'),';')}" />
                                            <xsd:fractionDigits value="{substring-after($type,';')}" />
                                        </xsd:restriction> 
                                    </xsl:when>
                                    <xsl:otherwise><xsd:extension base="{$type}"/></xsl:otherwise>
                                </xsl:choose>
                            </xsd:simpleContent>
                        </xsl:if>
                    </xsl:if>
                    <xsl:if test="as:Rule/as:constraint[contains(@action,'restrictValues')] or(as:Rule/as:constraint[contains(@action,'lookup(')])">
                        <xsl:if test="not(as:Rule/as:constraint[contains(@action,'restrictValues(''true''|''false'')')])">
                            <xsl:variable name="valuesRule" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
                            <xsl:variable name="lookupRule" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/>
                            
                            <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
                            
                            <xsl:variable name="values" select="w2e:annoValues(.,$valuesRule,$lookupRule)" />
                            <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,$lookupRule)"/>
                            
                            <xsl:variable name="baseType">
                                <xsl:call-template name="findType">
                                    <xsl:with-param name="rules" select="as:Rule"/>
                                </xsl:call-template>
                            </xsl:variable>
                            <xsl:variable name="listBaseType">
                                <xsl:choose>
                                    <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                                    <xsl:otherwise>xsd:string</xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>
                            <xsl:choose>
                                <xsl:when test="count(as:Attribute) = 0">
                                    <xsd:simpleType>
                                        <xsd:restriction base="{$listBaseType}">
                                            <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                                <xsl:call-template name="setLength">
                                                    <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                                </xsl:call-template>
                                            </xsl:if>
                                            <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                                        </xsd:restriction>
                                    </xsd:simpleType>
                                </xsl:when>
                             <xsl:otherwise>
                             <xsd:simpleContent>
                                <xsd:restriction base="xsd:anyType">
                                    <xsd:simpleType>
                                        <xsd:restriction base="{$listBaseType}">
                                            <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                                    <xsl:call-template name="setLength">
                                                        <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                                    </xsl:call-template>
                                            </xsl:if>
                                            <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                                        </xsd:restriction>
                                    </xsd:simpleType>
                                    <xsl:for-each select="as:Attribute">
                                        <xsl:variable name="attrMode" select="if(contains(@name,':')) then 'ns' else 'local'"/>
                                        <xsl:choose>
                                            <xsl:when test="contains(@name,':')">
                                                <xsl:apply-templates select="." mode="ns"/>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:apply-templates select="."/>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:for-each>
                                </xsd:restriction>
                             </xsd:simpleContent>
                             </xsl:otherwise>
                             </xsl:choose>
                        </xsl:if>
                    </xsl:if>
                    <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                        <xsl:variable name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                        <xsl:variable name="minLength" select="substring-after(substring-before($lengthRule[1],'-'),'(')"/>
                        <xsl:variable name="maxLength">
                            <xsl:choose>
                                <xsl:when test="contains($lengthRule[1],'-')"><xsl:value-of select="substring-before(substring-after($lengthRule[1],'-'),')')"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="substring-before(substring-after($lengthRule[1],'('),')')"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                            <xsd:simpleContent>
                                <xsd:restriction base="xsd:anyType">
                                    <xsd:simpleType>
                                        <xsd:restriction base="xsd:string">
                                            <xsl:if test="not($minLength='')"><xsd:minLength value="{$minLength}"/></xsl:if>
                                            <xsl:if test="not($maxLength='')"><xsd:maxLength value="{$maxLength}"/></xsl:if>
                                        </xsd:restriction>
                                    </xsd:simpleType>
                                    <xsl:for-each select="as:Attribute">
                                        <xsl:variable name="attrMode" select="if(contains(@name,':')) then 'ns' else 'local'"/>
                                        <xsl:choose>
                                            <xsl:when test="contains(@name,':')">
                                                <xsl:apply-templates select="." mode="ns"/>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:apply-templates select="."/>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:for-each>
                                </xsd:restriction>
                            </xsd:simpleContent>
                    </xsl:if>
                    <xsl:if test="as:Element">
                        <xsd:sequence>
                            <xsl:choose>
                                <xsl:when test="as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]">
                                    <xsl:variable name="choiceRules" select="distinct-values(as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]/@action)"/>
                                    <xsl:variable name="elements" select="as:Element"/>
                                    <xsl:variable name="xsdChoice">
                                        <xsl:for-each select="$choiceRules">
                                            <xsl:variable name="thisRule" select="."/>
                                            <xsd:choice>
                                                <xsl:for-each select="$elements">
                                                  <xsl:if test="contains($thisRule,concat('''',@name,''''))">
                                                  <xsl:apply-templates select="."/>
                                                  </xsl:if>
                                                </xsl:for-each>
                                            </xsd:choice>
                                        </xsl:for-each>
                                    </xsl:variable>
                                    <xsl:for-each select="$elements">
                                        <xsl:variable name="thisName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                                        <xsl:variable name="choiceItem">
                                            <xsl:call-template name="chkChoiceItem">
                                                <xsl:with-param name="name" select="$thisName"/>
                                                <xsl:with-param name="xsdChoice" select="$xsdChoice"/>
                                            </xsl:call-template>
                                        </xsl:variable>
                                        <xsl:choose>
                                            <xsl:when test="$choiceItem='none'">
                                                <xsl:apply-templates select="."/>
                                            </xsl:when>
                                            <xsl:when test="$choiceItem='skip'"/>
                                            <xsl:otherwise>
                                                <xsl:copy-of select="$choiceItem"/>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:for-each>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:apply-templates select="as:Element">
                                        <xsl:with-param name="ref" select="$ref"/>
                                        <xsl:with-param name="prefix" select="$prefix"/>
                                    </xsl:apply-templates>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsd:sequence>
                        <xsl:for-each select="as:Attribute">
                            <xsl:variable name="attrMode"
                                select="if(contains(@name,':')) then 'ns' else 'local'"/>
                            <xsl:choose>
                                <xsl:when test="contains(@name,':')">
                                    <xsl:apply-templates select="." mode="ns"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:apply-templates select="."/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:for-each>
                    </xsl:if>
                    <!-- xsl:if test="as:Attribute and(not(as:Element))">
                        <xsd:complexType>
                        <xsl:apply-templates select="as:Attribute"/>
                        </xsd:complexType>
                        </xsl:if -->
                </xsl:element>
            </xsl:if>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="as:Element" mode="simpleType">
        <xsl:variable name="nsPrefix" select="substring-before(@name,':')"/>
        <xsl:variable name="definitionSuffix" select="if(ends-with(@name,'._.cam._.')) then '' else 'Definition'"></xsl:variable>
        <xsl:variable name="itemName" select="replace(if(contains(@name,':')) then substring-after(@name,':') else @name,'._.cam._.','')"/>
        <xsl:variable name="valuesRule" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
        <xsl:variable name="lookupRule" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/>
        <xsl:variable name="lookupName" select="substring-before(substring-after($lookupRule[1],'lookup('),')')"/>
        
        <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
        
        <xsl:variable name="values" select="w2e:annoValues(.,$valuesRule,$lookupRule)" />
        <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,$lookupRule)"/>
        
        <xsl:variable name="baseType">
            <xsl:call-template name="findType">
                <xsl:with-param name="rules" select="as:Rule"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="as:Element"><xsl:apply-templates select="as:Element" mode="simpleType"/></xsl:when>
            <xsl:when test="string-length($nsPrefix) &gt; 0"><!-- ignore items not in default namespace --></xsl:when>
            <xsl:otherwise>
                <!-- Work out base type from rules definitions -->
                <xsl:variable name="type">
                    <xsl:call-template name="findType">
                        <xsl:with-param name="rules" select="as:Rule"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="typesChk" select="tokenize($type,'xsd:')"/>
               <xsl:choose>
                   <xsl:when test="$type='xsd:any'">
                       <xsd:complexType name="{concat($itemName,$definitionSuffix)}">
                           <xsd:annotation>
                               <xsd:documentation>The any type for <xsl:value-of select="$itemName"/>. </xsd:documentation>
                           </xsd:annotation>
                           <xsd:sequence>
                               <xsd:any namespace="##any" processContents="lax" minOccurs="0" maxOccurs="unbounded"/>
                           </xsd:sequence>   
                       </xsd:complexType>
                   </xsl:when>
                   <xsl:otherwise>
                
                <!-- Now output xsd definition for that type -->
                    <xsd:simpleType name="{concat($itemName,$definitionSuffix)}">
                        <xsd:annotation>
                            <xsd:documentation>The type for <xsl:value-of select="$itemName"/>. </xsd:documentation>
                        </xsd:annotation>
                    <xsl:choose>
                        <xsl:when test="count($typesChk) &gt; 2">
                            <xsl:variable name="typesList"><xsl:for-each select="$typesChk"><xsl:if test="string-length(.) &gt; 0"><xsl:value-of select="concat('xsd:',.,'|')"/></xsl:if></xsl:for-each></xsl:variable>
                            
                            <xsl:variable name="typeDecl" exclude-result-prefixes="camed as dblook look">
                                <xsl:choose>
                                    <xsl:when test="count(tokenize($typesList,'\|')) &gt; 1">
                                            <xsd:union>
                                                <xsl:attribute name="memberTypes" select="replace($typesList,'\|',' ')"/>
                                            </xsd:union>
                                    </xsl:when>
                                    <xsl:otherwise>
                                            <xsd:restriction>
                                                <xsl:attribute name="base" select="$typesList"/>
                                            </xsd:restriction>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>
                            <xsl:copy-of select="$typeDecl"/>
                        </xsl:when>
                        <xsl:when test="not(contains($type,'|')) and (count($values) &gt; 0)">
                            <!-- Determine base type -->
                            <xsl:variable name="listBaseType">
                                <xsl:choose>
                                    <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                                    <xsl:otherwise>xsd:string</xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>
                            <xsd:restriction base="{$listBaseType}">
                                <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                    <xsl:call-template name="setLength">
                                        <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                    </xsl:call-template>
                                </xsl:if>
                                <xsl:if test="as:Rule/as:constraint[contains(@action,'setStringMask')]">
                                    <xsl:call-template name="setPattern">
                                        <xsl:with-param name="pattern" select="as:Rule/as:constraint[contains(@action,'setStringMask')]/@action"/>
                                    </xsl:call-template>
                                </xsl:if>
                                <xsl:if test="not($listBaseType='xsd:boolean') and( (string-length($lookupName) &gt; 0) or(as:Rule/as:constraint[contains(@action,'restrictValues')]/@action) )">
                                    <xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/>
                                </xsl:if>
                            </xsd:restriction>
                        </xsl:when>
                        <xsl:when test="contains($type,'|')">
                            <xsd:restriction base="{if(starts-with($type,'xsd:float')) then 'xsd:decimal' else substring-before($type,'|')}">
                                <xsd:totalDigits value="{substring-before(substring-after($type,'|'),';')}" />
                                <xsd:fractionDigits value="{substring-after($type,';')}" />
                            </xsd:restriction> 
                        </xsl:when>
                         <xsl:otherwise><xsd:restriction base="{$type}"/></xsl:otherwise>
                    </xsl:choose>
                </xsd:simpleType>
                   </xsl:otherwise>
               </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="as:Element" mode="root"> <!-- This is applied when root element has a namespace (targetNamespace) -->
        <xsl:variable name="nsPrefix" select="substring-before(@name,':')"/>
        <xsl:variable name="tempNames">
            <xsl:for-each select="as:Element/@name">
                <xsl:value-of select="concat(.,'|')"/>
            </xsl:for-each>
        </xsl:variable>
        <xsl:variable name="elementNames" select="tokenize($tempNames,'\|')"/>
        <xsl:variable name="changeNS">
            <xsl:for-each select="$elementNames">
                <xsl:if test="not($nsPrefix=substring-before(.,':'))"><xsl:value-of
                    select="substring-before(.,':')"/>|true|</xsl:if>
            </xsl:for-each>
        </xsl:variable>
        
        <xsl:choose>
            <xsl:when test="contains($changeNS,'|true|')">
                <!-- xsl:message>Change complexType <xsl:value-of select="@name"></xsl:value-of></xsl:message -->
                <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                <xsl:variable name="outputType">
                    <xsl:choose>
                        <xsl:when test="not($duplicateElements/dup[@name=$itemName])">true</xsl:when>
                        <xsl:otherwise>
                            <xsl:variable name="chkPath" select="$duplicateElements/dup[@name=$itemName][1]/@path"/>
                            <xsl:if test="$chkPath=@path">true</xsl:if><!-- only process first duplicate -->
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:if test="$outputType='true'">
                    <xsd:complexType name="{concat(if(contains(@name,':')) then substring-after(@name,':') else @name,$rootSuffix)}">
                        <xsd:annotation>
                            <xsd:documentation>The root container element for <xsl:value-of select="@name"/>
                                content model. </xsd:documentation>
                        </xsd:annotation>
                        <xsl:message> --- <xsl:value-of select="@name"/></xsl:message>
                        <xsl:variable name="anyChoices" select="count(as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')])"/>
                        <xsl:choose>
                            <xsl:when test="$anyChoices &gt; 0">
                                <!-- structure contains choices -->
                                <xsd:sequence>
                                <xsl:variable name="choiceRules" select="distinct-values(as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]/@action)"/>
                                <xsl:variable name="xsdChoice">
                                    <xsd:choice>
                                        <xsl:for-each select="as:Element">
                                            <xsl:if test="contains($choiceRules[1],concat('''',@name,''''))">
                                                <xsl:call-template name="outputRootElement">
                                                    <xsl:with-param name="elementName" select="@name"/>
                                                    <xsl:with-param name="changeNS" select="$changeNS"/>
                                                    <xsl:with-param name="rules" select="as:Rule"/>
                                                </xsl:call-template>
                                                <!-- xsd:element ref="{if(contains(@name,':')) then @name else concat(substring-before($changeNS,'|'),':',@name)}"/ -->
                                            </xsl:if>
                                        </xsl:for-each>
                                    </xsd:choice>
                                </xsl:variable>
                                <!-- See if elements start before the choice starts -->
                                <xsl:variable name="firstChoice" select="$xsdChoice/xsd:choice/xsd:element[1]/@ref"/>
                                <xsl:variable name="frontElements">
                                    <xsl:for-each select="as:Element"><xsl:if test="@name=$firstChoice"><xsl:value-of select="position()"/></xsl:if></xsl:for-each>
                                </xsl:variable>
                                <xsl:for-each select="as:Element">
                                    <xsl:if test="position() &lt; $frontElements">
                                        <xsl:call-template name="outputRootElement">
                                            <xsl:with-param name="elementName" select="@name"/>
                                            <xsl:with-param name="changeNS" select="$changeNS"/>
                                            <xsl:with-param name="rules" select="as:Rule"/>
                                        </xsl:call-template>
                                    </xsl:if>
                                </xsl:for-each>
                                <xsl:variable name="lastChoice" select="count($xsdChoice/xsd:choice/xsd:element) + $frontElements"/>
                                <xsl:copy-of select="$xsdChoice"/>
                                <!-- Now handle those following items that are not part of the structure choice, if any -->
                                <xsl:for-each select="as:Element">
                                    <xsl:variable name="thisName" select="@name"/>
                                    <xsl:choose>
                                        <xsl:when test="position() &lt; $lastChoice"></xsl:when><!-- skip over choice items -->
                                        <xsl:when test="$xsdChoice/xsd:choice/xsd:element[@ref=$thisName]"/><!-- Already part of choice, so skip -->
                                        <xsl:otherwise>
                                            <xsl:call-template name="outputRootElement">
                                                <xsl:with-param name="elementName" select="@name"/>
                                                <xsl:with-param name="changeNS" select="$changeNS"/>
                                                <xsl:with-param name="rules" select="as:Rule"/>
                                            </xsl:call-template>
                                            <!-- xsd:element ref="{if(contains(@name,':')) then @name else concat(substring-before($changeNS,'|'),':',@name)}"/ --></xsl:otherwise>
                                    </xsl:choose>
                                </xsl:for-each>
                                </xsd:sequence>
                            </xsl:when>
                            <xsl:otherwise><!-- simple root structure definition -->
                                <xsd:sequence>
                                    <xsl:for-each select="as:Element">
                                        <xsl:call-template name="outputRootElement">
                                            <xsl:with-param name="elementName" select="@name"/>
                                            <xsl:with-param name="changeNS" select="$changeNS"/>
                                            <xsl:with-param name="rules" select="as:Rule"/>
                                        </xsl:call-template>
                                      <!-- xsd:element ref="{if(contains(@name,':')) then @name else concat(substring-before($changeNS,'|'),':',@name)}"/ -->
                                    </xsl:for-each>
                                </xsd:sequence>
                            </xsl:otherwise>
                        </xsl:choose>              
                        <xsl:apply-templates select="as:Attribute"/><!-- add any root node local attribute definitions inline here -->
                    </xsd:complexType>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise><!-- This should not be reached, but left here as catch all -->
                <xsl:comment>WARNING: Root handling encountered unexpected structure scenario!</xsl:comment>
                <xsl:apply-templates select="as:Element" mode="group"/>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>

    <xsl:template match="as:Element" mode="group">
        <!-- check if change of namespace about to occur -->
        <!-- xsl:message>parent [<xsl:value-of select="@name"/>]</xsl:message -->
        <xsl:variable name="nsPrefix" select="substring-before(@name,':')"/>
        <xsl:variable name="tempNames">
            <xsl:for-each select="as:Element/@name">
                <xsl:value-of select="concat(.,'|')"/>
            </xsl:for-each>
        </xsl:variable>
        <xsl:variable name="elementNames" select="tokenize($tempNames,'\|')"/>
        <xsl:variable name="changeNS">
            <xsl:for-each select="$elementNames">
                <xsl:if test="not($nsPrefix=substring-before(.,':'))"><xsl:value-of
                        select="substring-before(.,':')"/>|true|</xsl:if>
            </xsl:for-each>
        </xsl:variable>

        <xsl:choose>
            <xsl:when test="contains($changeNS,'|true|')">
                <!-- xsl:message>Change complexType <xsl:value-of select="@name"></xsl:value-of></xsl:message -->
                <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                <xsl:variable name="outputType">
                    <xsl:choose>
                        <xsl:when test="not($duplicateElements/dup[@name=$itemName])">true</xsl:when>
                        <xsl:otherwise>
                            <xsl:variable name="chkPath" select="$duplicateElements/dup[@name=$itemName][1]/@path"/>
                            <xsl:if test="$chkPath=@path">true</xsl:if><!-- only process first duplicate -->
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:if test="$outputType='true'">
                  <xsl:message> --- <xsl:value-of select="@name"/></xsl:message>
                  <xsd:complexType name="{if(contains(@name,':')) then substring-after(@name,':') else @name}">
                    <xsd:annotation>
                        <xsd:documentation>The container element for <xsl:value-of select="@name"/>
                            content model. </xsd:documentation>
                    </xsd:annotation>
                    <xsd:complexContent>
                        <xsd:extension base="{if(contains(@name,':')) then @name else concat(substring-before($changeNS,'|'),':',@name)}"/>
                    </xsd:complexContent>
                  </xsd:complexType>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="as:Element" mode="group"/>
            </xsl:otherwise>
        </xsl:choose>

    </xsl:template>

    <xsl:template match="as:Element" mode="nsModels">
        <!-- check if change of namespace about to occur -->
        <!-- xsl:message>parent [<xsl:value-of select="@name"/>]</xsl:message -->
        <xsl:variable name="nsPrefix" select="substring-before(@name,':')"/>
        <xsl:variable name="tempNames">
            <xsl:for-each select="as:Element/@name">
                <xsl:value-of select="concat(.,'|')"/>
            </xsl:for-each>
        </xsl:variable>
        <xsl:variable name="elementNames" select="tokenize($tempNames,'\|')"/>
        <xsl:variable name="changeNS">
            <xsl:for-each select="$elementNames">
                <xsl:if test="not($nsPrefix=substring-before(.,':'))"><xsl:value-of select="substring-before(.,':')"/>|true|</xsl:if>
            </xsl:for-each>
        </xsl:variable>
        <xsl:message> -- <xsl:value-of select="@name"/></xsl:message>
        <!-- xsl:message><xsl:value-of select="$nsPrefix"/>[<xsl:value-of select="$changeNS"/>]</xsl:message -->
        <xsl:choose>
            <xsl:when test="as:Rule//as:constraint[@action='excludeTree()']"></xsl:when>
            <xsl:when test="as:Rule//as:constraint[@action='excludeElement()']"></xsl:when>            
            <xsl:when test="contains($changeNS,'|true|')">
                <xsl:message> --- ns <xsl:value-of select="if($nsPrefix='') then substring-before($changeNS,'|') else $nsPrefix"/></xsl:message>
                <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                <xsl:variable name="outputType">
                    <xsl:choose>
                        <xsl:when test="not($duplicateElements/dup[@name=$itemName])">true</xsl:when>
                        <xsl:otherwise>
                            <xsl:variable name="chkPath" select="$duplicateElements/dup[@name=$itemName][1]/@path"/>
                            <xsl:if test="$chkPath=@path">true</xsl:if><!-- only process first duplicate -->
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:if test="$outputType='true'">
                <pack type="{if($nsPrefix='') then substring-before($changeNS,'|') else $nsPrefix}">
                    <xsd:complexType name="{if(contains(@name,':')) then substring-after(@name,':') else @name}">
                        <xsl:if test="$debug=1"><xsl:comment>Pack definition <xsl:value-of select="@name"/><xsl:value-of select="'&#xD; &#xD;' "/>
                        </xsl:comment></xsl:if>
                        <xsd:sequence>
                            <xsl:choose>
                                <xsl:when test="as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]">
                                    <xsl:variable name="choiceRules" select="distinct-values(as:Element/as:Rule/as:constraint[contains(@action,'setChoice(')]/@action)"/>
                                    <xsl:variable name="xsdChoice">
                                        <xsd:choice>
                                            <xsl:for-each select="as:Element">
                                                <xsl:if test="contains($choiceRules[1],concat('''',@name,''''))">
                                                    <xsl:apply-templates select="."/>
                                                </xsl:if>
                                            </xsl:for-each>
                                        </xsd:choice>
                                    </xsl:variable>
                                    <xsl:for-each select="as:Element">
                                        <xsl:message> --- <xsl:value-of select="@name"/></xsl:message>
                                        <xsl:variable name="thisName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                                        <xsl:variable name="choiceItem">
                                            <xsl:call-template name="chkChoiceItem">
                                                <xsl:with-param name="name" select="$thisName"/>
                                                <xsl:with-param name="xsdChoice" select="$xsdChoice"/>
                                            </xsl:call-template>
                                        </xsl:variable>
                                        <xsl:choose>
                                            <xsl:when test="$choiceItem='none'">
                                                <xsl:apply-templates select=".">
                                                    <xsl:with-param name="ref" select="'true'"/>
                                                    <xsl:with-param name="prefix" select="substring-before($changeNS,'|')"/>
                                                </xsl:apply-templates>
                                            </xsl:when>
                                            <xsl:when test="$choiceItem='skip'"/>
                                            <xsl:otherwise>
                                                <xsl:copy-of select="$choiceItem"/>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:for-each>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:variable name="referenceMode" select="if(as:Element/as:Rule/as:constraint[contains(@action,'datatype(any)')]) then 'false' else 'true'"/>
                                    <xsl:message> --- <xsl:value-of select="@name"/></xsl:message>
                                    <xsl:apply-templates select="as:Element" mode="ns">
                                        <xsl:with-param name="ref" select="$referenceMode"/>
                                        <xsl:with-param name="prefix" select="substring-before($changeNS,'|')"/>
                                    </xsl:apply-templates>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsd:sequence>
                        <xsl:apply-templates select="as:Attribute" mode="model"/>
                    </xsd:complexType>    
                </pack>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="as:Element" mode="nsModels"/>
            </xsl:otherwise>
        </xsl:choose>

    </xsl:template>

    <xsl:template match="as:Attribute" mode="model">
        <!-- decide type referencing needed -->
        <xsl:if test="not(as:Rule/as:constraint[@action='excludeAttribute()'])">
            <xsl:choose>
                <xsl:when test="contains(@name,':')">
                    <xsl:apply-templates select="." mode="ns"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="." mode="type"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template match="as:Attribute" mode="ns">
        <!-- import definition ref by namespace only -->
        <!-- xsl:message>attr [<xsl:value-of select="@name"/>] <xsl:copy-of select="as:Rule"/></xsl:message -->
        <xsl:if test="not(as:Rule//as:constraint[@action='excludeAttribute()'])">
            <xsl:variable name="QName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
            <xsl:variable name="use" select="if(as:Rule/as:constraint[@action='makeOptional()']) then 'optional' else 'required'"/>
            <xsd:attribute ref="{@name}" use="{$use}">
                <xsl:if test="count(camed:annotation) &gt; 0">
                    <xsl:call-template name="outputAnnotation">
                        <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                    </xsl:call-template>
                </xsl:if>
            </xsd:attribute>
        </xsl:if>
    </xsl:template>

    <xsl:template match="as:Attribute">
        <!-- local definition -->
        <xsl:if test="not(as:Rule//as:constraint[@action='excludeAttribute()'])">
            <xsl:variable name="QName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
            <xsl:variable name="type">
                <xsl:call-template name="findType">
                    <xsl:with-param name="rules" select="as:Rule"/>
                </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="use" select="if(as:Rule/as:constraint[@action='makeOptional()']) then 'optional' else 'required'"/>
            <xsl:choose>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues(''true''|''false'')')]">
                        <xsl:call-template name="outputAttribute">
                            <xsl:with-param name="type" select="$type"/>
                            <xsl:with-param name="QName" select="$QName"/>
                            <xsl:with-param name="use" select="$use"/>
                            <xsl:with-param name="fixed" select="normalize-space(@exampleValue)"/>
                        </xsl:call-template>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')] or(as:Rule/as:constraint[contains(@action,'lookup(')])">
                    <xsl:variable name="valuesRule" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
                    <xsl:variable name="lookupRule" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/>
                    
                    <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
                    
                    <xsl:variable name="values" select="w2e:annoValues(.,$valuesRule,$lookupRule)" />
                    <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,$lookupRule)"/>
                    
                    <xsl:variable name="baseType">
                        <xsl:call-template name="findType">
                            <xsl:with-param name="rules" select="as:Rule"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="listBaseType">
                        <xsl:choose>
                            <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                            <xsl:otherwise>xsd:string</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsd:attribute name="{$QName}" id="{generate-id()}" use="{$use}">
                        <xsl:if test="count(camed:annotation) &gt; 0">
                            <xsl:call-template name="outputAnnotation">
                                <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                            </xsl:call-template>
                        </xsl:if>
                        <xsd:simpleType>
                            <xsd:restriction base="{$listBaseType}">
                                <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                    <xsl:call-template name="setLength">
                                        <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                    </xsl:call-template>
                                </xsl:if>
                                <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                            </xsd:restriction>
                        </xsd:simpleType>
                    </xsd:attribute>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="outputAttribute">
                        <xsl:with-param name="type" select="$type"/>
                        <xsl:with-param name="QName" select="$QName"/>
                        <xsl:with-param name="use" select="$use"/>
                        <xsl:with-param name="fixed" select="normalize-space(@exampleValue)"/>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template match="as:Attribute" mode="type">
        <!-- type reference with namespace -->
        <xsl:if test="not(as:Rule/as:constraint[@action='excludeAttribute()'])">
            <xsl:variable name="QName"
                select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
            <xsl:variable name="type">
                <xsl:call-template name="findType">
                    <xsl:with-param name="rules" select="as:Rule"/>
                </xsl:call-template>
            </xsl:variable>
            <xsl:choose>
                <xsl:when
                    test="as:Rule/as:constraint[contains(@action,'restrictValues(''true''|''false'')')]">
                    <xsd:attribute name="{$QName}" type="{$type}" id="{generate-id()}">
                        <xsl:if test="count(camed:annotation) &gt; 0">
                            <xsl:call-template name="outputAnnotation">
                                <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                            </xsl:call-template>
                        </xsl:if>
                    </xsd:attribute> 
                </xsl:when>
                <xsl:when
                    test="as:Rule/as:constraint[contains(@action,'restrictValues(''true''| ''false'')')]">
                    <xsd:attribute name="{$QName}" type="{$type}" id="{generate-id()}">
                        <xsl:if test="count(camed:annotation) &gt; 0">
                            <xsl:call-template name="outputAnnotation">
                                <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                            </xsl:call-template>
                        </xsl:if>
                    </xsd:attribute> 
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'restrictValues')]">
                    <xsl:variable name="this" select="@name"/>
                    <xsl:variable name="parent" select="parent::as:Element/@name"/>
                    <xsl:variable name="occurs"
                        select="$source//as:Element[@name=$parent]/as:Attribute[@name=$this]/as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
                    <xsl:variable name="restrictions" select="distinct-values($occurs)"/>
                    <xsl:variable name="temp0">
                        <xsl:for-each select="$restrictions">
                            <xsl:variable name="temp" select="replace(.,'\)',' ')"/>
                            <xsl:variable name="temp1" select="tokenize(substring-after($temp,'restrictValues('),'\|')"/>
                            <xsl:for-each select="$temp1"><xsl:value-of select="."/>|</xsl:for-each>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="temp2" select="tokenize($temp0,'\|')"/>
                    <xsl:variable name="values" select="distinct-values($temp2)"/>
                    <!-- xsl:message><xsl:value-of select="$parent"/>/<xsl:value-of select="$this"/>[<xsl:value-of select="$values"/>]</xsl:message -->
                    <xsl:variable name="valuesRule" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/@action"/>
                    
                    <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
                    
                    <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,'')"/>
                    
                    
                    <xsl:variable name="baseType">
                        <xsl:call-template name="findType">
                            <xsl:with-param name="rules" select="as:Rule"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="listBaseType">
                        <xsl:choose>
                            <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                            <xsl:otherwise>xsd:string</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsd:attribute name="{$QName}" id="{generate-id()}">
                        <xsl:if test="count(camed:annotation) &gt; 0">
                            <xsl:call-template name="outputAnnotation">
                                <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                            </xsl:call-template>
                        </xsl:if>
                        <xsd:simpleType>
                            <xsd:restriction base="{$listBaseType}">
                                <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                    <xsl:call-template name="setLength">
                                        <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                    </xsl:call-template>
                                </xsl:if>
                                <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                            </xsd:restriction>
                        </xsd:simpleType>
                    </xsd:attribute>
                </xsl:when>
                <xsl:when test="as:Rule/as:constraint[contains(@action,'lookup(')]">
                    <xsl:variable name="lookupRule" select="as:Rule/as:constraint[contains(@action,'lookup(')]/@action"/>
                    <xsl:variable name="values">
                                <xsl:variable name="lookupName" select="substring-before(substring-after($lookupRule[1],'lookup('),')')"/>
                                <xsl:variable name="valuesString"><xsl:for-each select="$lookupTables[@name = $lookupName]/look:item/@name">'<xsl:value-of select="."/>'|</xsl:for-each></xsl:variable>
                                <xsl:value-of select="$valuesString"/>
                    </xsl:variable>
                    
                    <xsl:variable name="valuesRuleAnno" select="as:Rule/as:constraint[contains(@action,'restrictValues')]/as:annotation/as:documentation[@type = 'Definition' or(@type = 'documentation')]"/>
                    
                    <xsl:variable name="valuesAnno" select="w2e:annoAnno(.,$valuesRuleAnno,$lookupRule)"/>
                    
                    <xsl:variable name="this" select="@name"/>
                    <xsl:variable name="parent" select="parent::as:Element/@name"/>
                    <!-- xsl:message><xsl:value-of select="$parent"/>/<xsl:value-of select="$this"/>[<xsl:value-of select="$values"/>]</xsl:message -->
                    <xsl:variable name="baseType">
                        <xsl:call-template name="findType">
                            <xsl:with-param name="rules" select="as:Rule"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="listBaseType">
                        <xsl:choose>
                            <xsl:when test="string-length($baseType) &gt; 0"><xsl:value-of select="if(starts-with($baseType,'xsd:')) then $baseType else concat('xsd:',$baseType)"/></xsl:when>
                            <xsl:otherwise>xsd:string</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsd:attribute name="{$QName}" id="{generate-id()}">
                        <xsl:if test="count(camed:annotation) &gt; 0">
                            <xsl:call-template name="outputAnnotation">
                                <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                            </xsl:call-template>
                        </xsl:if>
                        <xsd:simpleType>
                            <xsd:restriction base="{$listBaseType}">
                                <xsl:if test="as:Rule/as:constraint[contains(@action,'setLength')]">
                                    <xsl:call-template name="setLength">
                                        <xsl:with-param name="lengthRule" select="as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                    </xsl:call-template>
                                </xsl:if>
                                <xsl:if test="not($listBaseType='xsd:boolean')"><xsl:copy-of select="w2e:enumerations($values,$valuesAnno)"/></xsl:if>
                            </xsd:restriction>
                        </xsd:simpleType>
                    </xsd:attribute>
                    
                </xsl:when>    
                    
                <xsl:otherwise>
                    <xsl:call-template name="outputAttribute">
                        <xsl:with-param name="type" select="$type"/>
                        <xsl:with-param name="QName" select="$QName"/>
                        <xsl:with-param name="use"/>
                        <xsl:with-param name="fixed" select="normalize-space(@exampleValue)"/>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template name="findType">
        <xsl:param name="rules"/>
        
        <xsl:variable name="ruleActions" select="$rules/as:constraint/@action"/>

        <xsl:variable name="typeResult">
            <xsl:for-each select="$ruleActions">
                <xsl:variable name="chkAction">
                    <xsl:choose>
                        <xsl:when test="contains(.,'(/')">
                            <xsl:value-of select="concat(substring-before(.,'('),'(',substring-after(.,','))"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="."/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <!-- xsl:message>rule [<xsl:value-of select="$chkAction"></xsl:value-of>]</xsl:message -->
                <xsl:choose>
                    <xsl:when
                        test="contains($chkAction,'restrictValues(') or(contains($chkAction,'lookup(')) or(contains($chkAction,'Mask(')) or(contains($chkAction,'datatype('))">
                        <xsl:choose>
                            <xsl:when test="$chkAction='datatype(hexBinary)'">xsd:hexBinary|</xsl:when>
                            <xsl:when test="$chkAction='datatype(anyURI)'">xsd:anyURI|</xsl:when>
                            <xsl:when test="$chkAction='datatype(normalizedString)'">xsd:normalizedString|</xsl:when>
                            <xsl:when test="$chkAction='datatype(ID)'">xsd:ID|</xsl:when>
                            <xsl:when test="$chkAction='datatype(IDREF)'">xsd:IDREF|</xsl:when>
                            <xsl:when test="$chkAction='datatype(IDREFS)'">xsd:IDREFS|</xsl:when>
                            <xsl:when test="$chkAction='datatype(double)'">xsd:double|</xsl:when>
                            <xsl:when test="$chkAction='datatype(float)'">xsd:float|</xsl:when>
                            <xsl:when test="$chkAction='datatype(byte)'">xsd:byte|</xsl:when>
                            <xsl:when test="$chkAction='datatype(duration)'">xsd:duration|</xsl:when>
                            <xsl:when test="$chkAction='datatype(base64Binary)'">xsd:base64Binary|</xsl:when>
                            <xsl:when test="$chkAction='datatype(long)'">xsd:long|</xsl:when>
                            <xsl:when test="$chkAction='datatype(short)'">xsd:short|</xsl:when>
                            <xsl:when test="$chkAction='datatype(unsignedLong)'">xsd:unsignedLong|</xsl:when>
                            <xsl:when test="$chkAction='datatype(unsignedShort)'">xsd:unsignedShort|</xsl:when>
                            <xsl:when test="$chkAction='datatype(QName)'">xsd:QName|</xsl:when>
                            <xsl:when test="$chkAction='datatype(Name)'">xsd:Name|</xsl:when>
                            <xsl:when test="$chkAction='datatype(NOTATION)'">xsd:NOTATION|</xsl:when>
                            <xsl:when test="$chkAction='datatype(NCName)'">xsd:NCName|</xsl:when>
                            <xsl:when test="$chkAction=$anyType">xsd:any|</xsl:when>
                            <xsl:when test="$chkAction=$dateTimeMask">xsd:dateTime|</xsl:when>
                            <xsl:when test="$chkAction=$boolean">xsd:boolean|</xsl:when>
                            <xsl:when test="$chkAction='datatype(boolean)'">xsd:boolean|</xsl:when>
                            <xsl:when test="$chkAction=$date">xsd:date|</xsl:when>
                            <xsl:when test="$chkAction=$date1">xsd:date|</xsl:when>
                            <xsl:when test="$chkAction=$year">xsd:gYear|</xsl:when>
                            <xsl:when test="$chkAction=$yearM">xsd:gYearMonth|</xsl:when>
                            <xsl:when test="$chkAction=$month">xsd:gMonth|</xsl:when>
                            <xsl:when test="$chkAction=$monthD">xsd:gMonthDay|</xsl:when>
                            <xsl:when test="$chkAction=$day">xsd:gDay|</xsl:when>
                            <xsl:when test="$chkAction=$time">xsd:time|</xsl:when>
                            <xsl:when test="$chkAction=$idRef">xsd:IDREF|</xsl:when>
                            <xsl:when test="$chkAction=$idRefs">xsd:IDREFS|</xsl:when>
                            <xsl:when test="$chkAction=$id">xsd:ID|</xsl:when>
                            <xsl:when test="$chkAction=$duration">xsd:duration|</xsl:when>
                            <xsl:when test="$chkAction=$token">xsd:token|</xsl:when>
                            <xsl:when test="$chkAction=$decimal">xsd:decimal|</xsl:when>
                            <xsl:when test="$chkAction=$integer">xsd:integer|</xsl:when>
                            <xsl:when test="$rules/as:constraint[contains(@action,'setNumberMask')] and(contains($chkAction,'#') or(contains($chkAction,'0')))">
                                <xsl:choose>
                                    <xsl:when test="contains($chkAction,'.')">xsd:float|</xsl:when>
                                    <xsl:when test="contains($chkAction,'.##)')">xsd:decimal|</xsl:when>
                                    <xsl:when test="contains($chkAction,'.0)')">xsd:decimal|</xsl:when>
                                    <xsl:otherwise>xsd:int|</xsl:otherwise>
                                </xsl:choose>
                                </xsl:when>                           
                            <xsl:otherwise>xsd:string|</xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise/>
                </xsl:choose>
            </xsl:for-each>
        </xsl:variable>
        <!-- xsl:message>typeResult [<xsl:value-of select="$typeResult"></xsl:value-of>]</xsl:message -->
        <xsl:choose>
            <xsl:when test="string-length($typeResult)=0">xsd:string</xsl:when>
            <xsl:when test="$typeResult='xsd:string|'">xsd:string</xsl:when>
            <xsl:otherwise>
                <xsl:variable name="result" select="distinct-values(tokenize($typeResult,'\|'))"/>
                <xsl:variable name="baseResult">
                    <xsl:for-each select="$result">
                        <xsl:if test="not(.='xsd:string')">
                            <xsl:value-of select="."/>
                        </xsl:if>
                    </xsl:for-each>
                </xsl:variable>
                <xsl:choose>
                    <xsl:when test="string-length($baseResult) &gt; 0"><xsl:value-of select="$baseResult"/></xsl:when>
                    <xsl:otherwise>xsd:string</xsl:otherwise>
                </xsl:choose>
                <xsl:variable name="digits">
                    <xsl:variable name="maskRule" select="$rules/as:constraint[contains(@action,'setNumberMask')]/@action"></xsl:variable>
                    <xsl:choose>
                        <xsl:when test="$baseResult='xsd:decimal'">
                            <xsl:variable name="main" select="replace(substring-after(substring-before($maskRule,'.'),'('),'0','#')"/>
                            <xsl:variable name="number" select="replace($main,'#','')"/>
                            <xsl:variable name="digit1" select="if($number='') then string-length($main)+2 else $number+2"/>
                            <xsl:value-of select="concat($digit1,';2')"/>
                        </xsl:when>
                        <xsl:when test="starts-with($baseResult,'xsd:int')">
                            <xsl:variable name="number" select="replace(substring-before(substring-after($maskRule,'('),')'),'0','#')"/>
                            <xsl:variable name="digit1" select="replace($number,'#','')"/>
                            <xsl:value-of select="if($digit1='') then string-length($number) else $digit1"/>;0</xsl:when>
                        <xsl:when test="$baseResult='xsd:float'">
                            <xsl:variable name="fraction" select="replace(substring-before(substring-after($maskRule,'.'),')'),'0','#')"/>
                            <xsl:variable name="factor" select="replace($fraction,'#','')"/>
                            <xsl:variable name="main" select="replace(substring-after(substring-before($maskRule,'.'),'('),'0','#')"/>
                            <xsl:variable name="number" select="replace($main,'#','')"/>
                            <xsl:variable name="digit1" select="if($number='') then string-length($main) else $number"/>
                            <xsl:variable name="digit2" select="if($factor='') then string-length($fraction) else $factor"/>
                            <xsl:variable name="allDigits" select="xsd:integer($digit1)+xsd:integer($digit2)"/>
                            <xsl:value-of select="concat($allDigits,';',$digit2)"/>
                        </xsl:when>
                        <xsl:otherwise/>
                    </xsl:choose>
                </xsl:variable>
                <xsl:if test="not($digits='') and(string-length($baseResult) &gt; 0)">|<xsl:value-of select="$digits"></xsl:value-of></xsl:if>
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>

    <xsl:template name="chkChoiceItem">
        <xsl:param name="name"/>
        <xsl:param name="xsdChoice"/>

        <xsl:variable name="matchChoice">
            <xsl:for-each select="$xsdChoice/xsd:choice">
                <xsl:variable name="xsdChoiceItem" select="."/>
                <xsl:variable name="firstChoice" select="xsd:element[1]/@name"/>
                <xsl:if test="$firstChoice=$name">
                    <xsl:copy-of select="$xsdChoiceItem"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>

        <xsl:choose>
            <xsl:when test="count($matchChoice/xsd:choice) &gt; 0">
                <xsl:copy-of select="$matchChoice"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="chkMatch">
                    <xsl:for-each select="$xsdChoice/xsd:choice">
                        <xsl:for-each select="xsd:element">
                            <xsl:if test="$name=@name">skip</xsl:if>
                        </xsl:for-each>
                    </xsl:for-each>
                </xsl:variable>
                <xsl:choose>
                    <xsl:when test="contains($chkMatch,'skip')">skip</xsl:when>
                    <xsl:otherwise>none</xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>

    </xsl:template>
    
    <xsl:template name="whatgoeswhere">
        <xsl:param name="thisNS"/>
        <xsl:variable name="nsCount" select="count(//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))])"/>
        <xsl:choose>
            <xsl:when test="$nsCount &lt; 2"/>
            <xsl:otherwise>
        <!-- check for each namespace -->
        <xsl:for-each
            select="//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))]">
            <xsl:message> ---- checking <xsl:value-of select="@prefix"/> of <xsl:value-of select="count(//as:Namespaces/as:namespace[not(contains($ignoreNS,concat('|',@prefix,'|')))])"/></xsl:message>
            <xsl:variable name="ns" select="@prefix"/>
            <xsl:variable name="nsURI" select="if(starts-with(text(),'&#xA;')) then substring-after(text(),'&#xA;') else text()"/>
            
            <xsl:variable name="defsElements">
                <xsl:for-each select="$source//as:Element[starts-with(@name,concat($ns,':'))]">
                    <xsl:copy-of select="."/>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="declElem" select="distinct-values($defsElements/as:Element/@name)"/>
          
            <xsl:variable name="what-to-move">
                <moves>
                <xsl:for-each select="$declElem">
                    <xsl:variable name="this" select="."/>
                    <!-- xsl:message>Element [<xsl:value-of select="."/>] </xsl:message -->
                    
                    <xsl:variable name="getElem" select="$defsElements/as:Element[@name=$this][1]"/>
                    <xsl:variable name="lookDown" select="$getElem/as:Element/substring-before(@name,':')"/>
                    
                    <xsl:variable name="countDown" select="count($lookDown)" as="xsd:integer"/>
                    <xsl:variable name="chkDown" select="distinct-values($lookDown)" />
                    
                    <xsl:variable name="downChanged">
                        <xsl:if test="$countDown &gt; 0">
                            <xsl:for-each select="$chkDown">
                                <xsl:choose>
                                    <xsl:when test=".=''"/>
                                    <xsl:when test="not(. = $ns)">
                                        <changed><xsl:value-of select="."/></changed>
                                    </xsl:when>
                                    <xsl:when test=". = $ns">
                                        <notchanged><xsl:value-of select="$ns"/></notchanged>
                                    </xsl:when>
                                </xsl:choose>
                            </xsl:for-each>
                        </xsl:if>
                    </xsl:variable>
                    
                    <xsl:variable name="getElemName" select="$getElem[1]/@name"/>
                    <xsl:variable name="getElemPrefix" select="substring-before($getElemName,':')"/>
                    
                    <xsl:variable name="chkElemUp" select="$source//as:Element[@name=$this]/../../as:Element"/>
                    <xsl:variable name="chkElemName" select="$chkElemUp[1]/@name"/>
                    <xsl:variable name="chkElemUpPrefix" select="substring-before($chkElemName,':')"/>
                    
                    <!-- xsl:message>chkElem[<xsl:value-of select="$chkElemName"></xsl:value-of>]</xsl:message -->
                    
                    <xsl:choose>
                        <xsl:when test="count($downChanged//notchanged) &gt; 0 and($chkElemUpPrefix='' or($chkElemUpPrefix=$ns))">
                            <!-- xsl:comment> prefix matched 
                            </xsl:comment>
                            <skip><xsl:value-of select="$getElemName"/></skip-->
                        </xsl:when>
                        <xsl:when test="count($downChanged//changed) &gt; 0 and($countDown &gt; 0)">
                            <!-- xsl:comment> down changed 
                            </xsl:comment-->
                            <xsl:choose>
                                <xsl:when test="$chkElemUpPrefix=''">
                                    <!-- skip><xsl:value-of select="$getElemName"/></skip -->
                                </xsl:when>
                                <xsl:otherwise>
                                    <move ns="{$lookDown[1]}" name="{$getElemName}">
                                        <xsl:copy-of select="$getElem[1]"/>
                                    </move>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:when test="$chkDown = $ns and($chkElemUpPrefix = '')">
                            <!-- xsl:comment> down same 
                            </xsl:comment>
                            <skip><xsl:value-of select="$getElemName"/></skip -->
                        </xsl:when>
                        <xsl:when test="$chkElemUpPrefix = ''">
                            <!-- xsl:comment> up blank prefix 
                            </xsl:comment>
                            <skip><xsl:value-of select="$getElemName"/></skip -->
                        </xsl:when>
                        <xsl:when test="$chkElemUpPrefix = $getElemPrefix">
                            <!-- xsl:comment> up same prefix 
                            </xsl:comment>
                            <skip><xsl:value-of select="$getElemName"/></skip -->
                        </xsl:when>
                        <xsl:when test="$lookDown = '' or(count($lookDown) = 0)">
                            <!-- xsl:comment> look down blank
                            </xsl:comment>
                            <skip><xsl:value-of select="$getElemName"/></skip -->
                        </xsl:when>
                        <xsl:otherwise>
                            <!-- xsl:comment> declaration 
                            </xsl:comment>
                            <skip><xsl:value-of select="$getElemName"/></skip -->
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
                </moves>
            </xsl:variable>
            
           <xsl:copy-of  select="$what-to-move"></xsl:copy-of>
            
        </xsl:for-each>
                
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:template>
    
    <xsl:template name="setLength">
        <xsl:param name="lengthRule"/>
        <xsl:variable name="minLength" select="substring-after(substring-before($lengthRule[1],'-'),'(')"/>
        <xsl:variable name="maxLength">
            <xsl:choose>
                <xsl:when test="contains($lengthRule[1],'-')"><xsl:value-of select="substring-before(substring-after($lengthRule[1],'-'),')')"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="substring-before(substring-after($lengthRule[1],'('),')')"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="not($minLength='')"><xsd:minLength value="{$minLength}"/></xsl:if>
        <xsl:if test="not($maxLength='')"><xsd:maxLength value="{$maxLength}"/></xsl:if>
    </xsl:template>
    
    <xsl:template name="setRange">
        <xsl:param name="rangeRule"/>
        <xsl:variable name="minValue">
            <xsl:choose><!-- See if negative number or regular minimum -->
                <xsl:when test="contains($rangeRule[1],'(-')"><xsl:value-of select="concat('-',substring-before(substring-after($rangeRule[1],'-'),'-'))"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="substring-after(substring-before($rangeRule[1],'-'),'(')"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="maxValue">
            <xsl:choose>
                <xsl:when test="contains($rangeRule[1],'(-')"><xsl:value-of select="substring-before(substring-after(substring-after($rangeRule[1],'-'),'-'),')')"/></xsl:when>
                <xsl:when test="contains($rangeRule[1],'-')"><xsl:value-of select="substring-before(substring-after($rangeRule[1],'-'),')')"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="substring-before(substring-after($rangeRule[1],'('),')')"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="not($minValue='')"><xsd:minInclusive value="{$minValue}"/></xsl:if>
        <xsl:if test="not($maxValue='')"><xsd:maxInclusive value="{$maxValue}"/></xsl:if>
    </xsl:template>
    
    <xsl:template name="setPattern">
        <xsl:param name="pattern"/>
        <xsl:variable name="mask">
            <xsl:variable name="mask1"><xsl:value-of select="substring-after($pattern,'(')"/></xsl:variable>
            <xsl:variable name="maskLength" select="string-length($mask1)"/>
            <xsl:value-of select="substring($mask1,1,$maskLength - 1)"/>            
        </xsl:variable>
        <!--  xsl:message>[<xsl:value-of select="$mask"></xsl:value-of>]</xsl:message -->
        
        <!-- if Java regexp mask - just output it - otherwise create regexp -->
        
        <xsl:variable name="maskChrs" select="w2e:unpackString($mask)"/> 
        <!-- xsl:message>mask [<xsl:value-of select="$maskChrs"/>]</xsl:message -->
        <xsl:variable name="maskStr"><xsl:for-each select="$maskChrs"><xsl:value-of select="."/></xsl:for-each></xsl:variable>
        <xsl:element name="xsd:pattern">
            <xsl:attribute name="value">
                <xsl:variable name="rawRegExp"><xsl:for-each select="tokenize($maskStr,'\{\}')"> <xsl:variable name="this1" select="position()"/><xsl:value-of select="w2e:mask2Pattern(.)"/></xsl:for-each></xsl:variable>
                <xsl:value-of select="replace($rawRegExp,'\*\*','')"/></xsl:attribute>
        </xsl:element>
    </xsl:template>
    
    <xsl:template name="outputAttribute">
        <xsl:param name="type"/>
        <xsl:param name="QName"/>
        <xsl:param name="use"/>
        <xsl:param name="fixed"/>
        <xsl:choose>
            <xsl:when test="$use=''">
                <xsl:choose>
                    <xsl:when test="contains($type,'|')">
                        <xsd:attribute name="{$QName}" id="{generate-id()}">
                            <xsl:if test="count(camed:annotation) &gt; 0">
                                <xsl:call-template name="outputAnnotation">
                                    <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                                </xsl:call-template>
                            </xsl:if>
                            <xsd:simpleType>
                                <xsd:restriction base="{substring-before($type,'|')}">
                                    <xsd:totalDigits value="{substring-before(substring-after($type,'|'),';')}" />
                                    <xsd:fractionDigits value="{substring-after($type,';')}" />
                                </xsd:restriction> 
                            </xsd:simpleType>
                        </xsd:attribute>
                    </xsl:when>
                    <xsl:when test="string-length($fixed) &gt; 0 and not(starts-with($fixed,'%') and ends-with($fixed,'%'))"><xsd:attribute name="{$QName}" type="{$type}" fixed="{$fixed}" id="{generate-id()}"/></xsl:when>
                    <xsl:otherwise><xsd:attribute name="{$QName}" type="{$type}" id="{generate-id()}"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:choose>
                    <xsl:when test="contains($type,'|')">
                        <xsd:attribute name="{$QName}" id="{generate-id()}" use="{$use}">
                            <xsl:if test="count(camed:annotation) &gt; 0">
                                <xsl:call-template name="outputAnnotation">
                                    <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                                </xsl:call-template>
                            </xsl:if>
                            <xsd:simpleType>
                                <xsd:restriction base="{substring-before($type,'|')}">
                                    <xsd:totalDigits value="{substring-before(substring-after($type,'|'),';')}" />
                                    <xsd:fractionDigits value="{substring-after($type,';')}" />
                                </xsd:restriction> 
                            </xsd:simpleType>
                        </xsd:attribute>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsd:attribute name="{$QName}" type="{$type}" id="{generate-id()}" use="{$use}">
                            <xsl:if test="count(camed:annotation) &gt; 0">
                                <xsl:call-template name="outputAnnotation">
                                    <xsl:with-param name="annoDetails" select="camed:annotation/camed:documentation"/>
                                </xsl:call-template>
                            </xsl:if>
                        </xsd:attribute>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="outputRootElement">
        <xsl:param name="elementName"/>
        <xsl:param name="changeNS"/>
        <xsl:param name="rules"/>
        <xsl:variable name="thisName" select="if(contains($elementName,':')) then substring-after($elementName,':') else $elementName"/>
        <xsl:message> --- <xsl:value-of select="$elementName"/></xsl:message>
        
        <!-- xsl:choose>
            <xsl:when test="count($duplicateElements/dup[@name=$thisName]) &gt; 0"><xsl:comment>root element - duplicate element definition <xsl:value-of select="$thisName"/></xsl:comment>
                <xsd:element name="{if(contains($elementName,':')) then $elementName else concat(substring-before($changeNS,'|'),':',$elementName)}" type="{$elementName}"/>
            </xsl:when>
            <xsl:when test="count($nsContentModels/pack//xsd:element[@name=$thisName][@type='']) &gt; 0"><xsl:comment>root element - duplicate packed definition <xsl:value-of select="$thisName"/></xsl:comment>
                <xsd:element name="{if(contains($elementName,':')) then $elementName else concat(substring-before($changeNS,'|'),':',$elementName)}" type="{$elementName}"/>
            </xsl:when>
            <xsl:otherwise -->
        <xsl:variable name="repeat" select="if ($rules//as:constraint[starts-with(@action,'setLimit')]) then $rules//as:constraint[starts-with(@action,'setLimit')]/substring-before(substring-after(@action,'('),')') else 'unbounded'"></xsl:variable>
         <xsl:element name="xsd:element">
             <xsl:attribute name="ref" select="if(contains($elementName,':')) then $elementName else concat(substring-before($changeNS,'|'),':',$elementName)"/>
             <xsl:if test="$rules//as:constraint[@action='makeOptional()']">
                 <xsl:attribute name="minOccurs">0</xsl:attribute>
             </xsl:if>
             <xsl:if test="$rules//as:constraint[@action='makeRepeatable()']">
                 <xsl:attribute name="maxOccurs"><xsl:value-of select="w2e:maxOccurs($repeat)"/></xsl:attribute>
             </xsl:if>
         </xsl:element>
       <!-- /xsl:otherwise> 
        </xsl:choose -->
    </xsl:template>
    
    <xsl:template name="outputAnnotation">
        <xsl:param name="annoDetails"/>
        <!-- xsl:message>[<xsl:copy-of select="$annoDetails"/>]</xsl:message -->
        <xsd:annotation>
            <xsl:variable name="URNpath" select="$annoDetails/.[@type = 'URN']"/>
            <xsl:variable name="URN" select="normalize-space($URNpath[1])"/>
            <xsl:for-each select="$annoDetails">
                <xsl:choose>
                    <xsl:when test="@type = 'Type'">
                        <xsd:documentation>Original type - <xsl:value-of select="normalize-space(.)"/></xsd:documentation>
                    </xsl:when>
                    <xsl:when test="@type = 'Definition'">
                        <xsd:documentation><xsl:value-of select="normalize-space(.)"/></xsd:documentation>
                    </xsl:when>
                    <xsl:when test="@type = 'URN'"/>
                    <xsl:when test="string-length($URN) &gt; 0">
                       <xsd:appinfo source="{concat($URN,':',@type)}"><xsl:value-of select="normalize-space(.)"/></xsd:appinfo>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsd:documentation><xsl:value-of select="normalize-space(.)"/></xsd:documentation>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsd:annotation>
    </xsl:template>

    <!-- deprecated method for distinct list generation -->
    <!-- xsl:variable name="declElem" >
        <xsl:variable name="tempNames">
        <xsl:for-each select="$defsElements/as:Element"><name><xsl:value-of select="./@name"/></name></xsl:for-each>
        </xsl:variable>
        <xsl:variable name="tempList">
        <xsl:for-each select="$tempNames/name">
        <xsl:variable name="this1" select="."/>
        <xsl:variable name="pos1" select="position()"/>
        <xsl:choose>
        <xsl:when test="count($tempNames/name[text()=$this1]) &gt; 1">
        <xsl:variable name="chk1" select="$tempNames/name[text()=$this1]/position()"/>
        <! pick the last duplicate item >
        <xsl:if test="subsequence($chk1,count($chk1),1)=$pos1"><xsl:value-of select="."/>|</xsl:if></xsl:when>
        <xsl:otherwise><xsl:value-of select="."/>|</xsl:otherwise>
        </xsl:choose>
        </xsl:for-each>
        </xsl:variable>
        <xsl:value-of select="$tempList"></xsl:value-of>
        </xsl:variable -->
    <!-- xsl:message>elem <xsl:value-of select="$ns"/>-<xsl:value-of select="count(tokenize($declElem,'\|'))"/>-[<xsl:value-of select="$declElem"/>]</xsl:message -->

    <xsl:function name="w2e:unpackString">
        <xsl:param name="string"/>
        <!-- xsl:message><xsl:value-of select="$string"/></xsl:message -->
        <xsl:if test="string-length($string) &gt; 0">
            <xsl:variable name="this" select="substring($string,1,1)"/>
            <xsl:variable name="next" select="substring-after($string,$this)"/>
            <xsl:variable name="chkValue1" select="if (string-length($next) &gt; 0) then substring($next,1,1) else 'X'"/>
            <xsl:variable name="chkValue2" select="if (string-length($next) &gt; 1) then substring($next,2,1) else 'X'"/>
            <!-- xsl:message><xsl:value-of select="$this"/></xsl:message -->
            <xsl:choose>
                <xsl:when test="$this='|'">|<xsl:value-of select="w2e:unpackString($next)"/></xsl:when>
                <xsl:when test="$this='_'">_<xsl:value-of select="w2e:unpackString($next)"/></xsl:when>
                <xsl:when test="$this=''''"><xsl:value-of select="concat($this,substring-before($next,''''),$this,'{}')"/><xsl:value-of select="w2e:unpackString(substring-after($next,''''))"/></xsl:when>
                <xsl:when test="contains($isNumeric,$chkValue1)">
                    <xsl:choose>                       
                        <xsl:when test="contains($isNumeric,$chkValue2)"><xsl:value-of select="concat($this,'**',$chkValue1,$chkValue2,'{}')"/><xsl:value-of select="w2e:unpackString(substring-after($next,concat($chkValue1,$chkValue2)))"/></xsl:when>
                        <xsl:otherwise><xsl:value-of select="concat($this,'**',$chkValue1,'{}')"/><xsl:value-of select="w2e:unpackString(substring-after($next,$chkValue1))"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise><xsl:value-of select="concat($this,'{}')"/><xsl:value-of select="w2e:unpackString($next)"/></xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:function>

    <xsl:function name="w2e:mask2Pattern">
        <xsl:param name="mask"/>
        <xsl:variable name="count" select="if(contains($mask,'**')) then xsd:integer(substring-after($mask,'**')) else 1"></xsl:variable>
        <xsl:variable name="maskChr" select="if(contains($mask,'**')) then substring-before($mask,'**') else $mask"></xsl:variable> 
        <xsl:choose>
            <xsl:when test="$maskChr='|_'">)?</xsl:when>
            <xsl:when test="$maskChr='|'">)</xsl:when>
            <xsl:when test="$maskChr='0'">[0-9]<xsl:if test="$count &gt; 1">{<xsl:value-of select="$count"/>}</xsl:if></xsl:when>
            <xsl:when test="$maskChr='X'">[a-z,A-Z,0-9]<xsl:if test="$count &gt; 1">{<xsl:value-of select="$count"/>}</xsl:if></xsl:when>
            <xsl:when test="$maskChr='U'">[A-Z]<xsl:if test="$count &gt; 1">{<xsl:value-of select="$count"/>}</xsl:if></xsl:when>
            <xsl:when test="$maskChr='L'">[a-z]<xsl:if test="$count &gt; 1">{<xsl:value-of select="$count"/>}</xsl:if></xsl:when>
            <xsl:when test="$maskChr='A'">[A-Z,0-9]<xsl:if test="$count &gt; 1">{<xsl:value-of select="$count"/>}</xsl:if></xsl:when>
                <!-- optional mask characters -->
            <xsl:when test="$maskChr='_'">[a-z]{0,<xsl:value-of select="$count"/>}</xsl:when>
            <xsl:when test="$maskChr='a'">[a-z,0-9]{0,<xsl:value-of select="$count"/>}</xsl:when>
            <xsl:when test="$maskChr='?'">[a-z,A-Z,0-9]{0,<xsl:value-of select="$count"/>}</xsl:when>
            <xsl:when test="$maskChr='^'">[A-Z]{0,<xsl:value-of select="$count"/>}</xsl:when>
            <xsl:when test="$maskChr='#'">[0-9]{0,<xsl:value-of select="$count"/>}</xsl:when>
            <xsl:when test="$maskChr=''"/>
            <xsl:when test="substring($maskChr,1,1)=''''">[<xsl:value-of select="substring-before(substring-after($maskChr,''''),'''')"/>]</xsl:when>
            <xsl:otherwise><xsl:value-of select="$mask"/></xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    <xsl:function name="w2e:packageName">
        <xsl:param name="seedURN"/>
        <xsl:variable name="domainName" select="substring-before($seedURN,'/')"/>
        <xsl:variable name="domainCount" select="count(tokenize($domainName,'\.'))"/>
        <!-- xsl:message> - domain - <xsl:value-of select="$seedURN"/> - <xsl:value-of select="$domainName"/> - <xsl:value-of select="$domainCount"/></xsl:message -->
        <xsl:variable name="seedURNstring" select="replace($seedURN,'\.','/')"/>
        <xsl:variable name="seedName">
            <xsl:choose>
                <xsl:when test="starts-with($seedURNstring,'www/')"><xsl:value-of select="substring-after($seedURNstring,'www/')"/></xsl:when>
                <xsl:when test="starts-with($seedURNstring,'www1/')"><xsl:value-of select="substring-after($seedURNstring,'www1/')"/></xsl:when>
                <xsl:when test="starts-with($seedURNstring,'www2/')"><xsl:value-of select="substring-after($seedURNstring,'www2/')"/></xsl:when>
                <xsl:when test="starts-with($seedURNstring,'www3/')"><xsl:value-of select="substring-after($seedURNstring,'www3/')"/></xsl:when>
                <xsl:when test="starts-with($seedURNstring,'www4/')"><xsl:value-of select="substring-after($seedURNstring,'www4/')"/></xsl:when>
                <xsl:when test="starts-with($seedURNstring,'www5/')"><xsl:value-of select="substring-after($seedURNstring,'www5/')"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="$seedURNstring"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!-- xsl:message> - seed <xsl:value-of select="$seedURN"/> - <xsl:value-of select="$seedURNstring"/></xsl:message -->
        <xsl:variable name="partsCount" select="count(tokenize($seedName,'/'))"/>
        <xsl:variable name="nameParts" select="tokenize($seedName,'/')"/>
        <xsl:variable name="returnResult">
        <xsl:choose>
            <xsl:when test="$partsCount &gt; 2">
                <xsl:for-each select="tokenize($seedName,'/')">
                    <xsl:variable name="this">
                    <xsl:choose>
                        <xsl:when test="position() = 1"><xsl:value-of select="$nameParts[$domainCount]"/></xsl:when>
                        <xsl:when test="position() = $domainCount"><xsl:value-of select="$nameParts[1]"/></xsl:when>
                        <xsl:when test=". &gt; ' ' and(position() &gt; 2)"><xsl:value-of select="."/></xsl:when>
                        <xsl:otherwise/>
                    </xsl:choose>
                    </xsl:variable>
                    <xsl:choose>
                        <xsl:when test="contains('0123456789',substring($this,1,1))">
                            <xsl:variable name="previous" select="position() - 1"/>
                            <xsl:variable name="next" select="position() + 1"/>
                            <xsl:choose>
                                <xsl:when test="not(contains('0123456789',substring($nameParts[$previous],1,1)))">v<xsl:value-of select="$this"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="$this"/></xsl:otherwise>
                            </xsl:choose>
                            <xsl:if test="not(contains('0123456789',substring($nameParts[$next],1,1))) and(position() &lt; $partsCount)">.</xsl:if>
                        </xsl:when>
                        <xsl:otherwise><xsl:value-of select="$this"/><xsl:if test="position() &lt; $partsCount">.</xsl:if></xsl:otherwise>
                    </xsl:choose>                                    
                </xsl:for-each>
            </xsl:when>
            <xsl:otherwise><xsl:value-of select="$seedName"/></xsl:otherwise>
        </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="replace(replace(replace(replace($returnResult,':','.'),'/','.'),'-','.'),'_','.')"/>
    </xsl:function>
    
    <xsl:function name="w2e:maxOccurs">
        <xsl:param name="maxValue"/>
        <xsl:choose>
            <xsl:when test="string(number($maxValue))='NaN'"><xsl:value-of select="$maxValue"/></xsl:when>
            <xsl:otherwise>
                <xsl:variable name="chkValue" select="if($maxValue = 'unbounded') then 0 else xsd:integer($maxValue)"/>
                <xsl:choose>
                    <xsl:when test="$chkValue &gt; 200">unbounded</xsl:when>
                    <xsl:otherwise><xsl:value-of select="$maxValue"/></xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    <xsl:function name="w2e:annoValues">
        <xsl:param name="context"/>
        <xsl:param name="valuesRule"/>
        <xsl:param name="lookupRule"/>
       
        <xsl:variable name="values" >
            <xsl:choose>
                <xsl:when test="$context/as:Rule/as:constraint[contains(@action,'lookup(')]">
                    <xsl:variable name="lookupName" select="substring-before(substring-after($lookupRule[1],'lookup('),')')"/>
                    <xsl:variable name="valuesString"><xsl:for-each select="$lookupTables[@name = $lookupName]/look:item/@name">'<xsl:value-of select="."/>'|</xsl:for-each></xsl:variable>
                    <xsl:value-of select="$valuesString"/>
                </xsl:when>
                <xsl:otherwise><xsl:value-of select="substring-after($valuesRule[1],'restrictValues(')"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="$values"/>
    </xsl:function>
    
    <xsl:function name="w2e:annoAnno">
        <xsl:param name="context"/>
        <xsl:param name="valuesRuleAnno"/>
        <xsl:param name="lookupRule"/>
        
        <xsl:variable name="valuesAnno">
            <xsl:choose>
                <xsl:when test="$context/as:Rule/as:constraint[contains(@action,'lookup(')]">
                    <xsl:variable name="lookupName" select="substring-before(substring-after($lookupRule[1],'lookup('),')')"/>
                    <xsl:variable name="valuesString"><xsl:for-each select="$lookupTables[@name = $lookupName]/look:item">'<xsl:value-of select="."/>'|</xsl:for-each></xsl:variable>
                    <xsl:value-of select="$valuesString"/>
                </xsl:when>
                <xsl:otherwise><xsl:value-of select="$valuesRuleAnno[1]"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="$valuesAnno"/>
    </xsl:function>
    
    <xsl:function name="w2e:enumerations">
        <xsl:param name="values"/>
        <xsl:param name="annotations"/>
        
        <xsl:if test="string-length($values) &gt; 0">
            <xsl:variable name="unpackAnno">
                <xsl:for-each select="tokenize($annotations,'\|')">
                    <anno name="{normalize-space(substring-before(.,'-'))}" value="{normalize-space(substring-after(.,'-'))}"/>
                </xsl:for-each>
            </xsl:variable>
            
            <xsl:for-each select="tokenize($values,'\|')">
                <xsl:if test="substring-before(substring-after(.,''''),'''') &gt; ' '">
                    <xsd:enumeration value="{substring-before(substring-after(.,''''),'''')}">
                        <xsl:variable name="this" select="substring-before(substring-after(.,''''),'''')"/>
                        <xsl:if test="$unpackAnno/anno[@name = $this]">
                            <xs:annotation>
                                <xs:documentation><xsl:value-of select="$unpackAnno/anno[@name = $this]/@value"/></xs:documentation>
                            </xs:annotation>
                        </xsl:if>
                    </xsd:enumeration>
                </xsl:if>
            </xsl:for-each>
        </xsl:if>
    </xsl:function>

</xsl:stylesheet>